# Copyright (c) 2010 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from dip.model import clone_model, implements, Model

from dip.builder import IDistributionsManager


@implements(IDistributionsManager)
class DistributionsManager(Model):
    """ The DistributionsManager class is the default implementation of the
    :class:`~dip.builder.IDistributionsManager` interface.
    """

    def distribution(self, id):
        """ Find the distribution with a particular identifier.

        :param id:
            is the identifier.
        :return:
            the distribution or ``None`` if none was found.
        """

        for distribution in self.distributions:
            if distribution.id == id:
                break
        else:
            distribution = None

        return distribution

    def get_defaults(self, project, id):
        """ Get a set of defaults for a distribution from a project if they
        are present.

        :param project:
            is the project.
        :param id:
            is the identifier of the distribution.
        :return:
            the defaults.
        """

        for deflts in project.distributions_defaults:
            if deflts.distribution.id == id:
                defaults = clone_model(deflts)
                break
        else:
            distribution = self.distribution(id)
            if distribution is None:
                raise ValueError("unknown distribution '{0}'".format(id))

            defaults = distribution.defaults_factory()

        return defaults

    def update_defaults(self, project, defaults):
        """ Update the set of defaults for a distribution in a project.

        :param project:
            is the project.
        :param defaults:
            is the model containing the defaults.
        """

        # Remove any old defaults.
        for deflts in project.distributions_defaults:
            if deflts.distribution.id == defaults.distribution.id:
                project.distributions_defaults.remove(deflts)
                break

        project.distributions_defaults.append(defaults)
