# Copyright (c) 2010 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from dip.model import Str
from dip.shell import ITool
from dip.shell.actions import ShellAction

from dip.builder import IDipProject

from .distutils_template import DistutilsTemplate


class DistutilsAction(ShellAction):
    """ The DistutilsAction class implements a shell action that builds a
    distutils distribution for an application.
    """

    # The identifier of the action.
    id = 'dip.builder.actions.distutils'

    # Where the action should be placed.
    within = 'dip.builder.actions.build_collection'

    # The title to be used for dialogs.
    _title = Str("Create distutils distribution")

    def perform(self, tool):
        """ Reimplemented to create the application.

        :param tool:
            is the tool.
        """

        project = ITool(tool).obj
        distributions_manager = project.distributions_manager
        toolkit = self.shell.toolkit
        parent = self.shell.widget

        # Get any defaults from the project.
        defaults = distributions_manager.get_defaults(project,
                'dip.builder.distributions.distutils')

        # Get the additional data from the user.
        template = DistutilsTemplate(defaults=defaults, parent=parent,
                project=project, title=self._title, toolkit=toolkit)

        if not template.populate():
            return

        if template.save_as_defaults:
            # Save the defaults to the project.
            distributions_manager.update_defaults(project, defaults)

        try:
            created = template.create_distribution()
        except:
            import traceback

            toolkit.warning(self._title,
                    "There was an error creating the distribution.", parent,
                    traceback.format_exc())
            return

        if created:
            toolkit.information(self._title,
                    "The distribution <b>{0}</b> was created successfuly."
                            .format(template.defaults.name),
                    parent)

    def validate(self, tool):
        """ Reimplemented to see if the action can be performed on the tool's
        object.  We don't care about the tool itself.

        :param tool:
            is the tool.
        :return:
            ``True`` if the action can be performed.
        """

        return isinstance(ITool(tool).obj, IDipProject)

    @ShellAction.text.default
    def text(self):
        """ Create the default action text. """

        return self._title + '...'
