# Copyright (c) 2010 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


import os
import stat
import sys

from dip.model import implements, Model

from dip.builder import IDipProject


@implements(IDipProject)
class DipProject(Model):
    """ The DipProject class is the default implementation of a dip project.
    """

    _mode_flags = stat.S_IRWXU|stat.S_IRGRP|stat.S_IXGRP|stat.S_IROTH|stat.S_IXOTH

    def create_application(self, template):
        """ Create an application based on a template.

        :param template:
            is the template which will be an implementation of the
            :class:`~dip.builder.IApplicationTemplate` interface.
        """

        fout = open(template.script_name, 'w')

        fout.write(_python_executable.format(executable=sys.executable))

        header = self.file_header_text.strip()
        if header != '':
            for line in header.split('\n'):
                fout.write("# " + line + "\n")

            fout.write("\n\n")

        fout.write(_application_instance.format(
                application_name=template.script_description))

        fout.write(_plugin_manager.format(
                class_name=template.plugin_manager.class_name,
                module_name=template.plugin_manager.module_name))

        if len(template.additional_plugins) > 0:
            fout.write("\n# Create the plugins.")

            for additional_plugin in template.additional_plugins:
                fout.write(_plugin.format(
                        class_name=additional_plugin.class_name,
                        module_name=additional_plugin.module_name))

        if template.whats_this_action:
            fout.write(_whats_this)

        fout.write(_start)

        fout.close()

        # Make it executable.
        os.chmod(template.script_name, self._mode_flags)


_python_executable = """#!{executable}

"""


_application_instance = """import sys

from PyQt4.QtGui import QApplication


# Create the application instance.
app = QApplication(sys.argv, applicationName="{application_name}")
"""


_plugin_manager = """
# Create the plugin manager and core plugins.
from {module_name} import {class_name}
plugin_manager = {class_name}()
"""


_plugin = """
from {module_name} import {class_name}
plugin_manager.contribute('dip.plugins.plugins',
        {class_name}(plugin_manager=plugin_manager))
"""


_whats_this = """
# Enable the What's This help.
from dip.shell.actions import WhatsThisAction
plugin_manager.contribute('dip.shell.actions', WhatsThisAction())
"""


_start = """
# Enable the plugins.
plugin_manager.enable_plugins()

# Get the shell.
from dip.shell import IShell
shell = plugin_manager.service(IShell)

# Display the shell and start the application.
shell.widget.show()
sys.exit(app.exec_())
"""
