# Copyright (c) 2010 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


import functools

from PyQt4.QtGui import QTreeWidget, QTreeWidgetItem

from dip.model import List, Model, observe
from dip.plugins import IExtensionPoint


class ExtensionsViewer(QTreeWidget, Model):
    """ The ExtensionsViewer class is a widget that implements a read-only
    visualisation of all extension points.
    """

    # The extension points being viewed.
    extension_points = List(IExtensionPoint)

    def __init__(self):
        """ Initialise the viewer. """

        super(ExtensionsViewer, self).__init__()

        self.setHeaderLabels(["Identifier/Name", "Description"])

    @extension_points.observer
    def extension_points(self, change):
        """ Invoked when a new extension point is added. """

        # At the moment extension points can only be added.
        for extension_point in change.new:
            itm = QTreeWidgetItem(self, [extension_point.id, ''])

            # Add any existing contributions.
            for contribution in extension_point.contributions:
                self._add_contribution(itm, contribution)

            observe('contributions', extension_point,
                    functools.partial(self._new_contribution, itm))

    def _new_contribution(self, parent, change):
        """ Invoked when a new contribution is made to an extension point. """

        # At the moment contributions can only be added.
        for contribution in change.new:
            self._add_contribution(parent, contribution)

    @staticmethod
    def _add_contribution(parent, contribution):
        """ Add a new contribution. """

        # Use an identifier or name if there seems to be one.
        tool_tip = str(contribution)
        id = getattr(contribution, 'id', None)
        if id is None:
            id = getattr(contribution, 'name', None)
            if id is None:
                id = tool_tip
                tool_tip = ''

        # Use a description if there seems to be one.
        description = getattr(contribution, 'description', '')

        itm = QTreeWidgetItem(parent, [id, description])

        itm.setToolTip(0, tool_tip)
