# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


import base64
import pickle

from PyQt4.QtCore import QXmlStreamReader

from dip.io import FormatError, IDecoder
from dip.model import get_model_types, implements, Model


@implements(IDecoder)
class XmlDecoder(Model):
    """ The XmlDecoder class implements an object decoder that decodes a
    :class:`~dip.model.Model` instance from XML.
    """

    def decode(self, obj, source, location):
        """ An object is decoded from an XML byte stream.

        :param obj:
            is the object to populate from the decoded byte stream.
        :param source:
            is an iterator that will return the byte stream to be decoded.
        :param location:
            is the storage location where the encoded object is being read
            from.  It is mainly used for error reporting.
        :return:
            the decoded object.
        """

        reader = QXmlStreamReader()

        if not self._next_element(reader, source, location):
            raise FormatError("The XML file is empty", location)

        self.decode_document_element(obj, reader, location)

        self.decode_model(obj, reader, source, location)

        return obj

    def decode_document_element(self, obj, reader, location):
        """ Decode the document element (i.e. the outermost element).

        :param obj:
            is the object to populate from the decoded byte stream.
        :param reader:
            is the :class:`~PyQt4.QtCore.QXmlStreamReader` instance.
        :param location:
            is the storage location where the encoded object is being read
            from.  It is mainly used for error reporting.
        """

        if reader.name() != 'DipModel':
            raise FormatError(
                    "'DipModel' element expected, not '{0}'".format(
                            reader.name()),
                    location)

        attributes = reader.attributes()

        version = attributes.value('version')
        if version != '1':
            raise FormatError(
                    "Version '1' expected, not '{0}'".format(version),
                    location)

        format = attributes.value('format')
        if format != self.factory.id:
            raise FormatError(
                    "Format '{0}' expected, not '{1}'".format(self.factory.id,
                            format),
                    location)

    def decode_model(self, obj, reader, source, location):
        """ Decode the next element as a model.

        :param obj:
            is the model to populate from the decoded byte stream.
        :param reader:
            is the :class:`~PyQt4.QtCore.QXmlStreamReader` instance.
        :param source:
            is an iterator that will return the byte stream to be decoded.
        :param location:
            is the storage location where the encoded object is being read
            from.  It is mainly used for error reporting.
        """

        while self._next_element(reader, source, location):
            self.decode_attribute(obj, reader, source, location)

    def decode_attribute(self, obj, reader, source, location):
        """ Decode the current element as an attribute.

        :param obj:
            is the object to populate from the decoded byte stream.
        :param reader:
            is the :class:`~PyQt4.QtCore.QXmlStreamReader` instance.
        :param source:
            is an iterator that will return the byte stream to be decoded.
        :param location:
            is the storage location where the encoded object is being read
            from.  It is mainly used for error reporting.
        """

        name = str(reader.name())

        # Make sure there was a name.
        attributes = reader.attributes()
        aname = str(attributes.value('name'))

        if aname == '':
            raise FormatError(
                    "Element '{0}' has no 'name' attribute".format(name),
                    location)

        # Make sure the model has an attribute with that name.
        if aname not in get_model_types(type(obj)):
            raise FormatError(
                    "'{0}' type has no '{1}' attribute".format(
                            type(obj).__name__, aname),
                    location)

        # Convert the value.
        value = self._decode_element(name, reader, source, location)

        # Set the value.
        setattr(obj, aname, value)

    def _decode_element(self, name, reader, source, location):
        """ Decode the current element and return the value. """

        try:
            handler = getattr(self, '_decode_' + name)
        except AttributeError:
            raise FormatError("Unknown element '{0}'".format(name), location)

        return handler(reader, source, location)

    @staticmethod
    def _decode_bool(reader, source, location):
        """ Decode to a bool. """

        value = reader.readElementText()

        try:
            value = bool(value)
        except ValueError:
            raise FormatError(
                    "Unable to convert value from '{0}' to a bool".format(
                            value),
                    location)

        return value

    def _decode_Dict(self, reader, source, location):
        """ Decode to a dict. """

        value = {}

        while self._next_element(reader, source, location):
            item_key = self._decode_element(str(reader.name()), reader, source,
                    location)

            if not self._next_element(reader, source, location):
                raise FormatError("Dict has key but no corresponding value",
                        location)

            item_value = self._decode_element(str(reader.name()), reader,
                    source, location)

            value[item_key] = item_value

        return value

    @staticmethod
    def _decode_Enum(reader, source, location):
        """ Decode to an Enum. """

        value = reader.readElementText()

        return value

    @staticmethod
    def _decode_float(reader, source, location):
        """ Decode to a float. """

        value = reader.readElementText()

        try:
            value = float(value)
        except ValueError:
            raise FormatError(
                    "Unable to convert value from '{0}' to a float".format(
                            value),
                    location)

        return value

    @staticmethod
    def _decode_int(reader, source, location):
        """ Decode to an int. """

        value = reader.readElementText()

        try:
            value = int(value)
        except ValueError:
            raise FormatError(
                    "Unable to convert value from '{0}' to an int".format(
                            value),
                    location)

        return value

    def _decode_List(self, reader, source, location):
        """ Decode to a list. """

        value = []

        while self._next_element(reader, source, location):
            value.append(
                    self._decode_element(str(reader.name()), reader, source,
                            location))

        return value

    @staticmethod
    def _decode_pickle(reader, source, location):
        """ Decode from a pickle. """

        value = unicode(reader.readElementText()).encode('utf8')

        try:
            value = base64.b64decode(value)
        except TypeError as e:
            raise FormatError("Unable to base64 decode value: {0}".format(e),
                    location)

        try:
            value = pickle.loads(value)
        except pickle.UnpicklingError as e:
            raise FormatError("Unable to unpickle value: {0}".format(e),
                    location)

        return value

    _decode_Instance = _decode_pickle

    def _decode_Set(self, reader, source, location):
        """ Decode to a set. """

        value = set()

        while self._next_element(reader, source, location):
            value.add(
                    self._decode_element(str(reader.name()), reader, source,
                            location))

        return value

    @staticmethod
    def _decode_str(reader, source, location):
        """ Decode to a str. """

        value = reader.readElementText()

        return value

    @staticmethod
    def _next_element(reader, source, location):
        """ Get the next start element. """

        while True:
            if reader.readNextStartElement():
                break

            error = reader.error()

            if error == QXmlStreamReader.NoError:
                # All the data has been processed.
                return False

            if error != QXmlStreamReader.PrematureEndOfDocumentError:
                raise FormatError(
                        "XML reader error:{0}: {1}".format(reader.lineNumber(),
                                reader.errorString()),
                        location)

            # Get the next set of data.
            reader.addData(next(source))

        return True
