# Copyright (c) 2010 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from dip.model import Instance, Interface, List

from .i_codecs_factory import ICodecsFactory
from .i_storage_factory import IStorageFactory
from .i_storage_policy import IStoragePolicy


class IIOManager(Interface):
    """ The IIOManager interface defines the API of an i/o manager. """

    # The available codecs factories.
    codecs_factories = List(ICodecsFactory)

    # The policy that determines if an object is allowed to be stored in
    # persistent storage represented by a particular implementation of
    # :class:`~dip.io.IStorage`.
    policy = Instance(IStoragePolicy)

    # The available storage factories.
    storage_factories = List(IStorageFactory)

    def readable_storage(self, obj, exclude_format=None, include_format=None, storage_factories=None):
        """ Get the list of the storage instances that an object can be read
        from.

        :param obj:
            is the object.
        :param exclude_format:
            is the optional identifier of a format that will be used to
            exclude factories that can handle it.
        :param include_format:
            is the optional identifier of a format that will be used to
            include factories that can handle it.
        :param storage_factories:
            is the optional list of storage factories that are considered.  If
            it is ``None`` then all storage factories are considered.
        :return:
            the list of storage instances.
        """

    def writeable_storage(self, obj, exclude_format=None, include_format=None, storage_factories=None):
        """ Get the list of the storage instances that an object can be
        written to.

        :param obj:
            is the object.
        :param exclude_format:
            is the optional identifier of a format that will be used to
            exclude factories that can handle it.
        :param include_format:
            is the optional identifier of a format that will be used to
            include factories that can handle it.
        :param storage_factories:
            is the optional list of storage factories that are considered.  If
            it is ``None`` then all storage factories are considered.
        :return:
            the list of storage instances.
        """

    def storage_factories_for_location(self, location):
        """ Get the list of storage factories for which a location specified
        as a string is valid.

        :param location:
            is the location as a string.
        :return:
            the list of storage factories.
        """
