# Copyright (c) 2010 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from dip.io import (IIOManager, IStreamingStorageFactory,
        IStructuredStorageFactory)
from dip.model import implements, Model


@implements(IIOManager)
class IOManager(Model):
    """ The IOManager class is the default implementation of the
    :class:`~dip.storage.IIOManager` interface.
    """

    def readable_storage(self, obj, exclude_format=None, include_format=None, storage_factories=None):
        """ Get the list of the storage instances that an object can be read
        from.

        :param obj:
            is the object.
        :param exclude_format:
            is the optional identifier of a format that will be used to
            exclude factories that can handle it.
        :param include_format:
            is the optional identifier of a format that will be used to
            include factories that can handle it.
        :param storage_factories:
            is the optional list of storage factories that are considered.  If
            it is ``None`` then all storage factories are considered.
        :return:
            the list of storage instances.
        """

        return self._storage(obj, exclude_format, include_format,
                storage_factories, read=True)

    def writeable_storage(self, obj, exclude_format=None, include_format=None, storage_factories=None):
        """ Get the list of the storage instances that an object can be
        written to.

        :param obj:
            is the object.
        :param exclude_format:
            is the optional identifier of a format that will be used to
            exclude factories that can handle it.
        :param include_format:
            is the optional identifier of a format that will be used to
            include factories that can handle it.
        :param storage_factories:
            is the optional list of storage factories that are considered.  If
            it is ``None`` then all storage factories are considered.
        :return:
            the list of storage instances.
        """

        return self._storage(obj, exclude_format, include_format,
                storage_factories, read=False)

    def storage_factories_for_location(self, location):
        """ Get the list of storage factories for which a location specified
        as a string is valid.

        :param location:
            is the location as a string.
        :return:
            the list of storage factories.
        """

        return [factory for factory in self.storage_factories
                if factory.valid_location(location)]

    def _storage(self, obj, exclude_format, include_format, storage_factories, read):
        """ Return a list of storage instances that can handle an object. """

        storage_list = []

        if storage_factories is None:
            storage_factories = self.storage_factories

        for storage_factory in storage_factories:
            if isinstance(storage_factory, IStreamingStorageFactory):
                for codecs_factory in self.codecs_factories:
                    format_ok = self._check_format(obj, codecs_factory,
                            exclude_format, include_format, read)

                    if format_ok and self._check_policy(obj, storage_factory):
                        storage = storage_factory()
                        storage.codecs_factory = codecs_factory
                        storage_list.append(storage)

            elif isinstance(storage_factory, IStructuredStorageFactory):
                format_ok = self._check_format(obj, storage_factory,
                        exclude_format, include_format, read)

                if format_ok and self._check_policy(obj, storage_factory):
                    storage_list.append(storage_factory())

        return storage_list

    @staticmethod
    def _check_format(obj, format, exclude_format, include_format, read):
        """ Check if a format is valid in a given context. """

        # Check the format identifier.
        if exclude_format is not None and format.id == exclude_format:
            return False

        if include_format is not None and format.id != include_format:
            return False

        # Make sure the format really can handle the object.
        return format.decodes(obj) if read else format.encodes(obj)

    def _check_policy(self, obj, storage_factory):
        """ Apply the storage policy for an object. """

        return self.policy is None or self.policy(obj, storage_factory)
