# Copyright (c) 2010 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from dip.io import IStorageUi
from dip.model import implements, Instance, Model
from dip.ui import FileSelector, Form, ViewFactory


@implements(IStorageUi)
class FilesystemStorageUi(Model):
    """ The FilesystemStorageUi class implements the user interface for
    selecting filesystem locations.
    """

    # The view for the read browser.
    read_browser_view = Instance(ViewFactory)

    # The view for the write browser.
    write_browser_view = Instance(ViewFactory)

    def read_browser(self, toolkit, location):
        """ Create a widget that allows the user to browse the filesystem and
        select a file to read.

        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation that can be used to
            create the browser.
        :param location:
            is the location updated by the user.
        :return:
            the :class:`~PyQt4.QtGui.QWidget` that implements the browser.
        """

        return self.read_browser_view(location, toolkit, widget=True)

    def get_read_location(self, toolkit, location):
        """ Get an updated location from the user where an object should be
        read from.

        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation that can be used to
            create any user interface.
        :param location:
            is the location that is updated by the user.  If the user
            cancelled then its :attr:`~dip.io.IStorageLocation.location`
            attribute is set to an empty string.
        """

        location.location = toolkit.get_open_file(
                directory=location.location,
                filter=self.storage.codecs_factory.filter)

    def write_browser(self, toolkit, location):
        """ Create a widget that allows the user to browse the filesystem and
        select a file to write.

        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation that can be used to
            create the browser.
        :param location:
            is the location updated by the user.
        :return:
            the :class:`~PyQt4.QtGui.QWidget` that implements the browser.
        """

        return self.write_browser_view(location, toolkit, widget=True)

    def get_write_location(self, toolkit, location):
        """ Get an updated location from the user where an object should be
        written to.

        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation that can be used to
            create any user interface.
        :param location:
            is the location that is updated by the user.  If the user
            cancelled then its :attr:`~dip.io.IStorageLocation.location`
            attribute is set to an empty string.
        """

        location.location = toolkit.get_save_file(
                directory=location.location,
                filter=self.storage.codecs_factory.filter)

    @read_browser_view.default
    def read_browser_view(self):
        """ Create the view for the read browser. """

        # This can't be a class attribute because we need to bind the filter.
        return Form(
                FileSelector('location', label="File name", mode='open_file',
                        filter=self.storage.codecs_factory.filter))

    @write_browser_view.default
    def write_browser_view(self):
        """ Create the view for the write browser. """

        # This can't be a class attribute because we need to bind the filter.
        return Form(
                FileSelector('location', label="File name", mode='save_file',
                        filter=self.storage.codecs_factory.filter))
