# Copyright (c) 2010 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtCore import QIODevice

from dip.io import IDecoder, IEncoder, StorageError
from dip.model import Int, Model


class QIODeviceStorage(Model):
    """ The QIODeviceStorage class is an abstract base class for storage that
    is accessed via a :class:`~PyQt4.QtCore.QIODevice`.
    """

    # The number of bytes to read at a time.
    read_buffer_size = Int(16384)

    def implicit_location(self, obj):
        """ Return the implicit storage location for an object.

        :param obj:
            is the object to create the location for.
        :return:
            ``None`` as devices typically have explicit locations.
        """

        return None

    def read(self, obj, location):
        """ Read an object from a storage location.

        :param obj:
            is the object to populate from the storage location.
        :param location:
            is the storage location where the object is read from.
        :return:
            the read object.  This may be the original object populated
            from the storage location, or it may be a different object (of an
            appropriate type) created from the storage location.
        """

        device = self.qiodevice(location)

        if not device.open(QIODevice.ReadOnly):
            raise StorageError(device.errorString(), location)

        def reader():
            while True:
                bytes = device.read(self.read_buffer_size)

                if bytes is None:
                    raise StorageError(device.errorString(), location)

                if len(bytes) == 0:
                    break

                yield bytes

        decoder = IDecoder(self.codecs_factory.decoder())
        obj = decoder.decode(obj, reader(), location)

        device.close()

        return obj

    def write(self, obj, location):
        """ Write an object to a filesystem location.

        :param obj:
            is the object to write.
        :param location:
            is the storage location where the object is written to.
        """

        device = self.qiodevice(location)

        if not device.open(QIODevice.WriteOnly):
            raise StorageError(device.errorString(), location)

        encoder = IEncoder(self.codecs_factory.encoder())

        for bytes in encoder.encode(obj, location):
            if device.write(bytes) < 0:
                raise StorageError(device.errorString(), location)

        device.close()

    def qiodevice(self, location):
        """ A storage location is converted to a
        :class:`~PyQt4.QtCore.QIODevice` instance.  This must be reimplemented
        by a sub-class.

        :param location:
            is the storage location.
        :return:
            the :class:`~PyQt4.QtCore.QIODevice`.
        """
