# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


# The adapter targets keyed by the target interface.  A value is a list of
# 2-tuple of adapted type or interface and factory (i.e. Adapter sub-class).
_adapter_targets = {}


def add_adapter_factory(factory, adapted, interface):
    """ An internal function that registers an adapter factory (i.e. a
    sub-class of Adapter) for an adapted type and an interface.
    """

    new_af_pair = (adapted, factory)
    new_af_pair_list = [new_af_pair]

    af_pair_list = _adapter_targets.setdefault(interface, new_af_pair_list)

    if af_pair_list is not new_af_pair_list:
        # There was already an entry for the interface so update it.  First
        # check that there isn't an existing factory.
        for existing_adapted, _ in af_pair_list:
            if adapted is existing_adapted:
                raise TypeError("an adapter has already been defined between "
                        "'{0}' and '{1}'".format(
                                adapted.__name__, interface.__name__))

        af_pair_list.append(new_af_pair)


def find_adapter(adaptee, interface, exception=False):
    """ An internal function to find an existing adapter that adapts an object
    to an interface.
    """

    # We keep the adapter cache in the object itself so that the adapter and
    # adaptee reference each other.  They will get garbage collected if nothing
    # else is referencing either of them.
    adapter_cache = getattr(adaptee, '_dip_adapters', [])

    # Use an existing adapter if possble.
    for adapter, iface in adapter_cache:
        # An existing adapter for an exact or more specific interface is fine.
        if issubclass(iface, interface):
            return adapter, adapter_cache

        # An existing adapter for a less specific interface is an error because
        # creating an adapter for the required interface would result in
        # duplication of any state held in the existing adapter.
        if issubclass(interface, iface):
            if exception:
                raise TypeError("{0} cannot be adapted to '{1}' because it "
                        "has already been adapted to '{2}'".format(adaptee,
                                interface.__name__, iface.__name__))

            break

    return None, adapter_cache


def object_adapter(adaptee, interface, exception):
    """ An internal function to return an adapter that adapts an object to an
    interface, creating it if necessary.
    """

    # Find any existing adapter.
    adapter, adapter_cache = find_adapter(adaptee, interface, exception)
    if adapter is not None:
        return adapter

    # Find candidate adapter factories giving preference to exact matches.
    candidates = []
    for iface, af_pair_list in _adapter_targets.items():
        if iface is interface:
            candidates.insert(0, (iface, af_pair_list))
        elif issubclass(iface, interface):
            candidates.append((iface, af_pair_list))

    adaptee_type = type(adaptee)
    adaptee_interfaces = getattr(adaptee_type, '_dip_interfaces', ())

    for iface, af_pair_list in candidates:
        # Try the adaptee's type.
        factory = pick_factory(adaptee_type, af_pair_list)
        if factory is None:
            # Try any interfaces that the adaptee implements.
            for adaptee_interface in adaptee_interfaces:
                factory = pick_factory(adaptee_interface, af_pair_list)
                if factory is not None:
                    break

        if factory is not None:
            adapter = factory(adaptee=adaptee)
            adapter_cache.append((adapter, iface))
            break

    if adapter is None:
        if exception:
            raise TypeError("{0} does not implement '{1}' and could not be "
                    "adapted".format(adaptee, interface.__name__))
    elif len(adapter_cache) == 1:
        # This must be the first adapter so save the cache.
        setattr(adaptee, '_dip_adapters', adapter_cache)

    return adapter


def pick_factory(from_type, af_pair_list):
    """ Pick a factory to adapt a type from a list of type/factory pairs. """

    # Find candidate factories giving preference to exact matches.  If there is
    # no exact match then pick the first one found (which may not be what the
    # user wants).
    candidates = []
    for adapted, factory in af_pair_list:
        if from_type is adapted:
            candidates.insert(0, factory)
            break

        if issubclass(from_type, adapted):
            candidates.append(factory)

    try:
        factory = candidates[0]
    except IndexError:
        factory = None

    return factory
