# Copyright (c) 2010 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from .exceptions import ValidationError, ValidationTypeError
from .value_type_factory import ValueTypeFactory


class Enum(ValueTypeFactory):
    """ The Enum class encapsulates a fixed set of string values. """

    def __init__(self, *members, **metadata):
        """ Initialise the object.

        :param \*members:
            the list of members.
        :param default:
            the default attribute value.  If omitted then the first member is
            used.
        :param \*\*metadata:
            is additional meta-data stored with the type.
        """

        try:
            default = metadata.pop('default')
        except KeyError:
            default = None

        if len(members) == 0:
            raise ValidationError(Enum, message="no members defined")

        for member in members:
            if not isinstance(member, str):
                raise ValidationTypeError(Enum, str, member)

        self.members = members

        if default is None:
            default = members[0]

        super(Enum, self).__init__(default, metadata)

    def copy(self, default):
        """ Create a copy of this instance with a different default value.

        :param default:
            is the default value of the copied instance.
        :return:
            the copied instance.
        """

        return self.__class__(*self.members, default=default, **self.metadata)

    def validate(self, value):
        """ Validate an object according to the constraints.  An exception is
        raised if the object is invalid.

        :param value:
            the object to validate.
        :return:
            the validated object.
        """

        if value not in self.members:
            member_list = ', '.join(
                    ['\'' + member + '\'' for member in self.members])

            raise ValidationError(Enum,
                    message="expected one of {0}, not '{1}'".format(
                            member_list, value))

        return value
