# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from .collection_type_factory import CollectionTypeFactory
from .exceptions import ValidationError
from .interface import Interface


class Instance(CollectionTypeFactory):
    """ The Instance class encapsulates an instance of a number of types. """

    def __init__(self, *types, **metadata):
        """ Initialise the object.

        :param \*types:
            the allowable types of the instance.  If no types are specified
            then an instance of any type is allowed.  If any type is a string
            then it is the "full" name of the type.  The string form allows
            types to be specified while avoiding circular imports.
        :param default:
            the default attribute value.  If omitted then ``None`` is used.
        :param \*\*metadata:
            is additional meta-data stored with the type.
        """

        try:
            default = metadata.pop('default')
        except KeyError:
            default = None

        self.types = types
        self._resolved = None

        super(Instance, self).__init__(default, metadata)

    def copy(self, default):
        """ Create a copy of this instance with a different default value.

        :param default:
            is the default value of the copied instance.
        :return:
            the copied instance.
        """

        return self.__class__(*self.types, default=default, **self.metadata)

    def validate(self, value):
        """ Validate an instance according to the constraints.  An exception
        is raised if the instance is invalid.

        :param value:
            the instance to validate.
        :return:
            the validated instance.
        """

        # None is a special case.
        if value is None:
            return value

        if self._resolved is None:
            self.types = self._resolved = [self.resolve_type(t, "instance")
                    for t in self.types]

        last_exception = None

        for instance_type in self._resolved:
            # Adapt the value if it is supposed to implement an interface.
            if Interface in instance_type.__mro__:
                validated = instance_type(value, exception=False)
                if validated is not None:
                    last_exception = None
                    break

            try:
                validated = self.validate_collection_value(instance_type, value)
            except ValidationError as e:
                last_exception = e
            else:
                last_exception = None
                break
        else:
            validated = value

        if last_exception is None:
            return validated

        # If there was only one possible type then re-raise the exception.
        if len(self.types) == 1:
            raise last_exception

        # Raise a more generic exception if there were a number of possible
        # types.
        raise ValidationError(*self.types,
                message="validation failed for all types")

    @classmethod
    def different(cls, value1, value2):
        """ Reimplemented to compare two instances to see if they are
        different.

        :param value1:
            is the first value.
        :param value2:
            is the second value.
        :return:
            ``True`` if the values are different.
        """

        return value1 is not value2
