# Copyright (c) 2009 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from .bool import Bool
from .dict import Dict
from .float import Float
from .int import Int
from .list import List
from .model import Model
from .set import Set
from .str import Str


class MappingProxy(object):
    """ The MappingProxy class implements a :class:`~dip.model.Model` based
    proxy for a mapping type.  It is assumed that the mapping has string keys.
    Note that changes made to the object being proxied cannot be observed.
    """

    def __new__(cls, mapping):
        """ Create the proxy.  We make modifications to the class dictionary
        that are specific to the mapping, therefore we need to create a new
        type each time.
        """

        cls_dict = {
            '__init__': __init__,
            '__getattr__': __getattr__,
            '__setattr__': __setattr__
        }

        metatype = type(Model)
        proxy_cls = metatype('MappingProxy', (Model, ), cls_dict)
        proxy = proxy_cls(mapping)

        return proxy


def __init__(self, mapping):
    """ Initialise the proxy. """

    self.__dict__['_dip_mapping'] = mapping

    # Create the types cache with a type for each entry in the mapping.
    atypes = {}

    for name, value in mapping.items():
        py_type = type(value)

        for builtin_type, type_type in _builtin_type_map:
            if issubclass(py_type, builtin_type):
                break
        else:
            raise TypeError(
                    "objects of type '{0}' are not supported in views".format(
                            py_type.__name__))

        atype = type_type()
        atype.name = name

        atypes[name] = atype

    setattr(self.__class__, '_dip_type_cache', atypes)


def __getattr__(self, name):
    """ Get an attribute from the mapping. """

    return self.__dict__['_dip_mapping'][name]


def __setattr__(self, name, value):
    """ Set an attribute of the mapping. """

    # setattr() is called by the meta-type before __init__() so we might not
    # know about the mapping yet.  It will be setting the initial value which
    # should be ignored anyway.
    try:
        mapping = self.__dict__['_dip_mapping']
    except KeyError:
        pass
    else:
        mapping[name] = value


# The map of the types used to handle each supported builtin type.  Note that
# the order is important so we don't use a dict.
_builtin_type_map = (
    (bool,  Bool),
    (float, Float),
    (int,   Int),
    (str,   Str),
    (dict,  Dict),
    (list,  List),
    (set,   Set)
)
