# Copyright (c) 2009 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from .adapter_internals import object_adapter
from .meta_model import MetaModel


class MetaInterface(MetaModel):
    """ The MetaInterface class is the meta-class for Interface and is
    responsible for making an interface appear to be a super-class and invoking
    adapters when necessary.
    """

    def __call__(cls, obj, exception=True):
        """ This is called to create an instance of an object that implements
        an interface we are the meta-class of.

        :param obj:
            is the object that may need adapting.
        :param exception:
            is True if an exception should be raised if the object couldn't be
            adapted.  If False then None is returned if the object couldn't be
            adapted.
        :return:
            an object, possibly an adapter, that implements the interface.
        """

        # Handle the trvial case where an adapter isn't required.
        if isinstance(obj, cls):
            return obj

        return object_adapter(obj, cls, exception)

    def __instancecheck__(cls, instance):
        """ Reimplemented to ensure that isinstance() treats an instance that
        implements an interface as being an instance of that interface.
        """

        subclass = instance.__class__

        for interface in getattr(subclass, '_dip_interfaces', ()):
            if issubclass(interface, cls):
                return True

        # Fall back to the default implementation.
        return super(MetaInterface, cls).__instancecheck__(instance)

    def __subclasscheck__(cls, subclass):
        """ Reimplemented to ensure that issubclass() treats a class that
        implements an interface as being a sub-class of that interface.
        """

        for interface in getattr(subclass, '_dip_interfaces', ()):
            if issubclass(interface, cls):
                return True

        # Fall back to the default implementation.
        return super(MetaInterface, cls).__subclasscheck__(subclass)
