# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


import functools
import types

from PyQt4.QtCore import pyqtWrapperType, Qt

from .delegated_to import DelegatedTo
from .observe import observe
from .trigger import Trigger
from .type_factory import TypeFactory
from .value_type_factory import Uninitialized, ValueTypeFactory


class MetaModel(pyqtWrapperType):
    """ The MetaModel class is the meta-class for Model and is responsible for
    initialising the attributes of an instance.
    """

    def __new__(meta_cls, name, bases, cls_dict):
        " Reimplemented to create the type cache. """

        # Create the class as normal.
        cls = pyqtWrapperType.__new__(meta_cls, name, bases, cls_dict)

        type_cache = {}

        # Go through the MRO in reverse order so that local attributes
        # replace more distant ones automatically.
        for mro in reversed(cls.__mro__):
            for aname, atype in mro.__dict__.items():
                if isinstance(atype, TypeFactory):
                    # The descriptor needs to know the attribute name.
                    atype.name = aname

                    # Cache the completed type.
                    type_cache[aname] = atype

                elif type(atype) is types.FunctionType:
                    # See if this is a decorated observer.
                    observed = getattr(atype, '_dip_observed', None)
                    if observed is not None:
                        type_cache[aname] = observed

                else:
                    # See if this is overriding the default value of an
                    # attribute.
                    orig_atype = type_cache.get(aname)
                    if isinstance(orig_atype, ValueTypeFactory):
                        # Create a clone of the original type but with the new
                        # default.
                        new_atype = orig_atype.clone(atype)

                        if new_atype is None:
                            raise TypeError("'{0}.{1}' cannot override the "
                                    "base declaration".format(
                                            cls.__name__, aname))

                        # Replace the value with the new type and update the
                        # type cache.
                        setattr(mro, aname, new_atype)
                        type_cache[aname] = new_atype

        setattr(cls, '_dip_type_cache', type_cache)

        return cls

    def __call__(cls, *args, **kwargs):
        """ This is called to create an instance of a class we are a
        meta-class of.
        """

        # Create the instance as normal.
        instance = cls.__new__(cls)

        # Get the type cache.
        type_cache = getattr(cls, '_dip_type_cache')

        # Handle any keyword arguments that correspond to types and remove
        # them from the dict.
        init_kwargs = kwargs.copy()
        delegates = []
        observers = []
        for aname, atype in type_cache.items():
            # Setup any decorated observer.
            if isinstance(atype, list):
                observer = getattr(instance, aname)

                for oname, otype in atype:
                    observers.append((oname, instance, observer, otype))

                continue

            # Get the type's value if given, otherwise get it later when it is
            # first referenced.
            value = init_kwargs.pop(aname, Uninitialized)

            observer_func = atype._observer_func

            if isinstance(atype, DelegatedTo):
                if value is not Uninitialized:
                    # Remember the initialisation of any delegates.
                    delegates.append((aname, value))
            elif isinstance(atype, ValueTypeFactory):
                shadow = '_' + aname

                if value is Uninitialized:
                    # Set the default if there is no default function.
                    if atype._default_func is None:
                        value = atype.get_default_value()
                        atype.bind(instance, value)

                    setattr(instance, shadow, value)
                else:
                    # We have an explicit initial value so make sure that any
                    # implict observer is in place.
                    if observer_func is not None:
                        observe(aname, instance,
                                functools.partial(observer_func, instance))
                        observer_func = None

                    # Make sure the shadow attribute contains the default value
                    # so that there is something to compare any explicit
                    # initial value with when it is set.
                    setattr(instance, shadow, atype.get_default_value())

                    # Save the explicit initial value.
                    setattr(instance, aname, value)

            if observer_func is not None:
                # Setup any implicit observer.  We need to set delegates up
                # after other attributes which they may depend on are
                # initialised.  It's easier to handle them all in the same way.
                observers.append((aname, instance,
                        functools.partial(observer_func, instance),
                        Qt.AutoConnection))

        # We can now initialise any delegates now that the other (possibly
        # dependent) attributes have been set up.
        for aname, value in delegates:
            setattr(instance, aname, value)

        # Now set up the observers.
        for aname, instance, observer, otype in observers:
            observe(aname, instance, observer, connection_type=otype)

        # Initialise the instance with the modified keyword arguments.
        instance.__class__.__init__(instance, *args, **init_kwargs)

        return instance
