# Copyright (c) 2010 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from .type_factory import TypeFactory


def clone_model(model):
    """ Create a clone of a model.  Note that individual attributes are not
    recursively cloned.

    :param model:
        is the :class:`~dip.model.Model` instance.
    :return:
        the clone.
    """

    model_type = type(model)
    clone = model_type()

    for aname in get_model_types(model_type).keys():
        setattr(clone, aname, getattr(model, aname))

    return clone


def get_attribute_type(model, name):
    """ Get the :class:`~dip.model.TypeFactory` sub-class instance for an
    attribute of a model.

    :param model:
        is the :class:`~dip.model.Model` instance.
    :param name:
        is the name of the attribute.
    :return:
        the attribute's type object.
    """

    # Get the inner model and attribute name.
    path = name.split('.')
    name = path[-1]

    for part in path[:-1]:
        model = getattr(model, part)

    atypes = get_model_types(type(model))

    try:
        atype = atypes[name]
    except KeyError:
        raise AttributeError(
                "model {0} has no attribute '{1}'".format(model, name))

    return atype


def get_model_types(model_type):
    """ Get a copy of the dictionary of :class:`~dip.model.TypeFactory`
    sub-class instances, keyed by attribute name, for a model type.

    :param model_type:
        is the :class:`~dip.model.Model` sub-class.
    :return:
        the dictionary of type objects.
    """

    try:
        types = getattr(model_type, '_dip_type_cache')
    except:
        raise TypeError(
                "sub-class of dip.model.Model expected, not '{0}'".format(
                        type(model_type).__name__))

    # Filter out any internal non-TypeFactory stuff.
    model_types = {}
    for name, type in types.items():
        if isinstance(type, TypeFactory):
            model_types[name] = type

    return model_types
