# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from .collection_type_factory import CollectionTypeFactory
from .exceptions import ValidationTypeError


class Subclass(CollectionTypeFactory):
    """ The Subclass class encapsulates any type object. """

    def __init__(self, type, default=None, **metadata):
        """ Initialise the object.

        :param type:
            is the type object.
        :param default:
            the default attribute value.  If omitted then ``None`` is used.
        :param \*\*metadata:
            is additional meta-data stored with the type.
        """

        self.type = type

        super(Subclass, self).__init__(default, metadata)

    def copy(self, default):
        """ Create a copy of this instance with a different default value.

        :param default:
            is the default value of the copied instance.
        :return:
            the copied instance.
        """

        return self.__class__(self.type, default=default, **self.metadata)

    def validate(self, value):
        """ Validate an object according to the constraints.  An exception is
        raised if the object is invalid.

        :param value:
            the object to validate.
        :return:
            the validated object.
        """

        if value is not None:
            self.type = self.resolve_type(self.type, "subclass")

            try:
                valid = issubclass(value, self.type)
            except TypeError:
                valid = False

            if not valid:
                raise ValidationTypeError(type(self), self.type, value)

        return value

    @classmethod
    def different(cls, value1, value2):
        """ Reimplemented to compare two sub-classes to see if they are
        different.

        :param value1:
            is the first value.
        :param value2:
            is the second value.
        :return:
            ``True`` if the values are different.
        """

        return value1 is not value2
