# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from dip.model import get_model_types, implements, Model

from .i_plugin import IPlugin
from .contribution_to import ContributionTo


@implements(IPlugin)
class Plugin(Model):
    """ The Plugin class is a convenience base class for a :term:`plugin`. """

    def bind_extension_point(self, id, obj, name):
        """ Bind an extension point to the attribute of an object.  The
        attribute must have an ``append()`` method.

        :param id:
            is the identifier of the extension point.
        :param obj:
            is the object containing the attribute that the extension point is
            bound to.
        :param name:
            is the name of the attribute that the extension point is bound to.
        """

        self.plugin_manager.bind_extension_point(id, obj, name)

    def enable(self):
        """ The plugin is enabled.  It should not be called if it is already
        enabled.
        """

        # Register any contributions.
        for id, contributions in self.contributions():
            self.plugin_manager.contribute(id, *contributions)

        self.enabled = True

    def service(self, interface):
        """ Get a service that implements a particular interface.

        :param interface:
            is the interface that the service must implement.
        :return:
            the service.  An exception is raised if there is no service
            available.
        """

        return self.plugin_manager.service(interface)

    def contributions(self):
        """ This is called when the plugin is enabled to return an iterator of
        2-tuples of the extension point identifier and the list of
        contributions to the extension point.  This default implementation
        looks for any :class:`~dip.plugins.ContributionTo` attributes in the
        plugin and its super-classes.

        :return:
            the iterator.
        """

        for aname, atype in get_model_types(type(self)).items():
            if isinstance(atype, ContributionTo):
                # Get the contributions.
                contributions = getattr(self, aname)

                yield atype.id, contributions
