# Copyright (c) 2010 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from dip.model import implements, List, Model

from .i_extension_point import IExtensionPoint
from .i_plugin import IPlugin
from .i_plugin_manager import IPluginManager
from .i_service import IService
from .extension_point import ExtensionPoint


@implements(IPluginManager)
class PluginManager(Model):
    """ The PluginManager class is the default implementation of the
    :class:`~dip.plugins.IPluginManager` interface.
    """

    # The list of extension points.
    _extension_points = List(IExtensionPoint)

    # The list of contributed plugins.
    _plugins = List(IPlugin)

    # The list of contributed services.
    _services = List(IService)

    def __init__(self):
        """ Initialise the plugin manager. """

        # Bootstrap the root extension point.
        self.bind_extension_point('dip.plugins.extension_points', self,
                '_extension_points')
        root = self._extension_points[0]
        root.contributions.append(root)

        # Bind the remaining extension points.
        self.bind_extension_point('dip.plugins.plugins', self, '_plugins')
        self.bind_extension_point('dip.plugins.services', self, '_services')

    def bind_extension_point(self, id, obj, name):
        """ Bind an extension point to the attribute of an object.  The
        attribute must have an ``append()`` method.

        :param id:
            is the identifier of the extension point.
        :param obj:
            is the object containing the attribute that the extension point is
            bound to.
        :param name:
            is the name of the attribute that the extension point is bound to.
        """

        self._get_extension_point(id).bind(obj, name)

    def contribute(self, id, *contributions):
        """ Contribute some objects to an extension point.

        :param id:
            is the identifier of the extension point.
        :param \*contributions:
            are the contributions to make to the extension point.  The
            contributions are made one at a time.
        """

        self._get_extension_point(id).contribute(contributions)

    def enable_plugins(self):
        """ All plugins that are not already enabled are enabled. """

        for plugin in self._plugins:
            if not plugin.enabled:
                plugin.enable()

    def service(self, interface):
        """ Get a service that implements a particular interface.

        :param interface:
            is the interface that the service must implement.
        :return:
            the service.  An exception is raised if there is no service
            available.
        """

        # Get the list of services that implement this interface.
        services = [service.implementation for service in self._services
                if service.interface is interface]

        service = self.choose_service(interface, services)

        if service is None:
            raise TypeError(
                    "there is no service available that implements '{0}'"
                            .format(interface.__name__))

        # Adapt if necessary.
        return interface(service)

    def choose_service(self, interface, services):
        """ Choose a particular service from a list of possible services.

        :param interface:
            is the interface that the service must implement.
        :param services:
            a list of contributed services to choose from.  This can be
            ignored.
        :return:
            the chosen service or ``None`` if one wasn't chosen.
        """

        return services[0] if len(services) > 0 else None

    @_extension_points.default
    def _extension_points(self):
        """ Create the default list of extension points. """

        return [ExtensionPoint(id='dip.plugins.extension_points')]

    def _get_extension_point(self, id):
        """ Get the extension point for a given identifier, creating it if
        necessary.  Allowing extension points to be defined implicitly allows
        consumers of contributions to be enabled later and still have access
        to all contributions.
        """

        for extension_point in self._extension_points:
            if extension_point.id == id:
                break
        else:
            extension_point = ExtensionPoint(id=id)
            self.contribute('dip.plugins.extension_points', extension_point)

        return extension_point
