# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from dip.model import implements, Model, observe

from dip.shell import IShell, IShellAction


@implements(IShellAction)
class ShellAction(Model):
    """ The ShellAction class is an :term:`action` intended to be used by a
    :term:`shell`.  The action can automatically disable and enable itself
    depending on the shell's current active tool.  The class has been designed
    so that it doesn't normally need to be sub-classed.
    """

    def perform(self, tool):
        """ Perform the action (probably) on a tool.  This will not be called
        if :meth:`~dip.shell.IShellAction.validate` returns ``False``.

        :param tool:
            is the tool.
        """

        if self.on_perform is not None:
            self.on_perform(tool)

    def validate(self, tool):
        """ Check if the action can be performed on a tool.

        :param tool:
            is the tool.
        :return:
            ``True`` if the action can be performed.
        """

        return True if self.handles is None else isinstance(tool, self.handles)

    @IShellAction.action.default
    def action(self):
        """ Get the default QAction. """

        from PyQt4.QtGui import QAction

        action = QAction(self.text, None,
                enabled=self._is_valid(self.shell.shell_manager.active_tool),
                objectName=self.id, statusTip=self.statusTip,
                toolTip=self.toolTip, triggered=self._action_triggered,
                visible=self.visible)

        if self.shortcut is not None:
            action.setShortcut(self.shortcut)

        return action

    @IShellAction.enabled.setter
    def enabled(self, enabled):
        """ The setter for the enabled attribute. """

        # Only change the enabled state of the action if it is valid for the
        # active tool.
        if self._is_valid(self.shell.shell_manager.active_tool):
            self.action.setEnabled(enabled)

    @IShellAction.visible.setter
    def visible(self, visible):
        """ The setter for the visible attribute. """

        self.action.setVisible(visible)

    @IShellAction.shell.observer
    def shell(self, change):
        """ Invoked when the shell changes. """

        if change.old is not None:
            observe('active_tool', change.old.shell_manager,
                    self._active_tool_changed, remove=True)

        if change.new is not None:
            observe('active_tool', change.new.shell_manager,
                    self._active_tool_changed)

    def _active_tool_changed(self, change):
        """ Invoked when the shell's active tool changes. """

        self.action.setEnabled(self.enabled and self._is_valid(change.new))

    def _action_triggered(self):
        """ Invoked when the action is triggered. """

        self.perform(self.shell.shell_manager.active_tool)

    def _is_valid(self, tool):
        """ Check if a tool is valid for this action. """

        if self.needs_tool and tool is None:
            return False

        return self.validate(tool)
