# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtGui import QWidget

from dip.model import Instance, Interface, List, Str
from dip.plugins import IPluginManager
from dip.ui import IAction, IToolkit

from .i_shell_manager import IShellManager


class IShell(Interface):
    """ The IShell interface defines the API of a :term:`shell`.

    The following action identifiers are considered to be well known.  A shell
    would normally be expected to honour them.

        dip.shell.actions.close
        dip.shell.actions.export
        dip.shell.actions.import
        dip.shell.actions.new
        dip.shell.actions.open
        dip.shell.actions.print
        dip.shell.actions.print_preview
        dip.shell.actions.printer_settings
        dip.shell.actions.quit
        dip.shell.actions.redo
        dip.shell.actions.save
        dip.shell.actions.save_as
        dip.shell.actions.undo
        dip.shell.actions.whats_this

    The following action collection identifiers are considered to be well
    known.  A shell, if it supports the concept of action collections, would
    normally be expected to honour them.

        dip.shell.collections.edit
        dip.shell.collections.file
        dip.shell.collections.help
        dip.shell.collections.tools
        dip.shell.collections.view
    """

    # The available actions.
    actions = List(Instance(IAction))

    # The identifier of the shell.  The shell's object name will default to the
    # identifier.
    id = Str()

    # The optional plugin manager.  This is normally set by the plugin that
    # publishes the shell as a service.
    plugin_manager = Instance(IPluginManager)

    # The shell manager.
    shell_manager = Instance(IShellManager)

    # The toolkit.
    toolkit = Instance(IToolkit)

    # The :class:`~PyQt4.QtGui.QWidget` instance that implements the shell.
    widget = Instance(QWidget)

    def __init__(self, **properties):
        """ Initialise the shell.

        :param \*\*properties:
            are keyword arguments used as property names and values that are
            applied to the shell widget.
        """

    def add_tool(self, tool):
        """ Add a :term:`tool` to the shell.

        :param tool:
            is the tool to add.
        """

    def perform_close(self, tool):
        """ Close a tool asking the user for help where necessary.

        :param tool:
            is the tool to close.
        """

    def perform_new(self):
        """ Create a tool for a new shell object asking the user for help where
        necessary.
        """

    def perform_open(self):
        """ Create a tool for a shell object read from storage asking the user
        for help where necessary.
        """

    def perform_quit(self):
        """ Confirm that the user wants to quit given the state of any shell
        objects.
        """

    def perform_save(self, tool):
        """ Write a shell object to storage at its current location asking the
        user for help where necessary.

        :param tool:
            is the tool operating on the shell object to write.
        """

    def perform_save_as(self, tool):
        """ Write a shell object to storage at a new location asking the user
        for help where necessary.

        :param tool:
            is the tool operating on the shell object to write.
        """
