# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from dip.model import implements, Instance, Model, observe
from dip.ui import IView

from .i_shell_object import IShellObject
from .i_tool_factory import IToolFactory


@implements(IToolFactory)
class ModelToolFactory(Model):
    """ The ModelToolFactory class is a :term:`tool` factory for any tool that
    uses an automatically generated :term:`view` to edit a :term:`model`.  It
    is not registered as a tool factory itself, instead it will normally be
    sub-classed and those sub-classes will be registered.
    """

    # The sub-class of Model that the tool can handle.
    model_type = Instance(type)

    def __call__(self, model, toolkit):
        """ Create an instance of the tool.

        :param model:
            is the model on which the tool is to operate on.
        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation used to create the
            view.
        :return:
            the instance of the tool.
        """

        view_factory = self.create_view_factory()
        view = view_factory(model, toolkit)

        so = IShellObject(model, exception=False)
        if so is not None:
            controller = IView(view).controller

            # Initialise the model's valid state.
            so.valid = (controller.invalid_reason == '')

            def update_valid(change):
                so.valid = (change.new == '')

            # Handle future changes.
            observe('invalid_reason', controller, update_valid)

        return view

    def create_view_factory(self):
        """ Create the view factory that will be used to create the view.
         This default implementation creates a :class:`~dip.ui.Form`.

        :return:
            the view factory.
        """

        from dip.ui import Form

        return Form()

    def handles(self, obj):
        """ Check if the object can be handled.

        :param obj:
            is the object.
        :return:
            ``True`` if the tool can handle the object.
        """

        return isinstance(obj, self.model_type)

