# Copyright (c) 2010 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtGui import QDialog, QWizard

from dip.shell import IShellObject

from .wizard_pages import ListPage, LocationPage


class NewOpenWizard(QWizard):
    """ The NewOpenWizard is a wizard that walks the user through selecting a
    particular tool, object and storage.
    """

    def __init__(self, shell, new_object, storage_factories, location):
        """ Initialise the wizard. """

        super(NewOpenWizard, self).__init__(shell.widget,
                windowTitle="New" if new_object else "Open",
                options=QWizard.IndependentPages|QWizard.NoBackButtonOnStartPage)

        shell_manager = shell.shell_manager

        self._tool_page = _ToolPage(self, shell_manager.tool_factories)
        self._obj_page = _ObjectPage(self, shell, self._tool_page, new_object)

        if new_object:
            self._storage_page = self._location_page = None
        else:
            # If we have an explicit list of storage factories then we already
            # have the location.
            if storage_factories is None:
                self._location_page = _ReadLocationPage(self,
                        shell_manager.toolkit)
            else:
                self._location_page = None

            self._storage_page = _ReadStoragePage(self, shell_manager,
                    storage_factories, location, self._obj_page,
                    self._location_page)

            # If the storage page is the first one then we have to tell the
            # wizard as it doesn't have the numerically lowest identifier.
            if self._tool_page.id < 0 and self._obj_page.id < 0:
                self.setStartId(self._storage_page.id)

    def exec_(self):
        """ Interact with the wizard and return the user choices. """

        if super(NewOpenWizard, self).exec_() != QDialog.Accepted:
            return None, None, None, None

        tool_factory = self._tool_page.selected
        obj = self._obj_page.selected
        storage = self._storage_page.selected

        if self._location_page is None:
            location = None
        else:
            location = self._location_page.get_location()

        return tool_factory, obj, storage, location

    def nextId(self):
        """ Reimplemented to determine the id of the next page. """

        page = self.currentPage()

        if page is self._tool_page:
            if self._obj_page.need_page():
                return self._obj_page.id

            page = self._obj_page

        if page is self._obj_page:
            if self._storage_page is not None:
                if self._storage_page.need_page():
                    return self._storage_page.id

                return self._location_page.id

        if page is self._storage_page:
            if self._location_page is not None:
                return self._location_page.id

        return -1


class _ToolPage(ListPage):
    """ A _ToolPage is a wizard page that allows the user to select a tool. """

    def __init__(self, wizard, tool_factories):
        """ Initialise the page. """

        super(_ToolPage, self).__init__(title="Choose a Tool",
                subTitle="From the list below choose the tool you want to "
                        "use.")

        self._tool_factories = tool_factories

        self.set_names([f.name for f in tool_factories])

        # See if this page is really needed.
        if len(tool_factories) > 1:
            self.id = wizard.addPage(self)

    def name_selected(self, name):
        """ Invoked when a tool name is selected. """

        # Update the selected object according to the name of the factory.
        for tool_factory in self._tool_factories:
            if tool_factory.name == name:
                self.selected = tool_factory
                self.selected_changed.emit(self.selected)
                break


class _ObjectPage(ListPage):
    """ An _ObjectPage is a wizard page that allows the user to select a tool
    object.
    """

    def __init__(self, wizard, shell, tool_page, new_object):
        """ Initialise the page. """

        verb = "create" if new_object else "open"

        super(_ObjectPage, self).__init__(title="Choose an Object Type",
                subTitle="From the list below choose the type of object that "
                        "you want to {0}.".format(verb))

        # Both tool factories and dip.io need the object itself (rather than
        # an object factory) so create one of each now.
        self._shell_objects = []
        for shell_object_factory in shell.shell_manager.shell_object_factories:
            obj = shell_object_factory(shell)
            IShellObject(obj).factory = shell_object_factory
            self._shell_objects.append(obj)

        self._update_objects(tool_page.selected)
        tool_page.selected_changed.connect(self._update_objects)

        # See if this page is really needed.
        if tool_page.id >= 0 or self.need_page():
            self.id = wizard.addPage(self)

    def _update_objects(self, tool_factory):
        """ Update the page with the objects that can be handled by a tool. """

        self.set_names(
                [IShellObject(obj).factory.name for obj in self._shell_objects
                        if tool_factory.handles(obj)])

    def name_selected(self, name):
        """ Invoked when an object name is selected. """

        # Update the selected object according to the name of the factory.
        for obj in self._shell_objects:
            if IShellObject(obj).factory.name == name:
                self.selected = obj
                self.selected_changed.emit(self.selected)
                break


class _ReadStoragePage(ListPage):
    """ A _ReadStoragePage is a wizard page that allows the user to select the
    storage to read an object from.
    """

    def __init__(self, wizard, shell_manager, storage_factories, location, obj_page, location_page):
        """ Initialise the page. """

        # Provide detail about where we are reading from if possible.
        reading = "the object" if location == '' else "<b>{0}</b>".format(
                location)

        super(_ReadStoragePage, self).__init__(title="Choose a Source",
                subTitle="From the list below choose the where you want to "
                        "read {0} from.".format(reading))

        self._shell_manager = shell_manager
        self._storage_factories = storage_factories
        self._location_page = location_page

        self._update_storage(obj_page.selected)
        obj_page.selected_changed.connect(self._update_storage)

        # See if this page is really needed.
        if obj_page.id >= 0 or self.need_page():
            self.id = wizard.addPage(self)

    def _update_storage(self, obj):
        """ Update the page with the storage names for the given object. """

        to = IShellObject(obj)

        self._storage_list = self._shell_manager.io_manager.readable_storage(
                obj, include_format=to.factory.native_format,
                storage_factories=self._storage_factories)

        if self._location_page is not None:
            self._location_page.set_storage_list(self._storage_list)

        self.set_names([s.factory.name for s in self._storage_list])

    def name_selected(self, name):
        """ Invoked when a storage name is selected. """

        # Update the selected storage according to the name.
        for storage in self._storage_list:
            if storage.factory.name == name:
                self.selected = storage

                if self._location_page is not None:
                    self._location_page.set_storage(storage)

                break


class _ReadLocationPage(LocationPage):
    """ A _ReadLocationPage is a base class for a wizard page that is embeds
    the widget supplied by storage to select a read location.
    """

    def __init__(self, wizard, toolkit, default_location=None):
        """ Initialise the page. """

        super(_ReadLocationPage, self).__init__(wizard, toolkit,
                default_location,
                "Choose the location in storage were you want to read the "
                "object from.")

    def get_browser(self, ui, location):
        """ Return the browser widget. """

        return ui.read_browser(self.toolkit, location)
