# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


import weakref

from PyQt4.QtGui import QDialog

from dip.io import StorageError
from dip.model import implements, Instance, List, Model, observe
from dip.shell import IShellAction, IShellManager, IShellObject, ITool

from .implicit_storage_wizard import ImplicitStorageWizard
from .new_open_wizard import NewOpenWizard
from .quit_confirmation_dialog import QuitConfirmationDialog
from .save_as_wizard import SaveAsWizard


@implements(IShellManager)
class ShellManager(Model):
    """ The ShellManager class is the default implementation of a
    :term:`shell manager`.
    """

    # The active managed tool, if there is one.
    _active_managed_tool = Instance(ITool)

    # The Close action.
    _close_action = Instance(IShellAction)

    # The New action.
    _new_action = Instance(IShellAction)

    # The Open action.
    _open_action = Instance(IShellAction)

    # The Save action.
    _save_action = Instance(IShellAction)

    # The Save As action.
    _save_as_action = Instance(IShellAction)

    # The list of weak references to the tools being managed.
    _tools = List()

    def close(self, tool):
        """ Close a tool asking the user for help where necessary.

        :param tool:
            is the tool to close.
        :return:
            ``True`` if the user didn't cancel.
        """

        so = IShellObject(ITool(tool).obj)

        if so.dirty:
            close = self.shell.toolkit.question("Close",
                    "<b>{0}</b> has been modified and all changes will be "
                    "lost.  Do you still want to close it?".format(so.name),
                    self.shell.widget)
        else:
            close = True

        if close:
            for weak_tool in self._tools:
                if weak_tool() is tool:
                    self._remove(weak_tool)
                    break

        return close

    def new(self):
        """ Create a tool for a new shell object asking the user for help where
        necessary.

        :return:
            the tool or None if the user cancelled.
        """

        return self._get_tool(new_object=True)

    def open(self, location=''):
        """ Create a tool for a shell object read from storage asking the user
        for help where necessary.

        :param location:
            is the optional location of the object to open.
        :return:
            the tool or None if the user cancelled.
        """

        return self._get_tool(new_object=False, location=location)

    def quit(self):
        """ Confirm that the user wants to quit given the state of any tool
        objects.

        :return:
            ``True`` if the user didn't cancel.
        """

        dirty_names = []
        for weak_tool in self._tools:
            so = IShellObject(ITool(weak_tool()).obj)

            if so.dirty:
                dirty_names.append(so.name)

        if len(dirty_names) == 0:
            return True

        if len(dirty_names) == 1:
            return self.shell.toolkit.question("Quit",
                    "<b>{0}</b> has been modified and all changes will be "
                    "lost.  Do you still want to quit the application?".format(
                            dirty_names[0]),
                    self.shell.widget)

        dlg = QuitConfirmationDialog(dirty_names, self.shell.widget)

        return dlg.exec_() == QDialog.Accepted

    def save(self, tool):
        """ Write a shell object to storage at its current location asking the
        user for help where necessary.

        :param tool:
            is the tool operating on the shell object to write.
        """

        obj = ITool(tool).obj
        so = IShellObject(obj)

        location = so.location

        if location is None:
            # We can only get here if there is a suitable storage with
            # implicit locations.
            storage_list = self.io_manager.writeable_storage(obj,
                    include_format=so.factory.native_format)

            # Get the list of implicit locations.
            locations = []
            for storage in storage_list:
                location = storage.implicit_location(obj)

                if location is not None:
                    locations.append(location)

            # Don't both the user if there is only one option.
            if len(locations) == 1:
                location = locations[0]
            else:
                wizard = ImplicitStorageWizard(locations, self.shell.widget)
                location = wizard.exec_()

        if location is not None:
            self._write_object(obj, location, self.shell)

    def save_as(self, tool):
        """ Write a shell object to storage at a new location asking the user
        for help where necessary.

        :param tool:
            is the tool operating on the shell object to write.
        """

        obj = ITool(tool).obj
        so = IShellObject(obj)

        storage_list = self.io_manager.writeable_storage(obj,
                include_format=so.factory.native_format)

        # See if a wizard is needed.
        if len(storage_list) == 1:
            # A single storage type will never have implicit locations.
            storage = storage_list[0]

            location = so.location
            if location is None:
                location = storage.explicit_location()

            storage.ui.get_write_location(self.shell.toolkit, location)

            if location.location == '':
                location = None
        else:
            wizard = SaveAsWizard(self, obj, storage_list, so.location,
                    self.shell.widget)
            location = wizard.exec_()

        if location is not None:
            self._write_object(obj, location, self.shell)

    @_close_action.default
    def _close_action(self):
        """ Get the Close action. """

        return self._find_action('dip.shell.actions.close')

    @_new_action.default
    def _new_action(self):
        """ Get the New action. """

        return self._find_action('dip.shell.actions.new')

    @_open_action.default
    def _open_action(self):
        """ Get the Open action. """

        return self._find_action('dip.shell.actions.open')

    @_save_action.default
    def _save_action(self):
        """ Get the Save action. """

        return self._find_action('dip.shell.actions.save')

    @_save_as_action.default
    def _save_as_action(self):
        """ Get the Save As action. """

        return self._find_action('dip.shell.actions.save_as')

    @IShellManager.io_manager.default
    def io_manager(self):
        """ The default i/o manager. """

        from dip.io.io_manager import IOManager

        return IOManager()

    @IShellManager.shell_object_factories.observer
    def shell_object_factories(self):
        """ Invoked when a new shell object factory is defined. """

        nr_shell_object_factories = len(self.shell_object_factories)

        if nr_shell_object_factories == 1:
            # This is the first one so configure the actions.
            if self._close_action is not None:
                self._close_action.visible = True

            if self._new_action is not None:
                self._new_action.visible = True

            if self._open_action is not None:
                self._open_action.visible = True

            if self._save_action is not None:
                self._save_action.visible = True

            if self._save_as_action is not None:
                self._save_as_action.visible = True

        elif nr_shell_object_factories == 2:
            if self._new_action is not None:
                # The New action will invoke a wizard.
                qaction = self._new_action.action

                text = qaction.text()
                if not text.endswith('...'):
                    qaction.setText(text + '...')

    @IShellManager.active_tool.observer
    def _active_tool_changed(self, change):
        """ Invoked when the shell's active tool changes. """

        tool = change.new

        # See if the tool is managed.
        itool = None
        if tool is not None:
            for weak_tool in self._tools:
                if weak_tool() is tool:
                    itool = ITool(tool)
                    break

        # See if the active managed tool has changed.
        if self._active_managed_tool is not itool:
            if self._active_managed_tool is not None:
                so = IShellObject(self._active_managed_tool.obj)
                observe('valid', so, self._update_actions, remove=True)

            if itool is not None:
                so = IShellObject(itool.obj)
                observe('valid', so, self._update_actions)

            self._active_managed_tool = itool

            self._update_actions()

    def _update_actions(self):
        """ Update the actions according to any current managed tool. """

        # Determine the valid lifecycle calls depending on the current object.
        save_enabled = save_as_enabled = close_enabled = False

        if self._active_managed_tool is not None:
            obj = self._active_managed_tool.obj
            so = IShellObject(obj)

            storage_list = self.io_manager.writeable_storage(obj,
                    include_format=so.factory.native_format)

            if len(storage_list) > 0:
                if so.location is not None:
                    save_enabled = so.valid

                nr_implicit_storage = 0

                for storage in storage_list:
                    if storage.implicit_location(obj) is None:
                        save_as_enabled = so.valid
                    else:
                        save_enabled = so.valid
                        nr_implicit_storage += 1

                if nr_implicit_storage > 1:
                    save_as_enabled = so.valid

            close_enabled = True

        if self._save_action is not None:
            self._save_action.enabled = save_enabled

        if self._save_as_action is not None:
            self._save_as_action.enabled = save_as_enabled

        if self._close_action is not None:
            self._close_action.enabled = close_enabled

    def _find_action(self, id):
        """ Return the shell action with the given identifier or None if it
        doesn't exist.
        """

        for action in self.shell.actions:
            if action.id == id:
                return action

        return None

    def _get_tool(self, new_object, location=''):
        """ Get a tool for a new or existing object. """

        need_wizard = False
        tool_factory = None
        obj = None
        storage_location = None

        # If we have been given a location then get the list of storage
        # factories that it is valid for.
        if new_object or location == '':
            storage_factories = None
        else:
            storage_factories = self.io_manager.storage_factories_for_location(
                    location)

            if len(storage_factories) == 0:
                self.shell.toolkit.warning("Read Error",
                        "{0} is not recognised as a valid storage location."
                                .format(location), self.shell.widget)
                return None

        # See if we need a wizard at all, i.e. see if the user needs to make
        # any choices.
        if len(self.tool_factories) != 1:
            need_wizard = True
        else:
            tool_factory = self.tool_factories[0]

            if len(self.shell_object_factories) != 1:
                need_wizard = True
            else:
                object_factory = self.shell_object_factories[0]
                obj = object_factory(self.shell)
                IShellObject(obj).factory = object_factory

                if not new_object:
                    storage_list = self.io_manager.readable_storage(obj,
                            include_format=object_factory.native_format,
                            storage_factories=storage_factories)

                    if len(storage_list) != 1:
                        need_wizard = True
                    else:
                        storage = storage_list[0]

        if need_wizard:
            wizard = NewOpenWizard(self.shell, new_object, storage_factories,
                    location)
            tool_factory, obj, storage, storage_location = wizard.exec_()

            if tool_factory is None:
                return None

        if not new_object:
            # Get the storage location if necessary (because we didn't use the
            # wizard or we were given an explicit location).
            if storage_location is None:
                storage_location = storage.explicit_location(location)

                # Ask the user if a location wasn't provided.
                if storage_location.location == '':
                    storage.ui.get_read_location(self.shell.toolkit,
                            storage_location)
                    if storage_location.location == '':
                        return None

            try:
                obj = storage_location.storage.read(obj, storage_location)
            except StorageError as e:
                self.shell.toolkit.warning("Read Error",
                        "There was an error reading from '{0}'.".format(
                                storage_location.location),
                        self.shell.widget, str(e))

        # Set up the shell object.
        so = IShellObject(obj)
        so.location = storage_location
        so.dirty = False

        # Create the tool.
        tool = tool_factory(obj, self.shell.toolkit)

        weak_tool = weakref.ref(tool, self._remove)
        self._tools.append(weak_tool)

        itool = ITool(tool)
        itool.factory = tool_factory
        itool.obj = obj

        return tool

    def _remove(self, weak_tool):
        """ Remove a tool. """

        self._tools.remove(weak_tool)

    def _write_object(self, obj, storage_location, shell):
        """ Write a managed object to a location. """

        try:
            storage_location.storage.write(obj, storage_location)
        except StorageError as e:
            self.shell.toolkit.warning("Write Error",
                    "There was an error writing to '{0}'.".format(
                            storage_location.location),
                    shell.widget, str(e))

        so = IShellObject(obj)
        so.location = storage_location
        so.dirty = False
