# Copyright (c) 2010 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtCore import pyqtSignal, Qt
from PyQt4.QtGui import (QAbstractItemView, QItemSelectionModel, QListView,
        QStackedLayout, QStringListModel, QVBoxLayout, QWizardPage)

from dip.model import observe


class _BasePage(QWizardPage):
    """ A _BasePage is the base wizard page. """

    def __init__(self, **kwargs):
        """ Initialise the page. """

        super(_BasePage, self).__init__(**kwargs)

        self.setLayout(self.contents())

        self.id = -1


class ListPage(_BasePage):
    """ A ListPage is a wizard page that is used to select from a list of
    names.
    """

    selected_changed = pyqtSignal(object)

    def __init__(self, **kwargs):
        """ Initialise the page. """

        super(ListPage, self).__init__(**kwargs)

        self.selected = None

    def contents(self):
        """ Return the page's contents. """

        self._list_view = QListView(
                editTriggers=QAbstractItemView.NoEditTriggers)

        self._model = QStringListModel()
        self._list_view.setModel(self._model)

        self._list_view.selectionModel().currentChanged.connect(
                self._on_current_changed)

        layout = QVBoxLayout()
        layout.addWidget(self._list_view)

        return layout

    def set_names(self, names):
        """ Display the given list of names. """

        self._model.setStringList(names)

        selection_model = self._list_view.selectionModel()
        if len(names) == 0:
            initial_name = None
            selection_model.clear()
        else:
            self._model.sort(0)

            idx = self._model.index(0)
            initial_name = self._model.data(idx, Qt.EditRole)
            selection_model.select(idx, QItemSelectionModel.Select)

        # If this is not the current page then the currentChanged() signal
        # won't get emitted.
        self.name_selected(initial_name)

    def need_page(self):
        """ Return True if the page is needed because it has zero or more than
        one name.
        """

        return self._model.rowCount() != 1

    def _on_current_changed(self, idx):
        """ Invoked when a name is selected. """

        self.name_selected(
                self._model.data(idx, Qt.EditRole) if idx.isValid() else None)


class LocationPage(_BasePage):
    """ A LocationPage is a base class for a wizard page that is embeds the
    widget supplied by storage to select a location.
    """

    def __init__(self, wizard, toolkit, default_location, subtitle):
        """ Initialise the page. """

        super(LocationPage, self).__init__(title="Choose a Location",
                subTitle=subtitle)

        self.toolkit = toolkit
        self._default_location = default_location

        # There is a browser for each storage factory.  This is a 2-tuple of
        # the location and the actual browser widget.
        self._browsers = []

        self.id = wizard.addPage(self)

    def contents(self):
        """ Return the page's contents. """

        return QStackedLayout()

    def get_location(self):
        """ Return the current location. """

        location, _ = self._browsers[self.layout().currentIndex()]

        return location if location.location != '' else None

    def set_storage_list(self, storage_list):
        """ The stack of browsers is updated for the given storage list. """

        layout = self.layout()

        # Remove any existing locations and browsers.
        for location, browser in self._browsers:
            observe('location', location, self._on_location_change,
                    remove=True)
            layout.removeWidget(browser)

        self._browsers = []

        # Create locations and browsers for the new storage.
        for storage in storage_list:
            # Use the default location if possible.
            location = storage.explicit_location('')
            if type(location) is type(self._default_location):
                location = self._default_location

            observe('location', location, self._on_location_change)

            browser = self.get_browser(storage.ui, location)

            self._browsers.append((location, browser))

            layout.addWidget(browser)

    def set_storage(self, storage):
        """ The browser for the given storage is made current. """

        for idx, (location, _) in enumerate(self._browsers):
            if location.storage is storage:
                self.layout().setCurrentIndex(idx)
                break

    def isComplete(self):
        """ Reimplemented to check that there is a location. """

        return self.get_location() is not None

    def _on_location_change(self):
        """ Invoked when the user changes the value of a storage location. """

        self.completeChanged.emit()
