# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtCore import Qt
from PyQt4.QtGui import (QApplication, QDockWidget, QMainWindow, QMenu,
        QMenuBar, QTabWidget)

from dip.automate import IAutomatedActionTrigger, IAutomatedShell
from dip.model import adapt, Adapter, Enum, List, implements, Instance, observe
from dip.shell import IShell, IShellAction, IShellObject, ITool, IUnmanagedTool
from dip.shell.actions import ShellAction
from dip.ui import Action, ActionCollection, MenuBar, QObjectFactory


@implements(IShell)
class QMainWindowShell(QObjectFactory):
    """ The QMainWindowShell class is an implementation of a user interface
    shell using :class:`~PyQt4.QtGui.QMainWindow`.  It places tools in the
    central widget according to the
    :attr:`~dip.shell.shells.qmainwindow.QMainWindowShell.tab_policy` and
    unmanaged tools in dock widgets.
    """

    # The shell's identifier.
    id = 'dip.shell'

    # The :class:`~PyQt4.QtGui.QMenuBar` that is added to the
    # :class:`~PyQt4.QtGui.QMainWindow`.  If it is ``None`` then no menu bar is
    # added.
    menu_bar = Instance(QMenuBar)

    # The list of menus to be added to the menu bar, typically by using
    # :class:`~dip.ui.MenuBar`.
    menus = List()

    # This specifies when tabs should be used.  ``'on_demand'`` means that tabs
    # are displayed when there is more than one tool.  ``'always'`` means that
    # tabs are always displayed, even when there is only one tool.
    # ``'never'`` means that tabs are never displayed and only one tool can be
    # used at a time.
    tab_policy = Enum('on_demand', 'always', 'never')

    # The list of QDockWidgets.
    _dock_widgets = List()

    def __init__(self, **properties):
        """ Initialize the QMainWindow shell. """

        super(QMainWindowShell, self).__init__(**properties)

        self.configure_properties(self.widget)
        self.finalize(self.widget)

        app = QApplication.instance()

        app_name = app.applicationName()
        if app_name != "":
            app_version = app.applicationVersion()
            if app_version != "":
                app_name = "{0} {1}".format(app_name, app_version)

            self.widget.setWindowTitle(app_name)

        tw = QTabWidget()
        tw.setTabsClosable(True)
        tw.tabCloseRequested.connect(self._close_tool)
        self._hide_tab_widget(tw)

        app.focusChanged.connect(self._on_focus_change)

        self.widget.setCentralWidget(tw)

        # Add any menu bar.
        if self.menu_bar is not None:
            self.widget.setMenuBar(self.menu_bar)

        # Add any actions.
        for action in self.actions:
            self._add_action(action)

        # Handle any actions added in the future.  Note that we don't use the
        # observe decorator to handle this because that would mean initial
        # actions would be added before __init__ was called.
        observe('actions', self, self._actions_changed)

    @IShell.widget.default
    def widget(self):
        """ The default QMainWindow instance. """

        return _QMainWindow(self)

    def add_tool(self, tool):
        """ Add a :term:`tool` to the shell.

        :param tool:
            is the tool to add.
        """
        so = IShellObject(ITool(tool).obj)

        tw = self.widget.centralWidget()

        idx = tw.addTab(tool, '')
        self._update_tab(idx, so)

        observe('name', so, lambda: self._update_tab(idx, so))

        # Show the QTabWidget if it is now needed.
        nr_tools = tw.count()

        if (nr_tools == 1 and self.tab_policy == 'always') or \
           (nr_tools == 2 and self.tab_policy == 'on_demand'):
            tw.tabBar().show()
            tw.setStyleSheet('')

        self._set_tool_focus(tool)

        tw.setCurrentIndex(idx)

    def perform_close(self, tool):
        """ Close a tool asking the user for help where necessary.

        :param tool:
            is the tool to close.
        """

        self._close_tool(self.widget.centralWidget().currentIndex())

    def perform_new(self):
        """ Create a tool for a new shell object asking the user for help where
        necessary.
        """

        if self._prepare_for_tool():
            tool = self.shell_manager.new()

            if tool is not None:
                self.add_tool(tool)

    def perform_open(self):
        """ Create a tool for a shell object read from storage asking the user
        for help where necessary.
        """

        if self._prepare_for_tool():
            tool = self.shell_manager.open()

            if tool is not None:
                self.add_tool(tool)

    def perform_quit(self):
        """ Confirm that the user wants to quit given the state of any shell
        objects.
        """

        self.widget.close()

    def perform_save(self, tool):
        """ Write a shell object to storage at its current location asking the
        user for help where necessary.

        :param tool:
            is the tool operating on the shell object to write.
        """

        self.shell_manager.save(tool)

    def perform_save_as(self, tool):
        """ Write a shell object to storage at a new location asking the user
        for help where necessary.

        :param tool:
            is the tool operating on the shell object to write.
        """

        self.shell_manager.save_as(tool)

    @menus.default
    def menus(self):
        """ Get the default list of menus. """

        return [
                ActionCollection(id='dip.shell.collections.file',
                        title="&File", members=[
                                'dip.shell.actions.new',
                                'dip.shell.actions.open',
                                'dip.shell.actions.save',
                                'dip.shell.actions.save_as',
                                'dip.shell.actions.close',
                                '',
                                'dip.shell.actions.import',
                                'dip.shell.actions.export',
                                '',
                                'dip.shell.actions.print_preview',
                                'dip.shell.actions.print',
                                'dip.shell.actions.printer_settings',
                                '',
                                'dip.shell.actions.quit']),
                ActionCollection(id='dip.shell.collections.edit',
                        title="&Edit", members=[
                                'dip.shell.actions.undo',
                                'dip.shell.actions.redo']),
                ActionCollection(id='dip.shell.collections.view',
                        title="&View"),
                ActionCollection(id='dip.shell.collections.tools',
                        title="&Tools"),
                ActionCollection(id='dip.shell.collections.help',
                        title="&Help", members=[
                                'dip.shell.actions.whats_this'])]

    @menu_bar.default
    def menu_bar(self):
        """ Get the default menu bar. """

        return MenuBar(*self.menus)(self.toolkit)

    @IShell.shell_manager.default
    def shell_manager(self):
        """ The default shell manager. """

        from dip.shell.shell_manager import ShellManager

        return ShellManager(shell=self)

    @shell_manager.observer
    def shell_manager(self, change):
        """ Invoked when the shell manager changes. """

        if change.old is not None:
            change.old.shell = None

        if change.new is not None:
            change.new.shell = self

    @IShell.toolkit.default
    def toolkit(self):
        """ The default toolkit. """

        from dip.ui.toolkits.qt import QtToolkit

        return QtToolkit()

    @IShell.actions.default
    def actions(self):
        """ The default shell actions. """

        from dip.shell.actions import (CloseAction, NewAction, OpenAction,
                QuitAction, SaveAction, SaveAsAction)

        return [CloseAction(), NewAction(), OpenAction(), QuitAction(),
                SaveAction(), SaveAsAction()]

    def _actions_changed(self, change):
        """ Invoked when a new action is available. """

        # FIXME: At the moment actions can only be added.
        for action in change.new:
            self._add_action(action)

    def _add_action(self, action):
        """ Add a new action. """

        before = action.before

        if isinstance(action, ActionCollection):
            # If this collection is at the top level but not explicitly placed
            # then put it before the Tools menu.
            if action.within == '' and before == '':
                before = 'dip.shell.collections.tools'

        elif isinstance(action, IUnmanagedTool):
            # Create a dock widget for the unmanaged tool.
            dock_widget = QDockWidget(action.name)
            dock_widget.setWidget(action.widget)
            self.widget.addDockWidget(Qt.LeftDockWidgetArea, dock_widget)
            self._dock_widgets.append(dock_widget)

            # The tool is initially hidden.
            dock_widget.hide()

            # Configure the QAction.
            action.action = qaction = dock_widget.toggleViewAction()
            qaction.setObjectName(action.id)

            # Only override values set by Qt if we have our own.
            if action.shortcut is not None:
                qaction.setShortcut(action.shortcut)

            if action.statusTip != "":
                qaction.setStatusTip(action.statusTip)

            if action.text != "":
                qaction.setText(action.text)

            if action.toolTip != "":
                qaction.setToolTip(action.toolTip)

            self._set_tool_focus(action.widget)

        elif isinstance(action, IShellAction):
            action.shell = self

        MenuBar.add(action, self.widget.menuBar(), self.toolkit, action.within,
                before)

    def _on_focus_change(self, old, now):
        """ Invoked when the focus changes. """

        # If a menu or menu bar now has the focus then ignore the change.
        if isinstance(now, (QMenu, QMenuBar)):
            return

        # Find the tool, if any, that has the focus widget.
        active_tool = None

        if now is not None:
            tool = self.widget.centralWidget().currentWidget()
            if tool is not None and tool.isAncestorOf(now):
                active_tool = tool
            else:
                for dock_widget in self._dock_widgets:
                    if dock_widget.isAncestorOf(now):
                        active_tool = dock_widget.widget()
                        break

        self.shell_manager.active_tool = active_tool

    def _set_tool_focus(self, tool):
        """ Set the focus to an appropriate widget in a tool. """

        # Set the focus to the first thing that will accept it.
        while tool is not None:
            if tool.focusPolicy() != Qt.NoFocus:
                tool.setFocus()
                break

            tool = tool.nextInFocusChain()

    @staticmethod
    def _hide_tab_widget(tw):
        """ Configure a QTabWidget so that it appears not to be there. """

        tw.tabBar().hide()
        tw.setStyleSheet(
                'QTabWidget::pane {margin: 0; border-width: 0; padding: 0;}')

    def _prepare_for_tool(self):
        """ Make sure we can add a tool and return True if we can. """

        # If only one tool is allowed at a time then close any existing one.
        if self.widget.centralWidget().count() == 1 and self.tab_policy == 'never':
            return self._close_tool(0)

        return True

    def _close_tool(self, idx):
        """ Invoked to close the tool at the given index. """

        tw = self.widget.centralWidget()
        tool = tw.widget(idx)

        if not self.shell_manager.close(tool):
            return False

        tw.removeTab(idx)

        # Reconfigure the tab widget according to the tab policy.
        nr_tools = tw.count()

        if nr_tools == 0 or (nr_tools == 1 and self.tab_policy == 'on_demand'):
            self._hide_tab_widget(tw)

        # Qt doesn't seem to generate a focus change when the last tool is
        # closed.
        if nr_tools == 0:
            self.shell_manager.active_tool = None

        return True

    def _update_tab(self, idx, so):
        """ Update the text and tool tip of a tab. """

        tw = self.widget.centralWidget()

        text = so.short_name
        if text == '':
            text = so.name
        else:
            tw.setTabToolTip(idx, so.name)

        tw.setTabText(idx, text)


@adapt(QMainWindow, to=IAutomatedShell)
class QMainWindowIAutomatedShellAdapter(Adapter):
    """ The QMainWindowIAutomatedShellAdapter adapts a
    :class:`~PyQt4.QtGui.QMainWindow` to the
    :class:`~dip.shell.IAutomatedShell` interface.
    """

    def simulate_trigger(self, id, delay, action_id):
        """ Simulate the user triggering an action.

        :param id:
            is the (possibly scoped) identifier of the widget.  This is
            normally used in exceptions.
        :param delay:
            is the delay in milliseconds before the simulation.
        :param action_id:
            is the identifier of the action.
        """

        # Delegate to the menu bar.
        IAutomatedActionTrigger(
                self.adaptee.menuBar()).simulate_trigger(id, delay, action_id)


class _QMainWindow(QMainWindow):
    """ A QMainWindow sub-class that intercepts a close event. """

    def __init__(self, shell):
        """ Initialise the object. """

        super(_QMainWindow, self).__init__()

        self._shell = shell

    def closeEvent(self, ev):
        """ Reimplemented to allow the shell manager to veto the close. """

        if self._shell.shell_manager.quit():
            ev.accept()
        else:
            ev.ignore()
