# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtGui import QAction, QMenu

from .i_action import IAction
from .i_qaction_action import IQActionAction
from .action_collection import ActionCollection
from .qobject_factory import QObjectFactory


class BaseMenu(QObjectFactory):
    """ The BaseMenu class is a base class for action containers, i.e. those
    that implement :class:`~PyQt4.QtGui.QMenuBar` or
    :class:`~PyQt4.QtGui.QMenu`.
    """

    @classmethod
    def add(cls, item, widget, toolkit, within='', before=''):
        """ An :class:`~dip.ui.IAction` or a :class:`~PyQt4.QtGui.QMenu` is
        added to a menu bar, menu or a sub-menu.

        If an existing item has the same identifier as the item being added
        then the existing item will be replaced with the new item.

        :param item:
            is the :class:`~dip.ui.IAction` or :class:`~PyQt4.QtGui.QMenu` to
            insert.
        :param widget:
            is the menu bar or menu.
        :param toolkit:
            is the toolkit.
        :param within:
            is the identifier of the action collection or menu that the item
            will be added to.  If it is empty then the item will be added to
            the widget itself.
        :param before:
            is the identifier of the existing item that the item is inserted
            immediately before.  An exception is raised if there is no such
            existing item.  If the identifier is empty then the item will be
            added at the end of the containing widget.
        """

        # Get the identifier.
        if isinstance(item, IAction):
            item_id = item.id
        else:
            item_id = item.objectName()

        # Try and replace an existing item.
        visible = cls._walk_widget(widget, toolkit, item_id,
                cls._replace_handler, item)

        if visible is None:
            if within != '':
                if before != '':
                    raise ValueError(
                            "cannot specify values for both 'before' and "
                            "'within'")

                visible = cls._walk_widget(widget, toolkit, within,
                        cls._add_handler, item)

            elif before != '':
                visible = cls._walk_widget(widget, toolkit, before,
                        cls._insert_handler, item)

            else:
                visible = cls._insert_item(item, widget, toolkit)

        if isinstance(widget, QMenu) and visible is not None and visible:
            widget.menuAction().setVisible(True)

    @classmethod
    def _walk_widget(cls, widget, toolkit, id, handler, item):
        """ Walk a hierachy of menus, depth first, looking for an item with a
        particular identifier.  If found, call a handler.  Return ``None`` if
        no item was found, ``True`` if a visible item was found, and ``False``
        if an invisible item was found.  A handler should return ``True`` if
        the container should be visible.
        """

        actions = widget.actions()

        for a, action in enumerate(actions):
            if action.objectName() == id:
                return handler(widget, toolkit, actions, a, item)

            # Recurse down any menus.
            sub_menu = action.menu()
            if sub_menu is not None:
                visible = cls._walk_widget(sub_menu, toolkit, id, handler,
                        item)
                if visible is not None:
                    if visible:
                        action.setVisible(True)

                    return visible

        return None

    @classmethod
    def _replace_handler(cls, widget, toolkit, actions, a, item):
        """ Replace an action in a widget with an item. """

        # Get the action being replaced.
        action = actions[a]

        # If the action is a separator then it corresponds to the start of a
        # collection.  Remove all the actions up to the corresponding closing
        # separator.
        if action.isSeparator():
            id = action.objectName()
            widget.removeAction(action)

            for action in actions[a + 1:]:
                if action.objectName() == id:
                    break

                widget.removeAction(action)

        visible = cls._insert_item(item, widget, toolkit, action)
        widget.removeAction(action)

        return visible

    @classmethod
    def _add_handler(cls, widget, toolkit, actions, a, item):
        """ Add an item to a widget. """

        # If the container is a menu then get it.
        action = actions[a]
        qmenu = action.menu()
        if qmenu is not None:
            widget = qmenu

        visible = cls._insert_item(item, widget, toolkit)
        if visible:
            action.setVisible(True)

        return visible

    @classmethod
    def _insert_handler(cls, widget, toolkit, actions, a, item):
        """ Insert an item in a widget before an action. """

        return cls._insert_item(item, widget, toolkit, actions[a])

    @classmethod
    def _insert_item(cls, item, widget, toolkit, before=None):
        """ Insert the QActions that implement the given item before the given
        action.  Return ``True`` if the actions are visible.
        """

        if isinstance(item, ActionCollection):
            qmenu = toolkit.menu(item.title, widget)
            qmenu.menuAction().setObjectName(item.id)
            cls._insert_menu(widget, before, qmenu)

            return cls._insert_collection_actions(item, qmenu, toolkit)

        if isinstance(item, QMenu):
            cls._insert_menu(widget, before, item)
        elif isinstance(item, IQActionAction):
            cls._insert_action(widget, before, item.action)
        else:
            raise TypeError("unknown item type: {0}".format(item))

        return True

    @classmethod
    def _insert_collection_actions(cls, collection, widget, toolkit, before=None):
        """ Recursively insert the actions for a collection to a widget before
        the given action.  Return ``True`` if any of the actions added are
        visible.
        """

        visible = False

        for member in collection.members:
            if isinstance(member, ActionCollection):
                if member.title != '':
                    sub_qmenu = toolkit.menu(member.title, widget)

                    menu_action = sub_qmenu.menuAction()
                    menu_action.setObjectName(member.id)
                    menu_action.setVisible(False)

                    cls._insert_menu(widget, before, sub_qmenu)
                else:
                    # Add the contents of the collection surrounded by
                    # separators.  Qt is smart about collapsing multiple
                    # separators.
                    id = collection.id

                    cls._insert_separator(qmenu, before, id)

                    if cls._insert_collection_actions(qmenu, before, toolkit, member):
                        visible = True

                    cls._insert_separator(qmenu, before, id)

            elif isinstance(member, str):
                if member == '':
                    cls._insert_separator(widget, before)
                else:
                    # Insert a place holder action.
                    cls._insert_action(widget, before,
                            _PlaceHolder(widget, member))

            elif isinstance(member, IQActionAction):
                cls._insert_action(widget, before, member.action)
                widget.menuAction().setVisible(True)

                # A real QAction so we need the parent hierarchy.
                visible = True
            else:
                raise TypeError("unknown member type: {0}".format(member))

        return visible

    @staticmethod
    def _insert_action(widget, before, action):
        """ Insert or add an action to a widget. """

        if before is None:
            widget.addAction(action)
        else:
            widget.insertAction(before, action)

    @staticmethod
    def _insert_menu(widget, before, menu):
        """ Insert or add a menu to a widget. """

        if before is None:
            widget.addMenu(menu)
        else:
            widget.insertMenu(before, menu)

    @staticmethod
    def _insert_separator(widget, before, id=None):
        """ Insert or add a separator with an optional identifier to a widget.
        """

        if before is None:
            separator = widget.addSeparator()
        else:
            separator = widget.insertSeparator(before)

        if id is not None:
            separator.setObjectName(id)


class _PlaceHolder(QAction):
    """ The _PlaceHolder class is an invisible QAction that keeps a place in a
    menu for another action with the same identifier.
    """

    def __init__(self, menu, id):
        """ Initialise the instance. """

        super(_PlaceHolder, self).__init__(menu, objectName=id, visible=False)
