# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from dip.model import Model, Subclass, ValueTypeFactory

from .editor_factory import EditorFactory


class BaseToolkit(Model):
    """ The BaseToolkit class is a base class for :term:`toolkits<toolkit>`
    that implements the default mapping of :mod:`dip.model` types to the
    :term:`editors<editor>` used to handle them.
    """

    # The editor factory to handle the Bool type.
    Bool_editor_factory = Subclass(EditorFactory)

    # The delegate editor factory to handle the Bool type.
    Bool_delegate_factory = Subclass(EditorFactory)

    # The editor factory to handle the Enum type.
    Enum_editor_factory = Subclass(EditorFactory)

    # The delegate editor factory to handle the Enum type.
    Enum_delegate_factory = Subclass(EditorFactory)

    # The editor factory to handle the Float type.
    Float_editor_factory = Subclass(EditorFactory)

    # The delegate editor factory to handle the Float type.
    Float_delegate_factory = Subclass(EditorFactory)

    # The editor factory to handle the Int type.
    Int_editor_factory = Subclass(EditorFactory)

    # The delegate editor factory to handle the Int type.
    Int_delegate_factory = Subclass(EditorFactory)

    # The editor factory to handle the List type.
    List_editor_factory = Subclass(EditorFactory)

    # The editor factory to handle the Str type.
    Str_editor_factory = Subclass(EditorFactory)

    # The delegate editor factory to handle the Str type.
    Str_delegate_factory = Subclass(EditorFactory)

    # The editor factory to handle the Trigger type.
    Trigger_editor_factory = Subclass(EditorFactory)

    @Bool_editor_factory.default
    def Bool_editor_factory(self):
        """ The editor factory to handle the Bool type. """

        from .check_box import CheckBox

        return CheckBox

    @Bool_delegate_factory.default
    def Bool_delegate_factory(self):
        """ The delegate factory to handle the Bool type. """

        from .check_box import CheckBox

        return CheckBox

    @Enum_editor_factory.default
    def Enum_editor_factory(self):
        """ The editor factory to handle the Enum type. """

        from .combo_box import ComboBox

        return ComboBox

    @Enum_delegate_factory.default
    def Enum_delegate_factory(self):
        """ The delegate factory to handle the Enum type. """

        from .combo_box import ComboBox

        return ComboBox

    @Float_editor_factory.default
    def Float_editor_factory(self):
        """ The editor factory to handle the Float type. """

        from .float_spin_box import FloatSpinBox

        return FloatSpinBox

    @Float_delegate_factory.default
    def Float_delegate_factory(self):
        """ The delegate factory to handle the Float type. """

        from .float_spin_box import FloatSpinBox

        return FloatSpinBox

    @Int_editor_factory.default
    def Int_editor_factory(self):
        """ The editor factory to handle the Int type. """

        from .spin_box import SpinBox

        return SpinBox

    @Int_delegate_factory.default
    def Int_delegate_factory(self):
        """ The delegate factory to handle the Int type. """

        from .spin_box import SpinBox

        return SpinBox

    @List_editor_factory.default
    def List_editor_factory(self):
        """ The editor factory to handle the List type. """

        from .list_widget import ListWidget

        return ListWidget

    @Str_editor_factory.default
    def Str_editor_factory(self):
        """ The editor factory to handle the Str type. """

        from .line_edit import LineEdit

        return LineEdit

    @Str_delegate_factory.default
    def Str_delegate_factory(self):
        """ The delegate factory to handle the Str type. """

        from .line_edit import LineEdit

        return LineEdit

    @Trigger_editor_factory.default
    def Trigger_editor_factory(self):
        """ The editor factory to handle the Trigger type. """

        from .push_button import PushButton

        return PushButton

    def delegate_for_attribute(self, attribute_type):
        """ Return an :class:`~dip.ui.EditorFactory` type object that will
        create a delegate factory to handle attributes of a particular type.

        :param attribute_type:
            is the type of the attribute.
        :return:
            the type object.
        """

        delegate_factory_type = self._lookup(attribute_type,
                '_delegate_factory')

        if delegate_factory_type is None:
            raise TypeError(
                    "there is no default delegate factory for objects of type "
                    "'{0}'".format(type(attribute_type).__name__))

        return delegate_factory_type

    def editor_for_attribute(self, attribute_type):
        """ Return an :class:`~dip.ui.EditorFactory` type object that will
        create an editor factory to handle attributes of a particular type.

        :param attribute_type:
            is the type of the attribute.
        :return:
            the type object.
        """

        editor_factory_type = self._lookup(attribute_type, '_editor_factory')

        if editor_factory_type is None:
            raise TypeError(
                    "there is no default editor factory for objects of type "
                    "'{0}'".format(type(attribute_type).__name__))

        return editor_factory_type

    def _lookup(self, attribute_type, suffix):
        """ Lookup an attribute for an attribute type and a suffix. """

        for mro in type(attribute_type).__mro__:
            if mro is ValueTypeFactory:
                break

            try:
                return getattr(self, mro.__name__ + suffix)
            except AttributeError:
                pass

        return None
