# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtGui import QWidget

from dip.model import get_attribute_type, MappingProxy, Model

from .editor_factory import EditorFactory
from .view_factory import ViewFactory


class Bindings(object):
    """ The Bindings class represents set of bindings between widgets of a user
    interface and attributes of a model.
    """

    def __init__(self, **bindings):
        """ Initialise the bindings.

        :param \*\*bindings:
            the bindings specified as keyword arguments.  The argument name is
            the object name of the widget that the attribute will be bound to.
            The argument value is the binding.  If the argument value is a
            string then an :class:`~dip.ui.EditorFactory` instance will be
            automatically created for it.
        """

        self._bindings = bindings

    def bind(self, view, model, toolkit, controller=None):
        """ Bind a view to a model.

        :param view:
            is the view.
        :param model:
            is the model which is either a :class:`~dip.model.Model` instance
            or a Python mapping object.
        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation used to create the
            view.
        :param controller:
            is the optional controller.  A new :class:`~dip.ui.Controller`
            instance will be created if necessary.
        """

        # Make sure we have a controller.
        if controller is None:
            from .controller import Controller

            controller = Controller()

        # Make sure we have a Model.
        if not isinstance(model, Model):
            model = MappingProxy(model)

        controller.model = model
        controller.view = view

        finalize = []
 
        for widget_name, itm in self._bindings.items():
            # Find the sub-view in the user interface.
            if view.objectName() == widget_name:
                subview = view
            else:
                subview = view.findChild(QWidget, widget_name)

            if subview is None:
                raise NameError(
                        "'{0}' has no layout or widget with an object name of "
                        "'{1}'".format(view, widget_name))

            # Automatically create a factory if needed.
            if isinstance(itm, EditorFactory):
                attribute_type = get_attribute_type(model, itm.bind_to)
                subview_factory = itm
            elif isinstance(itm, str):
                attribute_type = get_attribute_type(model, itm)

                # Create an editor factory based on the widget type.
                editor_factory_factory = toolkit.editor_for_widget(
                        type(subview))
                subview_factory = editor_factory_factory(itm, id=widget_name)
            elif isinstance(itm, ViewFactory):
                attribute_type = None
                subview_factory = itm
            else:
                raise TypeError(
                        "the binding for '{0}' must be either an instance of "
                        "a ViewFactory sub-class or a string, not "
                        "'{1}'".format(widget_name, type(itm).__name__))

            if attribute_type is None:
                subview_factory.bind(subview, controller, toolkit)
            else:
                subview_factory.bind(subview, controller, toolkit,
                        attribute_type)

            finalize.append((subview_factory, subview))

        for subview_factory, subview in finalize:
            subview_factory.finalize(subview)
