# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from dip.model import get_attribute_type, get_model_types, List

from .i_composite_view import ICompositeView
from .i_view import IView
from .view_factory import ViewFactory


class CompositeViewFactory(ViewFactory):
    """ The CompositeViewFactory class is the base factory class for all views
    that can contain sub-views.  A sub-class determines the layout of the
    sub-views.
    """

    # The specific interface that the view implements.
    view_interface = ICompositeView

    # The contents of the view.
    _contents = List()

    def __init__(self, *contents, **properties):
        """ Initialise the view.

        :param \*contents:
            is the list of the view's contents.  An individual item can either
            be a :class:`~dip.ui.ViewFactory` instance or a string.  Strings
            are are assumed to be the names of attributes within a model.  Such
            names may include one or more periods to specify an
            :term:`attribute path`.
        :param \*\*properties:
            are keyword arguments used as property names and values that are
            applied to the view.
        """

        super(CompositeViewFactory, self).__init__(**properties)

        self._contents = list(contents)

    def create_subviews(self, controller, toolkit, parent):
        """ Return a list of :class:`~PyQt4.QtGui.QLayout` and
        :class:`~PyQt4.QtGui.QWidget` instances that implement the sub-views.
        A sub-class may include additional contents of different types, but
        these must be able to be handled by the view that the factory creates.

        :param controller:
            is the controller.
        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation used to create the
            view.
        :param parent:
            is the optional parent widget.
        :return:
            the list of :class:`~PyQt4.QtGui.QLayout` and
            :class:`~PyQt4.QtGui.QWidget` instances.
        """

        # If there are no explicit contents then inspect the model.
        contents = self._contents

        if len(contents) == 0:
            contents = get_model_types(type(controller.model)).keys()

        # Go through the contents invoking the corresponding factories.
        subviews = []

        for content in contents:
            if isinstance(content, str):
                attribute_type = get_attribute_type(controller.model, content)

                # Create an editor factory based on the type.
                editor_factory_type = toolkit.editor_for_attribute(
                        attribute_type)
                factory = editor_factory_type(content)

                # Create the editor from the factory.
                subview = factory.create_view(controller, toolkit, parent)
            elif isinstance(content, ViewFactory):
                subview = content.create_view(controller, toolkit, parent)
            else:
                subview = self.create_additional_content(content)

            subviews.append(subview)

        return subviews

    def create_additional_content(self, content):
        """ Create additional content to be added to the list of sub-views.
        This default implementation will always raise an exception.

        :param content:
            the content from which additional content can be created.
        :return:
            the additional content.
        """

        raise TypeError("the contents of a view must be either an instance of "
                "a ViewFactory sub-class or a string, not "
                "'{0}'".format(type(content).__name__))

    def finalize(self, item):
        """ Reimplemented to finalize all sub-views.

        :param item:
            is the view created by this factory.
        """

        # subviews() is recursive so only look at them if we are the top-level
        # call.
        if not hasattr(self, '_dip_finalizing'):
            for subview in ICompositeView(item).subviews():
                factory = IView(subview).factory

                factory._dip_finalizing = True
                factory.finalize(subview)
                del factory._dip_finalizing

        super(CompositeViewFactory, self).finalize(item)

    @ViewFactory.id.default
    def id(self):
        """ The default value of id.  This will be ignored if the view is not a
        top-level widget.
        """

        return 'dip.view'
