# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from dip.model import (implements, Instance, List, Model, observe,
        ValueTypeFactory)

from .i_controller import IController
from .i_editor import IEditor
from .i_message_area import IMessageArea


@implements(IController)
class Controller(Model):
    """ The Controller class is the default implementation of a
    :term:`controller`.
    """

    # The editor that is currently causing a change to the model.
    _changing_editor = Instance()

    # The editors in the view.
    _editors = List()

    def add_editor(self, editor):
        """ Add an editor to the controller.

        :param editor:
            is the editor.
        """

        self._editors.append(editor)

        ieditor = IEditor(editor)

        # Do any setup that is specific to types that represent a value.
        if isinstance(ieditor.attribute_type, ValueTypeFactory):
            # Set the initial value.
            model, name = self._inner_model(ieditor)
            ieditor.value = getattr(model, name)

            # Watch for any subsequent model changes.
            observe(name, model, lambda c: self._model_changed(ieditor, c))

    def editor_changed(self, editor):
        """ This is called by an editor when the user has changed its value.

        :param editor:
            is the editor.
        """

        ieditor = IEditor(editor)

        if isinstance(ieditor.attribute_type, ValueTypeFactory):
            self.validate()

            if self.invalid_reason == '' and self.auto_update_model:
                self.update_model()
        else:
            # Non-value types must update the model immediately.
            model, name = self._inner_model(ieditor)
            setattr(model, name, ieditor.value)

    def find_editor(self, id):
        """ Find the editor with a specific identifier.

        :param id:
            is the identifier of the editor.
        :return:
            the editor or ``None`` if the view has no such editor.
        """

        for editor in self._editors:
            if IEditor(editor).factory.id == id:
                return editor

        return None

    def update_model(self):
        """ Update the all the attributes of a model that have a corresponding
        editor.
        """

        for editor in self._editors:
            ieditor = IEditor(editor)

            if self._valid_editor(ieditor):
                model, name = self._inner_model(ieditor)

                # Prevent recursive changes.
                self._changing_editor = ieditor
                setattr(model, name, ieditor.value)
                self._changing_editor = None

    def validate(self):
        """ Validate the view and update the explanation as to why it is (or
        isn't) invalid.
        """

        self.invalid_reason = self.validate_view()

        for message_area in self.message_areas:
             IMessageArea(message_area).text = self.invalid_reason

    def validate_view(self):
        """ Validate the data in the view.  This implementation considers the
        view to be valid if all its editors have valid values.

        :return:
            a string which will be empty if the view is valid, otherwise it
            will explain why the view is invalid.
        """

        for editor in self._editors:
            ieditor = IEditor(editor)

            if self._valid_editor(ieditor):
                invalid_reason = ieditor.invalid_reason
                if invalid_reason != '':
                    invalid_reason = "{0}: {1}.".format(ieditor.label,
                            invalid_reason)
                    break
        else:
            invalid_reason = ''

        return invalid_reason

    def __getattr__(self, name):
        """ Reimplemented to get an instance attribute allowing for names in
        the form 'id_editor' where 'id' is the identifier of the editor.
        """

        if name.endswith('_editor'):
            id = name[:-len('_editor')]

            editor = self.find_editor(id)
            if editor is not None:
                return IEditor(editor)

        raise AttributeError(
                "'{0}' object has no attribute '{1}'".format(
                        self.__class__.__name__, name))

    @staticmethod
    def _valid_editor(ieditor):
        """ Return True if the editor is valid because we are interested in its
        value.
        """

        # Ignore disabled editors as they may legitimately contain invalid
        # values.
        return isinstance(ieditor.attribute_type, ValueTypeFactory) and ieditor.enabled

    def _inner_model(self, ieditor):
        """ Return the inner model and attribute name after resolving a
        possible attribute path.
        """

        model = self.model
        path = ieditor.factory.bind_to.split('.')

        for name in path[:-1]:
            model = getattr(model, name)

        return model, path[-1]

    def _model_changed(self, ieditor, change):
        """ Invoked when the attribute of the model for an editor changes. """

        # Don't do anything if this editor triggered the change.
        if self._changing_editor is not ieditor:
            ieditor.value = change.new

            self.validate()
