# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


import io

from PyQt4.QtCore import QFile, QIODevice
from PyQt4.uic import loadUiType

from dip.model import Instance, Str, Tuple

from .i_view import IView
from .bindings import Bindings
from .view_factory import ViewFactory


class Designer(ViewFactory):
    """ The Designer class allows any user interface created with Qt Designer
    to be used in a view.
    """

    # The bindings that connect the GUI to the model.
    bindings = Instance(Bindings)

    # The name of the Qt Designer .ui file.
    ui_file_name = Str()

    # The tuple of the UI "painter" and the base Qt type to "paint".
    _ui_factory = Tuple()

    def __init__(self, ui_file_name, **properties):
        """ Initialise the item.

        :param ui_file_name:
            is the name of the Qt Designer .ui file.
        :param \*\*properties:
            are keyword arguments used as property names and values that are
            applied to the widget.
        """

        super(Designer, self).__init__(**properties)

        # It's more natural to allow this as a positional argument.
        self.ui_file_name = ui_file_name

    def create_view(self, controller, toolkit, parent):
        """ Create the editor for the user interface.

        :param controller:
            is the controller.
        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation.
        :param parent:
            is the optional parent widget.
        :return:
            the editor that is the root of the user interface defined in the
            .ui file.
        """

        ui, base = self._ui_factory

        # Create an instance of the base class and "paint" it.
        view = base(parent=parent)
        ui.setupUi(view)

        # Bind to the model.
        if self.bindings is not None:
            self.bindings.bind(view, controller.model, toolkit, controller)

        self.bind(view, controller, toolkit)

        return view

    def create_subview(self, controller, toolkit, attribute_type):
        """ Create the editor for the user interface.

        :param controller:
            is the controller.
        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation.
        :param parent:
            is the optional parent widget.
        :param attribute_type:
            is ``None`` in this context.
        :return:
            the editor that is the root of the user interface defined in the
            .ui file.
        """

        return self.create_view(controller, toolkit, None)

    @_ui_factory.default
    def _ui_factory(self):
        """ Attribute initialiser. """

        # We use QFile in order to support Qt resource files.
        ui_file = QFile(self.ui_file_name)

        if not ui_file.open(QIODevice.ReadOnly):
            raise IOError("unable to open '{0}'".format(self.ui_file_name))

        ui_xml = io.StringIO(unicode(ui_file.readAll(), encoding='utf-8'))

        ui_file.close()

        ui, base = loadUiType(ui_xml)

        return ui(), base
