# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from dip.model import observe

from .i_collection_editor import ICollectionEditor
from .linked_view_factory import LinkedViewFactory


class DownButton(LinkedViewFactory):
    """ The DownButton class implements a factory for a button for moving an
    item down a collection editor.
    """

    # The suffix to add to the identifier to distinguish it from other linked
    # views.
    id_suffix = '.down'

    def create_view(self, controller, toolkit, parent):
        """ Create the view.
    
        :param controller:
            is the controller.
        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation used to create the
            view.
        :param parent:
            is the optional parent widget.
        :return:
            the view which will be an instance of
            :class:`~PyQt4.QtGui.QAbstractButton`.
        """

        view = toolkit.down_button(self, parent)
        self.bind(view, controller, toolkit)

        return view

    def link(self, view, editor):
        """ This is reimplemented to link the button view to the collection
        editor.

        :param view:
            is the view.
        :param editor:
            is the editor.
        """

        if not view.isEnabled():
            return

        ieditor = ICollectionEditor(editor)

        view.clicked.connect(
                lambda: ieditor.move_down(self._row_index(ieditor)))

        observe('selection', ieditor,
                lambda: view.setEnabled(self._row_index(ieditor) is not None))

        if self._row_index(ieditor) is None:
            view.setEnabled(False)

    @staticmethod
    def _row_index(ieditor):
        """ Return the index from the selection that refers to a single row.
        None is returned if there is no selection or it spans more than one
        row or it refers to the last row.
        """

        selection = ieditor.selection

        rows = set()
        for index in selection:
            rows.add(index.row())

        if len(rows) == 1:
            index = selection[0]

            if index.row() != ieditor.item_count - 1:
                return index

        return None
