# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtCore import Qt
from PyQt4.QtGui import QStyledItemDelegate

from .i_collection_editor import ICollectionEditor
from .i_editor import IEditor


class EditorDelegate(QStyledItemDelegate):
    """ The EditorDelegate class is an internal class that acts as a Qt item
    delegate for any editor.
    """

    def __init__(self, editor_factory, attribute_type, toolkit, parent):
        """ Initialise the delegate.

        :param editor_factory:
            the editor factory that will create the editor.
        :param attribute_type:
            the type of the value that the editor will be bound to.
        :param toolkit:
            the toolkit.
        :param parent:
            the parent widget.
        """

        super(EditorDelegate, self).__init__(parent)

        self._editor_factory = editor_factory
        self._attribute_type = attribute_type
        self._toolkit = toolkit

        self.closeEditor.connect(self._editor_closed)

    def createEditor(self, parent, option, index):
        """ Reimplemented to use the editor factory to create the delegated
        editor.
        """

        editor_factory = self._editor_factory

        editor = editor_factory.create_editor(self._attribute_type,
                self._toolkit, parent)

        # Prevent the selected value showing through.
        editor.setAutoFillBackground(True)

        ieditor = editor_factory.view_interface(editor)
        ieditor.factory = editor_factory
        ieditor.attribute_type = self._attribute_type

        ieditor.configure()

        return editor

    def setEditorData(self, editor, index):
        """ Reimplemented to set the delegated editor's value using the IEditor
        interface rather than requiring editors to implement a user property.
        """

        data = index.data(Qt.EditRole).toPyObject()
        if data is not None:
            IEditor(editor).value = data

    def setModelData(self, editor, model, index):
        """ Reimplemented to get the delegated editor's value using the IEditor
        interface rather than requiring editors to implement a user property.
        """

        model.setData(index, IEditor(editor).value, Qt.EditRole)

    def _editor_closed(self, editor, hint):
        """ Invoked when a delegated editor is closed. """

        # Note that this implementation means that a delegated editor does not
        # update the invalid reason until the editing is finished, unlike a
        # regular editor which updates the invalid reason as the user types.

        collection_editor = self.parent()
        icollection_editor = ICollectionEditor(collection_editor)

        invalid_reason = IEditor(editor).invalid_reason
        icollection_editor.item_invalid_reason(
                collection_editor.currentIndex(), invalid_reason)

        if icollection_editor.controller is not None:
            icollection_editor.controller.editor_changed(collection_editor)
