# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from dip.model import Bool, get_attribute_type, List, Str

from .editor_delegate import EditorDelegate
from .view_factory import ViewFactory


class EditorFactory(ViewFactory):
    """ The EditorFactory class is a factory for views that implement the
    :class:`~dip.ui.IEditor` interface.
    """

    # The name of the attribute that the editor is bound to.  This may be an
    # :term:`attribute path`.
    bind_to = Str()

    # The default list of Qt properties that should be set for the editor with
    # the values taken from the model attribute's meta-data.
    default_metadata_properties = List(Str(),
            default=['statusTip', 'toolTip', 'whatsThis'])

    # The list of extra Qt properties that should be set for the editor with
    # the values taken from the model attribute's meta-data.
    extra_metadata_properties = List(Str())

    # A short, user friendly label for the attribute.  It is typically used as
    # a field name or the text of a push button.  If not given then the
    # ``label`` value of the type's meta-data is used instead.  If that hasn't
    # been defined then a label based on the name of the attribute in its
    # containing model is used.
    label = Str()

    # This is set if the attribute is read-only.
    read_only = Bool(False)

    def __init__(self, bind_to='', **properties):
        """ Initialise the factory.

        :param bind_to:
            the name of the attribute that the editor is bound to.  It should
            only be an empty string if the factory is being used to create a
            delegate.
        :param \*\*properties:
            are keyword arguments used as property names and values that are
            applied to the editor.
        """

        super(EditorFactory, self).__init__(**properties)

        # It's more natural to allow this as a positional argument.
        self.bind_to = bind_to

    def create_view(self, controller, toolkit, parent):
        """ Create the view.

        :param controller:
            is the controller.
        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation used to create the
            view.
        :param parent:
            is the optional parent widget.
        :return:
            the view.
        """

        attribute_type = get_attribute_type(controller.model, self.bind_to)
        editor = self.create_editor(attribute_type, toolkit, parent)
        self.bind(editor, controller, toolkit, attribute_type)

        return editor

    def bind(self, view, controller, toolkit, attribute_type):
        """ Reimplemented to bind an editor view to a model attribute and add
        it to a controller.

        :param view:
            is the view.
        :param controller:
            is the controller.
        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation used to create the
            view.
        :param attribute_type:
            is the type of the attribute of a model the view is being bound to.
        """

        ieditor = self.view_interface(view)
        ieditor.attribute_type = attribute_type

        super(EditorFactory, self).bind(view, controller, toolkit)

        controller.add_editor(view)

    def configure_editor(self, editor, attribute_type):
        """ Configure an editor using the factory's properties and any
        additional properties supplied when the factory was created.

        :param editor:
            is the editor.  Note that this isn't necessarily the
            :class:`~dip.ui.IEditor` implementation but may be an internal
            widget that the properties are to be applied to.
        :param attribute_type:
            is the type of the attribute in the model the editor is bound to.
        """

        properties = self.properties

        for name in self.default_metadata_properties + self.extra_metadata_properties:
            if name not in properties:
                value = attribute_type.metadata.get(name)
                if value is not None:
                    # Copy on write the properties.
                    if properties is self.properties:
                        properties = dict(self.properties)

                    properties[name] = value

        self.configure_properties(editor, properties)

    def create_editor(self, attribute_type, toolkit, parent):
        """ Create the :class:`~PyQt4.QtGui.QLayout` or
        :class:`~PyQt4.QtGui.QWidget` that implements the editor
        and can be adapted to :class:`~dip.ui.IEditor`.

        This must be reimplemented by a sub-class.

        :param attribute_type:
            is the type of the attribute in the model that the editor is bound
            to.
        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation used to create the
            editor.
        :param parent:
            is the optional parent widget.
        :return:
            the :class:`~PyQt4.QtGui.QLayout` or :class:`~PyQt4.QtGui.QWidget`
            that is the root of the editor's implementation.
        """

        raise NotImplementedError

    def set_delegate(self, item_view, delegate_type, toolkit, column_nr=0):
        """ Create a delegate and set it for a column of a
        :class:`~PyQt4.QtGui.QAbstractItemView`.

        :param item_view:
            is the :class:`~PyQt4.QtGui.QAbstractItemView`.
        :param delegate_type:
            is the type of the value that the delegate will edit.
        :param column_nr:
            is the number of the column.
        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation used to create the
            editor.
        """

        delegate = EditorDelegate(self, delegate_type, toolkit, item_view)
        item_view.setItemDelegateForColumn(column_nr, delegate)

    @classmethod
    def editor_as_delegate(cls, delegate_type, toolkit):
        """ Create an editor factory to be used as a delegate for a type.

        :param delegate_type:
            is the type of the value that the delegate will edit.
        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation that provides the
            mapping between types and delegates.
        :return:
            the editor factory.
        """

        return toolkit.delegate_for_attribute(delegate_type)()

    def get_attribute_label(self, attribute_type):
        """ Get the label for an attribute.  This will first try the factory
        itself, then the type's meta-data and finally the type's name.

        :param attribute_type:
            is the attribute type.
        :return:
            the label.
        """

        label = self.label

        if label == "":
            try:
                label = attribute_type.metadata['label']
            except KeyError:
                # There won't be a name if the type is of an element of a
                # collection
                if attribute_type.name is not None:
                    label = self.get_natural_name(attribute_type.name)

        return label

    @classmethod
    def get_natural_name(cls, name):
        """ Convert a name, typically of an attribute, to a more natural name.

        :param name:
            is the original name.
        :return:
            the more natural version.
        """

        name = name.replace('_', ' ')
        name = name.strip()
        name = name[0].upper() + name[1:]

        return name

    @ViewFactory.id.default
    def id(self):
        """ The default value of id is the name of the attribute that the
        editor is bound to is used.  The identifier only needs to be explicitly
        set if there is more than one editor in the same view that is bound to
        the same attribute and the editor is the target of a linked editor.
        """

        return self.bind_to
