# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from dip.model import Interface


class IToolkit(Interface):
    """ The IToolkit interface defines the API to be implemented by a
    :term:`toolkit`.
    """

    def add_button(self, factory, parent):
        """ Create a button to add an item to a collection.

        :param factory:
            is the factory creating the button.
        :param parent:
            is the optional parent widget.
        :return:
            the button which will be a :class:`~PyQt4.QtGui.QAbstractButton`.
        """

    def check_box(self, factory, attribute_type, parent):
        """ Create an editor based on a check box.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor.
        """

    def combo_box(self, factory, attribute_type, parent):
        """ Create an editor based on a combo box.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor.
        """

    def delegate_for_attribute(self, attribute_type):
        """ Return an :class:`~dip.ui.EditorFactory` type object that will
        create a delegate factory to handle attributes of a particular type.

        :param attribute_type:
            is the type of the attribute.
        :return:
            the type object.
        """

    def dialog(self, factory, content, parent):
        """ Create a dialog view.

        :param factory:
            is the factory creating the view.
        :param content:
            is the content of the dialog and is either a
            :class:`~PyQt4.QtGui.QLayout` or a :class:`~PyQt4.QtGui.QWidget`.
        :param parent:
            is the optional parent widget.
        :return:
            the view.
        """

    def down_button(self, factory, parent):
        """ Create a button to move an item in a collection down.

        :param factory:
            is the factory creating the button.
        :param parent:
            is the optional parent widget.
        :return:
            the button which will be a :class:`~PyQt4.QtGui.QAbstractButton`.
        """

    def editor_for_attribute(self, attribute_type):
        """ Return an :class:`~dip.ui.EditorFactory` type object that will
        create an editor factory to handle attributes of a particular type.

        :param attribute_type:
            is the type of the attribute.
        :return:
            the type object.
        """

    def editor_for_widget(self, widget_type):
        """ Return an :class:`~dip.ui.EditorFactory` type object that will
        create an editor factory that creates editors of a particular type.

        :param widget_type:
            is the type of the widget.
        :return:
            the type object.
        """

    def file_selector(self, factory, attribute_type, parent):
        """ Create an editor that implements a file/directory selector.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor.
        """

    def float_spin_box(self, factory, attribute_type, parent):
        """ Create an editor for floats based on a spin box.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor.
        """

    def form(self, factory, items):
        """ Create a view to contain a number of items arranged as a form.

        :param factory:
            is the optional factory creating the view.
        :param items:
            is a sequence of 2-tuple label and field pairs.  A label may be
            ``None``.  A field may be a :class:`~PyQt4.QtGui.QLayout` or a
            :class:`~PyQt4.QtGui.QWidget`.
        :return:
            the view.
        """

    def get_directory(self, directory='', parent=None):
        """ Get the name of an existing directory from the user.

        :param directory:
            is the name of an optional initial directory.
        :param parent:
            is the optional parent widget.
        :return:
            the name of the directory or an empty string if there was none.
        """

    def get_open_file(self, directory='', filter='', parent=None):
        """ Get the name of a file to open from the user.

        :param directory:
            is the name of an optional initial directory or file.
        :param filter:
            is the optional file filter.
        :param parent:
            is the optional parent widget.
        :return:
            the name of the file to open or an empty string if there was none.
        """

    def get_open_files(self, directory='', filter='', parent=None):
        """ Get a list of names of files to open from the user.

        :param directory:
            is the name of an optional initial directory or file.
        :param filter:
            is the optional file filter.
        :param parent:
            is the optional parent widget.
        :return:
            the list of names.
        """

    def get_save_file(self, directory='', filter='', parent=None):
        """ Get the name of a file to save from the user.

        :param directory:
            is the name of an optional initial directory or file.
        :param filter:
            is the optional file filter.
        :param parent:
            is the optional parent widget.
        :return:
            the name of the file to save or an empty string if there was none.
        """

    def group_box(self, factory, content, parent):
        """ Create a group box view.

        :param factory:
            is the factory creating the view.
        :param content:
            is the content of the view and is either a
            :class:`~PyQt4.QtGui.QLayout` or a :class:`~PyQt4.QtGui.QWidget`.
        :param parent:
            is the optional parent widget.
        :return:
            the view.
        """

    def h_box(self, factory, items):
        """ Create a view to contain a number of items arranged horizontally.

        :param factory:
            is the factory creating the view.
        :param items:
            is a sequence of :class:`~PyQt4.QtGui.QLayout`,
            :class:`~PyQt4.QtGui.QWidget` and :class:`~dip.ui.Stretch`
            instances.
        :return:
            the view.
        """

    def information(self, title, text, parent=None):
        """ Display a informational message to the user.

        :param title:
            is the title, typically used as the title of a dialog.
        :param text:
            is the text of the message.
        :param parent:
            is the optional parent widget.
        """

    def line_edit(self, factory, attribute_type, parent):
        """ Create an editor based on a line editor.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor.
        """

    def list_widget(self, factory, attribute_type, parent):
        """ Create an editor for a list.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor.
        """

    def menu(self, title, parent):
        """ Create a menu.

        :param title:
            is the menu's title.
        :param parent:
            is the menu's parent.
        :return:
            the menu which will be an instance of :class:`~PyQt4.QtGui.QMenu`.
        """

    def menu_bar(self):
        """ Create a menu bar.

        :return:
            the menu bar which will be an instance of
            :class:`~PyQt4.QtGui.QMenuBar`.
        """

    def message_area(self, factory, parent):
        """ Create a message area.

        :param factory:
            is the factory creating the message area.
        :param parent:
            is the optional parent widget.
        :return:
            the message area which will be an instance of
            :class:`~PyQt4.QtGui.QLabel`.
        """

    def push_button(self, factory, attribute_type, parent):
        """ Create an editor based on a push button.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor.
        """

    def question(self, title, text, parent=None, default_no=True):
        """ Ask the user a question with a yes or no answer.

        :param title:
            is the title, typically used as the title of a dialog.
        :param text:
            is the text of the question.
        :param parent:
            is the optional parent widget.
        :param default_no:
            is ``True`` if the default answer to the question is "no".
        :return:
            ``True`` if the answer was "yes".
        """

    def radio_buttons(self, factory, attribute_type, parent):
        """ Create an editor that implements a set of radio buttons.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor.
        """

    def remove_button(self, factory, parent):
        """ Create a button to add an item to a collection.

        :param factory:
            is the factory creating the button.
        :param parent:
            is the optional parent widget.
        :return:
            the button which will be a :class:`~PyQt4.QtGui.QAbstractButton`.
        """

    def spin_box(self, factory, attribute_type, parent):
        """ Create an editor based on a spin box.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor.
        """

    def splitter(self, factory, items):
        """ Create a splitter view to contain a number of items.

        :param factory:
            is the factory creating the view.
        :param items:
            is a sequence of :class:`~PyQt4.QtGui.QLayout` or
            :class:`~PyQt4.QtGui.QWidget` instances.
        :return:
            the view.
        """

    def table_widget(self, factory, attribute_type, parent):
        """ Create an editor for a table.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor which will be an instance of
            :class:`~PyQt4.QtGui.QTableWidget`.  The properties will be applied
            to the editor itself.
        """

    def text_edit(self, factory, attribute_type, parent):
        """ Create an editor based on a text editor.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor.
        """

    def tool_button(self, factory, attribute_type, parent):
        """ Create an editor based on a tool button.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor.
        """

    def up_button(self, factory, parent):
        """ Create a button to move an item in a collection up.

        :param factory:
            is the factory creating the button.
        :param parent:
            is the optional parent widget.
        :return:
            the button which will be a :class:`~PyQt4.QtGui.QAbstractButton`.
        """

    def v_box(self, factory, items):
        """ Create a view to contain a number of items arranged vertically.

        :param factory:
            is the factory creating the view.
        :param items:
            is a sequence of :class:`~PyQt4.QtGui.QLayout`,
            :class:`~PyQt4.QtGui.QWidget` and :class:`~dip.ui.Stretch`
            instances.
        :return:
            the view.
        """

    def warning(self, title, text, parent=None, detail=''):
        """ Display a warning message to the user.

        :param title:
            is the title, typically used as the title of a dialog.
        :param text:
            is the text of the warning.
        :param parent:
            is the optional parent widget.
        :param detail:
            is the optional additional detail.
        """

    def wizard(self, factory, pages, parent):
        """ Create a wizard view.

        :param factory:
            is the factory creating the view.
        :param pages:
            is the list of wizard pages.
        :param parent:
            is the optional parent widget.
        :return:
            the view.
        """

    def wizard_page(self, factory, content, parent):
        """ Create a wizard page view.

        :param factory:
            is the factory creating the view.
        :param content:
            is the content of the view and is either a
            :class:`~PyQt4.QtGui.QLayout` or a :class:`~PyQt4.QtGui.QWidget`.
        :param parent:
            is the optional parent widget.
        :return:
            the view.
        """
