# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtGui import QLayout

from dip.model import adapt, isadapted

from .i_composite_view import ICompositeView
from .i_view import IView
from .base_view_adapter import BaseViewAdapter
from .composite_view_factory import CompositeViewFactory


class LayoutFactory(CompositeViewFactory):
    """ The LayoutFactory class is the base class for all factories that create
    views that are implemented as :class:`~PyQt4.QtGui.QLayout` instances.
    """

    def create_view(self, controller, toolkit, parent):
        """ Reimplemented to ensure the view is a
        :class:`~PyQt4.QtGui.QWidget`.

        :param controller:
            is the controller.
        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation used to create the
            view.
        :param parent:
            is the optional parent widget.
        :return:
            the :class:`~PyQt4.QtGui.QWidget` that is the root of the view's
            implementation.
        """

        subviews = self.create_subviews(controller, toolkit, parent)
        layout = self.create_layout(subviews, toolkit)
        self.bind(layout, controller, toolkit)

        return layout

    def create_layout(self, items, toolkit):
        """ Create the specific :class:`~PyQt4.QtGui.QLayout` instance.

        This must be reimplemented by a sub-class

        :param items:
            is a sequence of :class:`~PyQt4.QtGui.QLayout`,
            :class:`~PyQt4.QtGui.QWidget` and :class:`~dip.ui.Stretch`
            instances.
        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation used to create the
            layout.
        :return:
            the layout.
        """

        raise NotImplementedError


@adapt(QLayout, to=ICompositeView)
class QLayoutICompositeViewAdapter(BaseViewAdapter):
    """ Adapt any QLayout sub-class to the ICompositeView interface. """

    def subviews(self):
        """ Return a sequence of sub-views, i.e. instances of
        :class:`~PyQt4.QtGui.QLayout` or :class:`~PyQt4.QtGui.QWidget` that
        implement or have been adapted to :class:`~dip.ui.IView`.

        :return:
            the sequence of sub-views.
        """

        layout = self.adaptee

        subviews = []

        for idx in range(layout.count()):
            item = layout.itemAt(idx)

            widget = item.widget()
            if widget is not None:
                next_view = widget
            else:
                sub_layout = item.layout()
                if sub_layout is not None:
                    next_view = sub_layout
                else:
                    continue

            # If it is being used as a composite view then we know how to
            # descend.
            if isadapted(next_view, ICompositeView):
                subviews.extend(ICompositeView(next_view).subviews())
            elif isadapted(next_view, IView):
                subviews.append(next_view)

        return subviews
