# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from dip.model import Str

from .i_editor import IEditor
from .i_view import IView
from .view_factory import ViewFactory


class LinkedViewFactory(ViewFactory):
    """ The LinkedViewFactory class is a base class for factories for views
    that are linked to editors and, in some way, supplement the behaviour of
    the editor being linked to.
    """

    # The suffix to add to the identifier to distinguish it from other views
    # linked to the same editor.
    id_suffix = Str()

    # The identifier of the editor that this view is linked to.
    linked_to = Str()

    def __init__(self, linked_to, **properties):
        """ Initialise the factory.

        :param linked_to:
            the identifier of the editor that the view is linked to.
        :param \*\*properties:
            are keyword arguments used as property names and values that are
            applied to the view.
        """

        super(LinkedViewFactory, self).__init__(**properties)

        # It's more natural to allow this as a positional argument.
        self.linked_to = linked_to

    def finalize(self, item):
        """ This is reimplemented to disable the view created by this factory
        if the linked editor is read-only.

        :param item:
            is the view.
        """

        super(LinkedViewFactory, self).finalize(item)

        editor = IView(item).controller.find_editor(self.linked_to)
        ieditor = IEditor(editor)

        if ieditor.factory.read_only or not ieditor.enabled:
            item.setEnabled(False)

        self.link(item, editor)

    def link(self, view, editor):
        """ This is normally reimplemented to link the view created by this
        factory to an editor once everything else has been set up.

        :param view:
            is the view.
        :param editor:
            is the editor.
        """

    @ViewFactory.id.default
    def id(self):
        """ The default value of id. """

        return self.linked_to + self.id_suffix
