# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtCore import Qt

from dip.model import Any, Callable, Instance, Str

from .i_list_widget_editor import IListWidgetEditor
from .editor_factory import EditorFactory
from .list_column import ListColumn


class ListWidget(EditorFactory):
    """ The ListWidget class implements a list widget editor factory for list
    attributes.
    """

    # The column.
    column = Instance(ListColumn)

    # When called this will create the data for a new item in the list.  If a
    # factory isn't provided then a value based on the default value of the
    # type of the list's items.
    item_factory = Callable()

    # The specific interface that the view implements.
    view_interface = IListWidgetEditor

    def create_editor(self, attribute_type, toolkit, parent):
        """ Create the editor.

        :param attribute_type:
            the type of the attribute the factory is being asked to handle.  It
            should be a :class:`~dip.model.List`.
        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation used to create the
            editor.
        :param parent:
            is the optional parent widget.
        :return:
            the editor.
        """

        return toolkit.list_widget(self, attribute_type, parent)

    def create_item(self, attribute_type):
        """ Create a new item for the list.

        :param attribute_type:
            is the list's type.
        :return:
            the new item.
        """

        if self.item_factory is not None:
            value = self.item_factory()
        else:
            value = self._column_type(attribute_type).get_default_value()

        return value

    def configure_delegate(self, editor, attribute_type, toolkit):
        """ Configure the delegate for an editor created by the factory.

        :param editor:
            is the editor.
        :param attribute_type:
            is the list's type.
        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation used to create the
            editor.
        """

        delegate_type = self._column_type(attribute_type)

        if isinstance(delegate_type, Any):
            delegate_type = Str()

        editor_factory = self.column.editor_factory

        if editor_factory is None:
            editor_factory = self.editor_as_delegate(delegate_type, toolkit)

        editor_factory.set_delegate(editor, delegate_type, toolkit)

    def configure_widget_item(self, widget_item, attribute_type):
        """ Configure a new widget item.

        :param widget_item:
            is the widget item, usually a :class:`~PyQt4.QtGui.QListWidgetItem`
            instance.
        :param attribute_type:
            is the list's type.
        """

        if not self.read_only:
            widget_item.setFlags(widget_item.flags() | Qt.ItemIsEditable)

        # Apply any help related meta-data giving priority to the column
        # definition over the attribute type.
        if self.column.column_type is not None:
            self._apply_help(widget_item, self.column.column_type)

        self._apply_help(widget_item, attribute_type)

    @column.default
    def column(self):
        """ Return the default column. """

        return ListColumn()

    @staticmethod
    def _apply_help(widget_item, attribute_type):
        """ Set the standard help related properties from a type's meta-data.
        """

        metadata = attribute_type.metadata

        if widget_item.statusTip() == '':
            widget_item.setStatusTip(metadata.get('statusTip', ''))

        if widget_item.toolTip() == '':
            widget_item.setToolTip(metadata.get('toolTip', ''))

        if widget_item.whatsThis() == '':
            widget_item.setWhatsThis(metadata.get('whatsThis', ''))

    def _column_type(self, attribute_type):
        """ Get the type of the column. """

        col_type = attribute_type.element_type

        if isinstance(col_type, Any):
            if self.column.column_type is not None:
                col_type = self.column.column_type

        return col_type
