# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from dip.model import Instance

from .action_collection import ActionCollection
from .base_menu import BaseMenu


class Menu(BaseMenu):
    """ The Menu class is a container for a list of actions that is
    implemented as a :class:`~PyQt4.QtGui.QMenu`.
    """

    # The action collection passed to __init__.
    _collection = Instance(ActionCollection)

    def __init__(self, collection, **properties):
        """ Initialise the menu from an action collection.

        :param collection:
            is the action collection.
        :param \*\*properties:
            are keyword arguments used as property names and values that are
            applied to a menu.
        """

        super(Menu, self).__init__(**properties)

        # It's more natural to allow this as a positional argument.
        self._collection = collection

    def __call__(self, toolkit, *actions, **_kwargs):
        """ Create the menu adding any actions that have appropriate
        identifiers.

        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implmentation used to create the
            menu.
        :param \*actions:
            the list of actions to be added.
        :param parent:
            the optional parent of the menu.
        :return:
            the :class:`~PyQt4.QtGui.QMenu` instance.
        """

        parent = _kwargs.pop('parent', None)

        if len(_kwargs) > 0:
            raise TypeError(
                    "__call__() got an unexpected keyword argument '{0}'".format(
                            _kwargs.keys()[0]))

        # Qt is smart about handling hidden menus, so we create all the menu
        # items and just hide them until we are given corresponding actions.
        # Menus are created for top-level collections and for sub-collections
        # that have titles.  Sub-collections without titles are placed in the
        # same menu as their siblings with separators in between.

        menu = toolkit.menu(self._collection.title, parent)

        self.configure_properties(menu)

        menu_action = menu.menuAction()
        menu_action.setObjectName(self._collection.id)
        menu_action.setVisible(False)

        self._insert_collection_actions(self._collection, menu, toolkit)

        for action in actions:
            self.add(action, menu, toolkit, within=self._collection.within,
                    before=self._collection.before)

        self.finalize(menu)

        return menu
