# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from dip.model import List

from .action_collection import ActionCollection
from .base_menu import BaseMenu
from .menu import Menu


class MenuBar(BaseMenu):
    """ The MenuBar class is a container for a list of menus that is
    implemented as a :class:`~PyQt4.QtGui.QMenuBar`.
    """

    # The list of menus passed to __init__.
    _menus = List(Menu)

    def __init__(self, *menus, **properties):
        """ Initialise the menu bar.

        :param \*menus:
            is the list of menus.  A menu may be an instance of either
            :class:`~dip.ui.Menu` or :class:`~dip.ui.ActionCollection`.
        :param \*\*properties:
            are keyword arguments used as property names and values that are
            applied to a menu bar.
        """

        super(MenuBar, self).__init__(**properties)

        # Copy the list of menus converting action collections as we go.
        self._menus = [Menu(itm) if isinstance(itm, ActionCollection) else itm
                for itm in menus]

    def __call__(self, toolkit, *actions):
        """ Create the menu bar adding any actions that have appropriate
        identifiers.

        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation used to create the
            menu bar.
        :param \*actions:
            the list of actions to be added.
        :return:
            the :class:`~PyQt4.QtGui.QMenuBar` instance.
        """

        menu_bar = toolkit.menu_bar()

        self.configure_properties(menu_bar)

        for menu in self._menus:
            qmenu = menu(toolkit, *actions, parent=menu_bar)
            menu_bar.addMenu(qmenu)

        self.finalize(menu_bar)

        return menu_bar
