# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from dip.model import Model, Str


class QObjectFactory(Model):
    """ The QObjectFactory class is the base class for all factories that
    can create a :class:`~PyQt4.QtCore.QObject` whose properties can be
    configured and that support the concept of an identifier.
    """

    # The identifier of the item that is created.  If it is not specified then
    # a sub-class may implement an appropriate default.  The item's object
    # name will default to the identifier.
    id = Str()

    def __init__(self, **properties):
        """ Initialize the item.

        :param \*\*properties:
            are keyword arguments used as property names and values that are
            applied to the :class:`~PyQt4.QtCore.QObject` instances created by
            the factory.
        """

        self.properties = properties

    def __call__(self):
        """ Create the :class:`~PyQt4.QtCore.QObject` instance.  This should be
        reimplemented by a sub-class

        :return:
            the object.
        """

        raise NotImplementedError

    def finalize(self, item):
        """ This is called once all similar items have been created.  This
        default implementation will set the item's object name to the factory's
        identifier if it doesn't already have a name.

        :param item:
            is the item that was created by this factory.
        """

        if item.objectName() == '':
            item.setObjectName(self.id)

    def configure_properties(self, item, properties=None):
        """ Configure an item's properties.

        :param item:
            is the item whose properties are being set.
        :param properties:
            is the dictionary of property names and values.  If it is ``None``
            then the properties supplied when the factory was created.
        """

        if properties is None:
            properties = self.properties

        try:
            item.pyqtConfigure(**properties)
        except AttributeError:
            pass
