# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtCore import Qt

from dip.model import (Any, Callable, get_attribute_type, get_model_types,
        Instance, List, MetaModel, Model)

from .i_table_widget_editor import ITableWidgetEditor
from .editor_factory import EditorFactory
from .table_column import TableColumn


class TableWidget(EditorFactory):
    """ The TableWidget class implements a table widget editor factory for list
    attributes.
    """

    # The list of columns.  If a string is given then it is assumed to be the
    # name of an attribute in the row model.
    columns = List(Instance(TableColumn, str))

    # When called this will create the data for a new row in the table.
    row_factory = Callable()

    # The specific interface that the view implements.
    view_interface = ITableWidgetEditor

    def create_editor(self, attribute_type, toolkit, parent):
        """ Create the editor.

        :param attribute_type:
            the type of the attribute the factory is being asked to handle.  It
            should be a :class:`~dip.model.List`.
        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation used to create the
            editor.
        :param parent:
            is the optional parent widget.
        :return:
            the editor.
        """

        return toolkit.table_widget(self, attribute_type, parent)

    def create_row(self, attribute_type, columns):
        """ Create a new row for the table.

        :param attribute_type:
            is the list's type.
        :param columns:
            is the list of :class:`~dip.ui.TableColumn` instances for the
            columns.
        :return:
            the new row.
        """

        if self.row_factory is not None:
            row = self.row_factory()
        else:
            row_type = attribute_type.element_type

            if issubclass(row_type, Model):
                row = row_type()
            else:
                row = []

                for col in columns:
                    col_type = self._column_type(attribute_type, col)
                    row.append(col_type.get_default_value())

        return row

    def configure_delegate(self, editor, attribute_type, column, column_nr, toolkit):
        """ Configure the delegate for a column of an editor created by the
        factory.

        :param editor:
            is the editor.
        :param attribute_type:
            is the list's type.
        :param column:
            is the :class:`~dip.ui.TableColumn` instance for the column.
        :param column_nr:
            is the number of the column
        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation used to create the
            editor.
        """

        if column.editable:
            delegate_type = self._column_type(attribute_type, column)

            if isinstance(delegate_type, Any):
                delegate_type = Str()

            editor_factory = column.editor_factory

            if editor_factory is None:
                editor_factory = self.editor_as_delegate(delegate_type,
                        toolkit)

            editor_factory.set_delegate(editor, delegate_type, toolkit,
                    column_nr)

    def table_columns(self, attribute_type, nr_columns):
        """ Return the list of :class:`~dip.ui.TableColumn` instances
        describing the columns of the table.

        :param attribute_type:
            is the type of the attribute the editor is bound to.
        :param nr_columns:
            is the number of columns that the editor has.  It is 0 if this is
            determined by the value returned by this method.
        :return:
            the list of columns.
        """

        # See what we have a list of.
        row_type = attribute_type.element_type

        if isinstance(row_type, MetaModel):
            model_types = get_model_types(row_type)

            if len(self.columns) == 0:
                # Create a column for each attribute of the model.
                columns = [TableColumn(bind_to=name)
                        for name in model_types.keys()]
            else:
                columns = []

                # Convert strings to columns.
                for col in self.columns:
                    if isinstance(col, str):
                        col = TableColumn(bind_to=col)

                    if col.bind_to not in model_types:
                        raise AttributeError(
                                "'{0}' has no attribute '{1}'".format(
                                        row_type.__name__, col.bind_to))

                    columns.append(col)

        elif isinstance(row_type, List):
            if len(self.columns) == 0:
                if nr_columns == 0:
                    raise AttributeError(
                        "unable to determine the number of columns in the "
                        "table unless 'columns' is specified")

                # Create default columns.
                columns = [TableColumn() for _ in range(nr_columns)]
            else:
                # Check that no columns are strings.
                for col in self.columns:
                    if isinstance(col, str):
                        raise AttributeError(
                                "'columns' may not contain strings unless the "
                                "elements of the List are a Model")

                columns = self.columns

        else:
            raise TypeError(
                    "the elements of the List must be either an Interface, "
                    "Model or another List, not {0}".format(row_type.__name__))

        # Check the number of columns is compatible.
        if nr_columns > 0 and nr_columns != len(columns):
            raise AttributeError(
                    "{0} columns have been defined but the editor has {1} "
                    "columns".format(len(columns), nr_columns))

        return columns

    def configure_widget_item(self, widget_item, attribute_type, column):
        """ Configure a new widget item.

        :param widget_item:
            is the widget item, usually a
            :class:`~PyQt4.QtGui.QTableWidgetItem` instance.
        :param attribute_type:
            is the list's type.
        """

        if not self.read_only and column.editable:
            widget_item.setFlags(widget_item.flags() | Qt.ItemIsEditable)

        # Apply any help related meta-data giving priority to the column
        # definition over the attribute type.
        self._apply_help(widget_item,
                self._column_type(attribute_type, column))
        self._apply_help(widget_item, attribute_type)

    @staticmethod
    def _apply_help(widget_item, attribute_type):
        """ Set the standard help related properties from a type's meta-data.
        """

        metadata = attribute_type.metadata

        if widget_item.statusTip() == '':
            widget_item.setStatusTip(metadata.get('statusTip', ''))

        if widget_item.toolTip() == '':
            widget_item.setToolTip(metadata.get('toolTip', ''))

        if widget_item.whatsThis() == '':
            widget_item.setWhatsThis(metadata.get('whatsThis', ''))

    def _column_type(self, attribute_type, column):
        """ Get the type of a column. """

        row_type = attribute_type.element_type

        if column.bind_to != '':
            col_type = get_model_types(row_type)[column.bind_to]
        else:
            col_type = row_type.element_type

            if isinstance(col_type, Any):
                if column.column_type is not None:
                    col_type = column.column_type

        return col_type
