# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtCore import Qt
from PyQt4.QtGui import QComboBox
from PyQt4.QtTest import QTest

from dip.automate import IAutomatedEditor
from dip.model import adapt, Adapter, isadapted
from dip.ui import BaseEditorAdapter, IComboBoxEditor, IEditor


@adapt(QComboBox, to=IComboBoxEditor)
class QComboBoxIComboBoxEditorAdapter(BaseEditorAdapter):
    """ An adapter to implement IEditor for the widget. """

    def configure(self):
        """ Configure the editor. """

        widget = self.adaptee

        # If the combo box already contains items then check there are the
        # right number of them.  Otherwise create the items.
        members = self.attribute_type.members
        nr_items = widget.count()

        if nr_items == 0:
            blocked = widget.blockSignals(True)
            widget.addItems(
                    [self.factory.get_natural_name(member)
                            for member in members])
            widget.blockSignals(blocked)
        else:
            nr_members = len(members)
            if nr_members != nr_items:
                raise TypeError(
                        "Enum has {0} members but the QComboBox contains {1} "
                        "items".format(nr_members, nr_items))

        if self.factory.read_only:
            widget.setEnabled(False)

        super(QComboBoxIComboBoxEditorAdapter, self).configure()

    @IComboBoxEditor.value.getter
    def value(self):
        """ The getter for the editor's value. """

        return self.attribute_type.members[self.adaptee.currentIndex()]

    @value.setter
    def value(self, value):
        """ The setter for the editor's value. """

        widget = self.adaptee

        blocked = widget.blockSignals(True)
        widget.setCurrentIndex(self.attribute_type.members.index(value))
        widget.blockSignals(blocked)

    @BaseEditorAdapter.change_signal.getter
    def change_signal(self):
        """ Get the editor's change signal. """

        return self.adaptee.currentIndexChanged


@adapt(QComboBox, to=IAutomatedEditor)
class QComboBoxIAutomatedEditorAdapter(Adapter):
    """ An adapter to implement IAutomatedEditor for the widget. """

    def simulate_set(self, id, delay, value):
        """ Simulate the user setting the editor value.

        :param id:
            is the (possibly scoped) identifier of the widget.
        :param delay:
            is the delay in milliseconds between simulated events.
        :param value:
            is the value to set.  If the QComboBox implements IEditor then the
            value must be valid for the attribute type.  Otherwise the value
            must be either an integer index of the item or the text of the
            item.
        """

        widget = self.adaptee

        if isadapted(widget, IEditor):
            try:
                to_idx = IEditor(widget).attribute_type.members.index(value)
            except ValueError:
                to_idx = -1
        elif isinstance(value, int):
            to_idx = value if value < widget.count() else -1
        else:
            to_idx = widget.findText(value)

        if to_idx < 0:
            raise ValueError(
                    "{0} is an invalid value for the QComboBox '{1}'".format(
                            value, id))

        from_idx = widget.currentIndex()

        if from_idx < to_idx:
            for _ in range(from_idx, to_idx):
                QTest.keyClick(widget, Qt.Key_Down, Qt.NoModifier, delay)
        elif to_idx < from_idx:
            for _ in range(to_idx, from_idx):
                QTest.keyClick(widget, Qt.Key_Up, Qt.NoModifier, delay)
