# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtCore import QPoint, Qt
from PyQt4.QtGui import QDialog, QDialogButtonBox
from PyQt4.QtTest import QTest

from dip.automate import AutomationError, IAutomatedDialog
from dip.model import adapt, Adapter, observe
from dip.ui import BaseViewAdapter, ICompositeView


@adapt(QDialog, to=ICompositeView)
class QDialogICompositeViewAdapter(BaseViewAdapter):
    """ An adapter to implement ICompositeView for the Qt view. """

    def configure(self):
        """ Configure the view.  This is called when everything else has been
        set up.
        """

        dialog = self.adaptee

        # Find the button box.
        button_box = dialog.findChild(QDialogButtonBox)
        if button_box is not None:
            button_box.accepted.connect(self._on_accept)
            button_box.rejected.connect(dialog.reject)

            # Any Ok button is only enabled when the view is valid.
            ok_button = button_box.button(QDialogButtonBox.Ok)
            ok_button.setEnabled(self.controller.invalid_reason == '')
            observe('invalid_reason', self.controller,
                    lambda c: ok_button.setEnabled(c.new == ''))

        # Do the standard configuration.
        super(QDialogICompositeViewAdapter, self).configure()

    def subviews(self):
        """ Return a sequence of sub-views, i.e. instances of
        :class:`~PyQt4.QtGui.QLayout` or :class:`~PyQt4.QtGui.QWidget` that
        implement or have been adapted to :class:`~dip.ui.IView`.

        :return:
            the sequence of sub-views.
        """

        layout = self.adaptee.layout()

        return [] if layout is None else ICompositeView(layout).subviews()

    def _on_accept(self):
        """ Invoked when the dialog is accepted. """

        # Update the model if it wasn't being done automatically.
        if not self.controller.auto_update_model:
            self.controller.update_model()

        self.adaptee.accept()


@adapt(QDialog, to=IAutomatedDialog)
class QDialogIAutomatedDialogAdapter(Adapter):
    """ An adapter to implement IAutomatedDialog for the Qt dialog. """

    def simulate_click(self, id, delay, button):
        """ Simulate the user clicking a button.

        :param id:
            is the (possibly scoped) identifier of the widget.
        :param delay:
            is the delay in milliseconds between simulated events.
        :param button:
            is the :class:`~PyQt4.QtGui.QDialogButtonBox.StandardButton` to
            click.
        """

        # Find the button widget in the button box.
        button_box = self.adaptee.findChild(QDialogButtonBox)

        if button_box is not None:
            button_widget = button_box.button(button)
        else:
            button_widget = None

        if button_widget is None:
            raise AutomationError(id, 'click',
                    "no {0} button in dialog".format(hex(button)))

        QTest.mouseClick(button_widget, Qt.LeftButton, Qt.NoModifier, QPoint(),
                delay)
