# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtCore import Qt
from PyQt4.QtGui import (QAbstractItemView, QApplication, QListWidget,
        QListWidgetItem)
from PyQt4.QtTest import QTest

from dip.automate import (AutomationError, IAutomatedEditor,
        IAutomatedListEditor)
from dip.model import adapt, Adapter, List
from dip.ui import BaseEditorAdapter, IListWidgetEditor


@adapt(QListWidget, to=IListWidgetEditor)
class QListWidgetIListWidgetEditorAdapter(BaseEditorAdapter):
    """ An adapter to implement IEditor for the Qt editor. """

    # The list of reasons why the editor is invalid.  Each element is a 2-tuple
    # of the row and reason.
    _invalid_reasons = List()

    def configure(self):
        """ Configure the editor.
        """

        widget = self.adaptee

        if self.factory.read_only:
            widget.setEditTriggers(QAbstractItemView.NoEditTriggers)
        else:
            self.factory.configure_delegate(widget, self.attribute_type,
                    self.toolkit)

        widget.selectionModel().selectionChanged.connect(
                self._update_selection)

    @IListWidgetEditor.value.getter
    def value(self):
        """ The getter for the editor's value. """

        widget = self.adaptee

        value = []

        for row_nr in range(widget.count()):
            data = widget.item(row_nr).data(Qt.EditRole).toPyObject()

            # The data may be None if there is an open delegate.
            if data is not None:
                value.append(data)

        return value

    @value.setter
    def value(self, value):
        """ The setter for the editor's value. """

        widget = self.adaptee

        old_len = widget.count()
        new_len = len(value)

        # Update any rows that will be kept.
        keep = min(old_len, new_len)

        for row_nr in range(keep):
            itm = widget.item(row_nr)

            if itm.data(Qt.EditRole).toPyObject() != value[row_nr]:
                itm.setData(Qt.EditRole, value[row_nr])

        row_nr = keep

        if old_len < new_len:
            # Add the rows for the new elements.
            while row_nr < new_len:
                self._new_item(value[row_nr])
                row_nr += 1

        elif old_len > new_len:
            # Remove the rows for the old elements.
            while row_nr < old_len:
                widget.takeItem(row_nr)
                row_nr += 1

    @IListWidgetEditor.invalid_reason.getter
    def invalid_reason(self):
        """ The getter for the invalid reason. """

        # Return the reason at the head of the list, if there is one.
        if len(self._invalid_reasons) == 0:
            return ''

        row_nr, reason = self._invalid_reasons[0]

        return "in row {0}, {1}".format(row_nr + 1, reason)

    def add(self, value=None, edit=True):
        """ Add a new item to the collection.

        :param value:
            is the value of the item.  If it is ``None`` then an appropriate
            value will be created automatically.
        :param edit:
            is set if an editor should be open automatically for the new item.
        """

        widget = self.adaptee

        if value is None:
            value = self.factory.create_item(self.attribute_type)

        itm = self._new_item(value)

        if not self.factory.read_only and edit:
            # Start editing the new row.
            widget.setCurrentItem(itm)
            widget.editItem(itm)

        self._editor_changed()

    def item_invalid_reason(self, index, invalid_reason):
        """ Update the invalid reason for an item in the collection.

        :param index:
            is the :class:`~PyQt4.QtCore.QModelIndex` of the item.
        :param index:
            is the invalid reason.  It will be an empty string if the item is
            valid.
        """

        row_nr = index.row()

        # Remove any existing reason.
        for i, (r, reason) in enumerate(self._invalid_reasons):
            if row_nr == r:
                del self._invalid_reasons[i]
                break

        # If the item is invalid then put the reason first so it is the one the
        # user sees.
        if invalid_reason != '':
            self._invalid_reasons.insert(0, (row_nr, invalid_reason))

    def move_down(self, index):
        """ Move an item down.

        :param index:
            is the :class:`~PyQt4.QtCore.QModelIndex` of the item.
        """

        widget = self.adaptee

        row_nr = index.row()

        assert row_nr >= 0 and row_nr + 1 < widget.count()

        # Renumber any invalid reason that refers to this row or the previous
        # one.
        for pos, (r, reason) in enumerate(self._invalid_reasons):
            if r == row_nr:
                self._invalid_reasons[pos] = (r + 1, reason)
            elif r == row_nr + 1:
                self._invalid_reasons[pos] = (r - 1, reason)

        # Move the item in the row.
        itm = widget.takeItem(row_nr)
        widget.insertItem(row_nr + 1, itm)
        widget.setCurrentRow(row_nr + 1)

        self._editor_changed()

    def move_up(self, index):
        """ Move an item up.

        :param index:
            is the :class:`~PyQt4.QtCore.QModelIndex` of the item.
        """

        widget = self.adaptee

        row_nr = index.row()

        assert row_nr > 0 and row_nr < widget.count()

        # Renumber any invalid reason that refers to this row or the previous
        # one.
        for pos, (r, reason) in enumerate(self._invalid_reasons):
            if r == row_nr:
                self._invalid_reasons[pos] = (r - 1, reason)
            elif r == row_nr - 1:
                self._invalid_reasons[pos] = (r + 1, reason)

        # Move the items in the row.
        itm = widget.takeItem(row_nr)
        widget.insertItem(row_nr - 1, itm)
        widget.setCurrentRow(row_nr - 1)

        self._editor_changed()

    def remove(self, indexes):
        """ Remove a number of items from the collection.

        :param indexes:
            is the list of :class:`~PyQt4.QtCore.QModelIndex` defining the
            items to be removed.
        """

        widget = self.adaptee

        # Get the list of rows to remove in ascending order.
        rows = set()
        for index in indexes:
            rows.add(index.row())

        rows = list(rows)
        rows.sort()

        # We don't rely on the widget for the item count because it doesn't get
        # updated early enough.
        self.item_count -= len(rows)

        for pos, row_nr in enumerate(rows):
            # Adjust the row number to account for rows we have already
            # removed.
            row_nr -= pos

            widget.takeItem(row_nr)

            # Remove any reasons related to the row being removed and adjust
            # the row number of any reasons after it.
            reasons = []

            for r, reason in self._invalid_reasons:
                if r != row_nr:
                    if r > row_nr:
                        r -= 1

                    reasons.append((r, reason))

            self._invalid_reasons = reasons

        self._editor_changed()

    def _new_item(self, value):
        """ Add a new item. """

        self.item_count += 1

        itm = QListWidgetItem()

        self.factory.configure_widget_item(itm, self.attribute_type)

        itm.setData(Qt.EditRole, value)

        self.adaptee.addItem(itm)

        return itm

    def _update_selection(self):
        """ Invoked when the selection changes. """

        self.selection = self.adaptee.selectionModel().selectedIndexes()

    def _editor_changed(self):
        """ This will tell the controller that the editor has changed. """

        if self.controller is not None:
            self.controller.editor_changed(self.adaptee)


@adapt(QListWidget, to=IAutomatedListEditor)
class QListWidgetIAutomatedListEditorAdapter(Adapter):
    """ An adapter to implement IAutomatedListEditor for the Qt editor. """

    def simulate_select(self, id, delay, index):
        """ Simulate the user selecting an item.

        :param id:
            is the (possibly scoped) identifier of the widget.
        :param delay:
            is the delay in milliseconds between simulated events.
        :param index:
            is the index of the item.
        """

        self._set_row('select', id, delay, index)

    def simulate_set(self, id, delay, value, index=None):
        """ Simulate the user setting the editor value for an item.

        :param id:
            is the (possibly scoped) identifier of the widget.
        :param delay:
            is the delay in milliseconds between simulated events.
        :param value:
            is the value to set.
        :param index:
            is the index of the item.  If this is ``None`` then it is assumed
            that an editor has already been opened for the item.
        """

        # Open an editor if there isn't already one.
        if index is not None:
            self._set_row('set', id, delay, index)
            QTest.keyClick(self.adaptee, Qt.Key_F2, Qt.NoModifier, delay)

        # We can't assume either that there is an editor available, or that it
        # might soon appear.
        timeout = 50
        editor = self._editor_widget()
        while editor is None:
            if timeout < 0:
                raise AutomationError(id, 'set', "no editor widget found")

            QTest.qWait(5)
            timeout -= 5
            editor = self._editor_widget()

        IAutomatedEditor(editor).simulate_set(id, delay, value)
        QTest.keyClick(editor, Qt.Key_Return, Qt.NoModifier, delay)

        # Give Qt some time to tidy up and update the model.
        QTest.qWait(10)

    def _editor_widget(self):
        """ Return the widget that seems to implement the current editor. """

        # It's a shame that Qt doesn't provide an appropriate call.
        editor = QApplication.focusWidget()
        if editor is not None:
            if not self.adaptee.isAncestorOf(editor):
                editor = None

        return editor

    def _set_row(self, command, id, delay, row):
        """ Set the current row. """

        widget = self.adaptee

        if row < 0 or row >= widget.count():
            raise AutomationError(id, command, "invalid index")

        if delay >= 0:
            QTest.qWait(delay)

        # This is the easiest way to make the row current and visible.
        widget.setCurrentRow(row)
