# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtCore import Qt
from PyQt4.QtGui import (QAbstractItemView, QApplication, QTableWidget,
        QTableWidgetItem)
from PyQt4.QtTest import QTest

from dip.automate import (AutomationError, IAutomatedEditor,
        IAutomatedTableEditor)
from dip.model import adapt, Adapter, List
from dip.ui import BaseEditorAdapter, ITableWidgetEditor, TableColumn


@adapt(QTableWidget, to=ITableWidgetEditor)
class QTableWidgetITableWidgetEditorAdapter(BaseEditorAdapter):
    """ An adapter to implement IEditor for the Qt editor. """

    # The list of reasons why the editor is invalid.  Each element is a 3-tuple
    # of the row, column and reason.
    _invalid_reasons = List()

    # The list of columns that define the table.
    _table_columns = List(TableColumn)

    def configure(self):
        """ Configure the editor.
        """

        widget = self.adaptee

        # Set the number of columns if it hasn't already been set.
        if widget.columnCount() == 0:
            widget.setColumnCount(len(self._table_columns))

            labels = []
            has_labels = False

            for col_nr, col in enumerate(self._table_columns):
                labels.append(col.label)
                if col.label != '':
                    has_labels = True

            if has_labels:
                widget.setHorizontalHeaderLabels(labels)
                widget.horizontalHeader().setStretchLastSection(True)
            else:
                widget.horizontalHeader().hide()

            widget.verticalHeader().hide()

        if self.factory.read_only:
            widget.setEditTriggers(QAbstractItemView.NoEditTriggers)
        else:
            for col_nr, col in enumerate(self._table_columns):
                self.factory.configure_delegate(widget, self.attribute_type,
                        col, col_nr, self.toolkit)

        widget.selectionModel().selectionChanged.connect(
                self._update_selection)

    @ITableWidgetEditor.value.getter
    def value(self):
        """ The getter for the editor's value. """

        widget = self.adaptee

        value = []

        for row_nr in range(widget.rowCount()):
            row = self.factory.create_row(self.attribute_type,
                    self._table_columns)

            for col_nr, col in enumerate(self._table_columns):
                cell = widget.item(row_nr, col_nr).data(Qt.EditRole).toPyObject()

                # The data may be None if there is an open delegate.
                if cell is None:
                    continue

                if col.bind_to != '':
                    setattr(row, col.bind_to, cell)
                else:
                    row[col_nr] = cell

            value.append(row)

        return value

    @value.setter
    def value(self, value):
        """ The setter for the editor's value. """

        widget = self.adaptee

        old_nr_rows = widget.rowCount()
        new_nr_rows = len(value)

        # Update any rows that will be kept.
        keep_rows = min(old_nr_rows, new_nr_rows)

        for row_nr in range(keep_rows):
            row = value[row_nr]

            for col_nr, col in enumerate(self._table_columns):
                if col.bind_to != '':
                    cell = getattr(row, col.bind_to)
                else:
                    cell = row[col_nr]

                widget.item(row_nr, col_nr).setData(Qt.EditRole, cell)

        row_nr = keep_rows

        if old_nr_rows < new_nr_rows:
            widget.setRowCount(new_nr_rows)

            # Add the rows for the new elements.
            while row_nr < new_nr_rows:
                self._new_row(value[row_nr], row_nr)
                row_nr += 1

        elif old_nr_rows > new_nr_rows:
            # Remove the rows for the old elements.
            cols = range(len(self._table_columns))
            while row_nr < old_nr_rows:
                for col_nr in cols:
                    widget.takeItem(row_nr, col_nr)

                row_nr += 1

            widget.setRowCount(new_nr_rows)

    @ITableWidgetEditor.invalid_reason.getter
    def invalid_reason(self):
        """ The getter for the invalid reason. """

        # Return the reason at the head of the list, if there is one.
        if len(self._invalid_reasons) == 0:
            return ''

        row_nr, col_nr, reason = self._invalid_reasons[0]

        col_itm = self.adaptee.horizontalHeaderItem(col_nr)
        if col_itm is None:
            col_name = "column {0}".format(col_nr)
        else:
            col_name = col_itm.text()

        return "in {0} of row {1}, {2}".format(col_name, row_nr + 1, reason)

    def add(self, value=None, edit=True):
        """ Add a new item to the collection.

        :param value:
            is the value of the item.  If it is ``None`` then an appropriate
            value will be created automatically.
        :param edit:
            is set if an editor should be open automatically for the new item.
        """

        widget = self.adaptee

        new_row_nr = widget.rowCount()
        widget.setRowCount(new_row_nr + 1)

        if value is None:
            value = self.factory.create_row(self.attribute_type,
                    self._table_columns)

        itm = self._new_row(value, new_row_nr)

        if not self.factory.read_only and edit:
            # Start editing the new row.
            widget.setCurrentItem(itm)
            widget.editItem(itm)

        self._editor_changed()

    def item_invalid_reason(self, index, invalid_reason):
        """ Update the invalid reason for an item in the collection.

        :param index:
            is the :class:`~PyQt4.QtCore.QModelIndex` of the item.
        :param index:
            is the invalid reason.  It will be an empty string if the item is
            valid.
        """

        row_nr = index.row()
        col_nr = index.column()

        # Remove any existing reason.
        for i, (r, c, reason) in enumerate(self._invalid_reasons):
            if row_nr == r and col_nr == c:
                del self._invalid_reasons[i]
                break

        # If the item is invalid then put the reason first so it is the one the
        # user sees.
        if invalid_reason != '':
            self._invalid_reasons.insert(0, (row_nr, col_nr, invalid_reason))

    def move_down(self, index):
        """ Move an item down.

        :param index:
            is the :class:`~PyQt4.QtCore.QModelIndex` of the item.
        """

        widget = self.adaptee

        row_nr = index.row()

        assert row_nr >= 0 and row_nr + 1 < widget.rowCount()

        # Renumber any invalid reason that refers to this row or the previous
        # one.
        for pos, (r, c, reason) in enumerate(self._invalid_reasons):
            if r == row_nr:
                self._invalid_reasons[pos] = (r + 1, c, reason)
            elif r == row_nr + 1:
                self._invalid_reasons[pos] = (r - 1, c, reason)

        # Move the items in the row.
        saved_items = [widget.takeItem(row_nr, col_nr)
                for col_nr in range(widget.columnCount())]

        widget.removeRow(row_nr)
        widget.insertRow(row_nr + 1)

        for col_nr, itm in enumerate(saved_items):
            widget.setItem(row_nr + 1, col_nr, itm)

        self._editor_changed()

    def move_up(self, index):
        """ Move an item up.

        :param index:
            is the :class:`~PyQt4.QtCore.QModelIndex` of the item.
        """

        widget = self.adaptee

        row_nr = index.row()

        assert row_nr > 0 and row_nr < widget.rowCount()

        # Renumber any invalid reason that refers to this row or the previous
        # one.
        for pos, (r, c, reason) in enumerate(self._invalid_reasons):
            if r == row_nr:
                self._invalid_reasons[pos] = (r - 1, c, reason)
            elif r == row_nr - 1:
                self._invalid_reasons[pos] = (r + 1, c, reason)

        # Move the items in the row.
        saved_items = [widget.takeItem(row_nr, col_nr)
                for col_nr in range(widget.columnCount())]

        widget.removeRow(row_nr)
        widget.insertRow(row_nr - 1)

        for col_nr, itm in enumerate(saved_items):
            widget.setItem(row_nr - 1, col_nr, itm)

        self._editor_changed()

    def remove(self, indexes):
        """ Remove a number of items from the collection.

        :param indexes:
            is the list of :class:`~PyQt4.QtCore.QModelIndex` defining the
            items to be removed.
        """

        widget = self.adaptee

        # Get the list of rows to remove in ascending order.
        rows = set()
        for index in indexes:
            rows.add(index.row())

        rows = list(rows)
        rows.sort()

        # We don't rely on the widget for the item count because it doesn't get
        # updated early enough.
        self.item_count -= len(rows)

        for pos, row_nr in enumerate(rows):
            # Adjust the row number to account for rows we have already
            # removed.
            row_nr -= pos

            widget.removeRow(row_nr)

            # Remove any reasons related to the row being removed and adjust
            # the row number of any reasons after it.
            reasons = []

            for r, c, reason in self._invalid_reasons:
                if r != row_nr:
                    if r > row_nr:
                        r -= 1

                    reasons.append((r, c, reason))

            self._invalid_reasons = reasons

        self._editor_changed()

    def _new_row(self, row, row_nr):
        """ Add a new row at a particular row number. """

        self.item_count += 1

        widget = self.adaptee
        model = widget.model()

        first_itm = None

        for col_nr, col in enumerate(self._table_columns):
            itm = QTableWidgetItem()

            if first_itm is None:
                first_itm = itm

            self.factory.configure_widget_item(itm, self.attribute_type, col)

            if col.bind_to != '':
                cell = getattr(row, col.bind_to)
            else:
                cell = row[col_nr]

            itm.setData(Qt.EditRole, cell)

            # Suppress the dataChanged() signal.
            blocked = model.blockSignals(True)
            widget.setItem(row_nr, col_nr, itm)
            model.blockSignals(blocked)

        return first_itm

    def _update_selection(self):
        """ Invoked when the selection changes. """

        self.selection = self.adaptee.selectionModel().selectedIndexes()

    def _editor_changed(self):
        """ This will tell the controller that the editor has changed. """

        if self.controller is not None:
            self.controller.editor_changed(self.adaptee)

    @_table_columns.default
    def _table_columns(self):
        """ The default list of table columns. """

        return self.factory.table_columns(self.attribute_type,
                self.adaptee.columnCount())


@adapt(QTableWidget, to=IAutomatedTableEditor)
class QTableWidgetIAutomatedTableEditorAdapter(Adapter):
    """ An adapter to implement IAutomatedTableEditor for the Qt editor. """

    def simulate_select(self, id, delay, row, column):
        """ Simulate the user selecting an item.

        :param id:
            is the (possibly scoped) identifier of the widget.
        :param delay:
            is the delay in milliseconds between simulated events.
        :param row:
            is the row of the item.
        :param column:
            is the column of the item.
        """

        self._set_cell('select', id, delay, row, column)

    def simulate_set(self, id, delay, value, row=None, column=None):
        """ Simulate the user setting the editor value for an item.

        :param id:
            is the (possibly scoped) identifier of the widget.
        :param delay:
            is the delay in milliseconds between simulated events.
        :param value:
            is the value to set.  If this is a list or a tuple, and the
            ``column`` parameter is not set, then it is assumed to contain the
            values for the whole row.
        :param row:
            is the row of the item.  If it is ``None`` then it is assumed that
            an editor has already been opened for the item.
        :param column:
            is the column of the item.
        """

        # Open an editor if needed and make sure the value is a sequence.
        if row is None:
            if column is not None:
                raise AutomationError(id, 'set', "column but no row specified")

            if not isinstance(value, (list, tuple)):
                value = [value]
        else:
            if column is None:
                if not isinstance(value, (list, tuple)):
                    raise AutomationError(id, 'set',
                            "no column specified for single value")

                column = 0
            else:
                value = [value]

            self._set_cell('set', id, delay, row, column)
            QTest.keyClick(self.adaptee, Qt.Key_F2, Qt.NoModifier, delay)

        # Set the value, tabbing between columns.
        for v in value[:-1]:
            self._set_value(v, Qt.Key_Tab, delay)

        self._set_value(value[-1], Qt.Key_Return, delay)

        # Give Qt some time to tidy up and update the model.
        QTest.qWait(10)

    def _set_value(self, value, end_key, delay):
        """ Set an individual value. """

        # We can't assume either that there is an editor available, or that it
        # might soon appear.
        timeout = 50
        editor = self._editor_widget()
        while editor is None:
            if timeout < 0:
                raise AutomationError(id, 'set', "no editor widget found")

            QTest.qWait(5)
            timeout -= 5
            editor = self._editor_widget()

        IAutomatedEditor(editor).simulate_set(id, delay, value)
        QTest.keyClick(editor, end_key, Qt.NoModifier, delay)

    def _editor_widget(self):
        """ Return the widget that seems to implement the current editor. """

        # It's a shame that Qt doesn't provide an appropriate call.
        editor = QApplication.focusWidget()
        if editor is not None:
            if not self.adaptee.isAncestorOf(editor):
                editor = None

        return editor

    def _set_cell(self, command, id, delay, row, column):
        """ Set the current cell. """

        widget = self.adaptee

        if row < 0 or row >= widget.rowCount():
            raise AutomationError(id, command, "invalid row")

        if column < 0 or column >= widget.columnCount():
            raise AutomationError(id, command, "invalid column")

        if delay >= 0:
            QTest.qWait(delay)

        # This is the easiest way to make the cell current and visible.
        widget.setCurrentCell(row, column)
