# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtGui import (QAbstractButton, QFileDialog, QHBoxLayout, QLayout,
        QLineEdit, QToolButton, QWidget)

from dip.model import adapt, Instance
from dip.ui import BaseEditorAdapter, IFileSelectorEditor


class FileSelector(QWidget):
    """ The FileSelector class implements a file/directory name editor based on
    a line edit and a browse button.
    """

    def __init__(self, toolkit, factory, parent):
        """ Initialise the editor.

        :param toolkit:
            is the toolkit creating the editor.
        :param factory:
            is the factory creating the editor.
        :param parent:
            is the optional parent widget.
        """

        super(FileSelector, self).__init__(parent)

        layout = QHBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)

        # Set objectName for automation.
        line_edit = QLineEdit(objectName=factory.id)
        self.setFocusProxy(line_edit)
        layout.addWidget(line_edit)

        if not factory.read_only:
            layout.addWidget(toolkit.browse_button())

        self.setLayout(layout)


@adapt(QLayout, QWidget, to=IFileSelectorEditor)
class QLayoutQLineEditIFileSelectorEditorAdapter(BaseEditorAdapter):
    """ An adapter to implement IEditor for a file selector that can be
    implemented as a QLineEdit or a QLayout or QWidget containing a QLineEdit
    and a browse button.
    """

    # The line editor.
    _line_edit = Instance(QLineEdit)

    # The optional browse button.
    _browse_button = Instance(QAbstractButton)

    def configure(self):
        """ Configure the editor. """

        line_edit = self._line_edit

        if self.factory.read_only:
            line_edit.setReadOnly(True)
        else:
            # Configure any validator.
            validator = self.factory.validator
            if validator is not None:
                line_edit.setValidator(validator(line_edit))

            # Connect up any browse button.
            browse_button = self._browse_button
            if browse_button is not None:
                browse_button.clicked.connect(self._on_clicked)

        # Apply any properties to the QLineEdit.
        self.factory.configure_editor(line_edit, self.attribute_type)

    @IFileSelectorEditor.enabled.setter
    def enabled(self, enabled):
        """ The setter for the enabled state. """

        self._line_edit.setEnabled(enabled)

        if self._browse_button is not None:
            self._browse_button.setEnabled(enabled)

    @IFileSelectorEditor.value.getter
    def value(self):
        """ The getter for the editor's value. """

        return self._line_edit.text()

    @value.setter
    def value(self, value):
        """ The setter for the editor's value. """

        self._line_edit.setText(value)

    @IFileSelectorEditor.invalid_reason.getter
    def invalid_reason(self):
        """ The getter for the invalid reason. """

        line_edit = self._line_edit

        if line_edit.hasAcceptableInput():
            invalid_reason = ''
        else:
            # Get a proper reason from the validator if it can provide one.
            validator = line_edit.validator()
            try:
                invalid_reason = validator.dip_invalid_reason
            except AttributeError:
                invalid_reason = "has an invalid format"

        return invalid_reason

    @BaseEditorAdapter.change_signal.getter
    def change_signal(self):
        """ Get the editor's change signal. """

        return self._line_edit.textEdited

    @_line_edit.default
    def _line_edit(self):
        """ Return the line editor. """

        if isinstance(self.adaptee, QLineEdit):
            return self.adaptee

        layout = self.adaptee
        if isinstance(layout, QWidget):
            layout = layout.layout()

        for idx in range(layout.count()):
            w = layout.itemAt(idx).widget()
            if isinstance(w, QLineEdit):
                return w

        raise TypeError("{0} implementing a FileSelector does not contain a "
                "QLineEdit".format(type(self.adaptee).__name__))

    @_browse_button.default
    def _browse_button(self):
        """ Return the browse button. """

        if isinstance(self.adaptee, QLineEdit):
            return None

        layout = self.adaptee
        if isinstance(layout, QWidget):
            layout = layout.layout()

        for idx in range(layout.count()):
            w = layout.itemAt(idx).widget()
            if isinstance(w, QAbstractButton):
                return w

        return None

    def _on_clicked(self):
        """ Invoked when the browse button is clicked. """

        value = self.value
        mode = self.factory.mode
        filter = self.factory.filter
        line_edit = self._line_edit

        if mode == 'open_file':
            value = QFileDialog.getOpenFileName(line_edit, directory=value,
                    filter=filter)
            ok = (value != '')

        elif mode == 'save_file':
            value = QFileDialog.getSaveFileName(line_edit, directory=value,
                    filter=filter)
            ok = (value != '')

        elif mode == 'directory':
            value = QFileDialog.getExistingDirectory(line_edit,
                    directory=value)
            ok = (value != '')

        if ok:
            self.value = value

            # We have to explicitly tell the editor's controller that the
            # value has changed.  There won't be a controller if we are acting
            # as a delegate.
            if self.controller is not None:
                self.controller.editor_changed(self.adaptee)

        line_edit.setFocus()
