# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


import os.path

from PyQt4.QtCore import Qt
from PyQt4.QtGui import (QApplication, QCheckBox, QComboBox, QDialog,
        QDialogButtonBox, QDoubleSpinBox, QFileDialog, QFormLayout, QGroupBox,
        QHBoxLayout, QIcon, QLabel, QLayout, QLineEdit, QListWidget,
        QMessageBox, QMenu, QMenuBar, QPushButton, QSpinBox, QSplitter, QStyle,
        QTableWidget, QTextEdit, QToolButton, QVBoxLayout, QWidget, QWizard,
        QWizardPage)

from dip.model import implements, Instance
from dip.ui import (BaseToolkit, CheckBox, ComboBox, FloatSpinBox, IToolkit,
        LineEdit, ListWidget, PushButton, SpinBox, TableWidget, TextEdit,
        ToolButton)


@implements(IToolkit)
class QtToolkit(BaseToolkit):
    """ The QtToolkit class implements the :term:`toolkit` that uses the Qt
    GUI library.
    """

    # The icon used to add an item to a collection.
    add_icon = Instance(QIcon)

    # The icon used to open a dialog to browse the file system.
    browse_icon = Instance(QIcon)

    # The icon used to move an item down in a collection.
    down_icon = Instance(QIcon)

    # The icon used to remove an item from a collection.
    remove_icon = Instance(QIcon)

    # The icon used to move an item up in a collection.
    up_icon = Instance(QIcon)

    # The mapping of QWidget sub-classes to editor factories.
    _widget_editor_map = (
        (QCheckBox,      CheckBox),
        (QComboBox,      ComboBox),
        (QDoubleSpinBox, FloatSpinBox),
        (QLineEdit,      LineEdit),
        (QListWidget,    ListWidget),
        (QPushButton,    PushButton),
        (QSpinBox,       SpinBox),
        (QTableWidget,   TableWidget),
        (QTextEdit,      TextEdit),
        (QToolButton,    ToolButton)
    )

    def add_button(self, factory, parent):
        """ Create a button to add an item to a collection.

        :param factory:
            is the factory creating the button.
        :param parent:
            is the optional parent widget.
        :return:
            the button which will be a :class:`~PyQt4.QtGui.QAbstractButton`.
        """

        return QToolButton(parent, icon=self.add_icon, toolTip="Add")

    def browse_button(self):
        """ Create a button to initiate browsing the file system.

        :return:
            the button which will be a :class:`~PyQt4.QtGui.QAbstractButton`.
        """

        return QToolButton(icon=self.browse_icon, toolTip="Browse")

    def check_box(self, factory, attribute_type, parent):
        """ Create an editor based on a check box.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor which will be an instance of
            :class:`~PyQt4.QtGui.QCheckBox`.  Any factory properties will be
            applied to the check box.
        """

        return QCheckBox(factory.get_attribute_label(attribute_type), parent)

    def combo_box(self, factory, attribute_type, parent):
        """ Create an editor based on a combo box.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor which will be an instance of
            :class:`~PyQt4.QtGui.QComboBox`.  Any factory properties will be
            applied to the combo box.
        """

        return QComboBox(parent)

    def dialog(self, factory, content, parent):
        """ Create a dialog view.

        :param factory:
            is the factory creating the view.
        :param content:
            is the content of the dialog and is either a
            :class:`~PyQt4.QtGui.QLayout` or a :class:`~PyQt4.QtGui.QWidget`.
        :param parent:
            is the optional parent widget.
        :return:
            the view which will be an instance of
            :class:`~PyQt4.QtGui.QDialog`.
        """

        dialog = QDialog(parent)

        layout = QVBoxLayout()

        if isinstance(content, QLayout):
            layout.addLayout(content)
        else:
            layout.addWidget(content)

        layout.addWidget(QDialogButtonBox(factory.buttons))

        dialog.setLayout(layout)

        return dialog

    def down_button(self, factory, parent):
        """ Create a button to move an item in a collection down.

        :param factory:
            is the factory creating the button.
        :param parent:
            is the optional parent widget.
        :return:
            the button which will be a :class:`~PyQt4.QtGui.QAbstractButton`.
        """

        return QToolButton(parent, icon=self.down_icon, toolTip="Move down")

    def editor_for_widget(self, widget_type):
        """ Return an :class:`~dip.ui.EditorFactory` type object that will
        create an editor factory that creates editors of a particular type.

        :param widget_type:
            is the type of the widget.
        :return:
            the type object.
        """

        for wt, ef in self._widget_editor_map:
            if issubclass(widget_type, wt):
                return ef

        raise TypeError(
                "there is no editor factory that creates {0} instances".format(
                        widget_type.__name__))

    def file_selector(self, factory, attribute_type, parent):
        """ Create an editor that implements a file/directory selector.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor.  Any factory properties will be applied to an internal
            :class:`~PyQt4.QtGui.QLineEdit`.
        """

        from .file_selector import FileSelector

        return FileSelector(self, factory, parent)

    def float_spin_box(self, factory, attribute_type, parent):
        """ Create an editor for floats based on a spin box.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor which will be an instance of
            :class:`~PyQt4.QtGui.QDoubleSpinBox`.  Any factory properties will
            be applied to the spin box.
        """

        return QDoubleSpinBox(parent)

    def form(self, factory, items):
        """ Create a view to contain a number of items arranged as a form.

        :param factory:
            is the optional factory creating the view.
        :param items:
            is a sequence of 2-tuple label and field pairs.  A label may be
            ``None``.  A field may be a :class:`~PyQt4.QtGui.QLayout` or a
            :class:`~PyQt4.QtGui.QWidget`.
        :return:
            the view which will be an instance of
            :class:`~PyQt4.QtGui.QFormLayout`.
        """

        layout = QFormLayout()

        for row, (label, field) in enumerate(items):
            if label is None:
                if isinstance(field, QLayout):
                    layout.setLayout(row, QFormLayout.SpanningRole, field)
                else:
                    layout.setWidget(row, QFormLayout.SpanningRole, field)
            else:
                layout.addRow(label, field)

        return layout

    def get_directory(self, directory='', parent=None):
        """ Get the name of an existing directory from the user.

        :param directory:
            is the name of an optional initial directory.
        :param parent:
            is the optional parent widget.
        :return:
            the name of the directory or an empty string if there was none.
        """

        return QFileDialog.getExistingDirectory(parent, directory=directory)

    def get_open_file(self, directory='', filter='', parent=None):
        """ Get the name of a file to open from the user.

        :param directory:
            is the name of an optional initial directory or file.
        :param filter:
            is the optional file filter.
        :param parent:
            is the optional parent widget.
        :return:
            the name of the file to open or an empty string if there was none.
        """

        return QFileDialog.getOpenFileName(parent, directory=directory,
                filter=filter)

    def get_open_files(self, directory='', filter='', parent=None):
        """ Get a list of names of files to open from the user.

        :param directory:
            is the name of an optional initial directory or file.
        :param filter:
            is the optional file filter.
        :param parent:
            is the optional parent widget.
        :return:
            the list of names.
        """

        return QFileDialog.getOpenFileNames(parent, directory=directory,
                filter=filter)

    def get_save_file(self, directory='', filter='', parent=None):
        """ Get the name of a file to save from the user.

        :param directory:
            is the name of an optional initial directory or file.
        :param filter:
            is the optional file filter.
        :param parent:
            is the optional parent widget.
        :return:
            the name of the file to save or an empty string if there was none.
        """

        return QFileDialog.getSaveFileName(parent, directory=directory,
                filter=filter)

    def group_box(self, factory, content, parent):
        """ Create a group box view.

        :param factory:
            is the factory creating the view.
        :param content:
            is the content of the view and is either a
            :class:`~PyQt4.QtGui.QLayout` or a :class:`~PyQt4.QtGui.QWidget`.
        :param parent:
            is the optional parent widget.
        :return:
            the view.
        """

        group_box = QGroupBox(parent)
        self._add_content(group_box, content)

        return group_box

    def h_box(self, factory, items):
        """ Create a view to contain a number of items arranged horizontally.

        :param factory:
            is the factory creating the view.
        :param items:
            is a sequence of :class:`~PyQt4.QtGui.QLayout`,
            :class:`~PyQt4.QtGui.QWidget` and :class:`~dip.ui.Stretch`
            instances.
        :return:
            the view which will be an instance of
            :class:`~PyQt4.QtGui.QHBoxLayout`.
        """

        layout = QHBoxLayout()

        self._configure_box_layout(layout, items)

        return layout

    def information(self, title, text, parent=None):
        """ Display a informational message to the user.

        :param title:
            is the title, typically used as the title of a dialog.
        :param text:
            is the text of the message.
        :param parent:
            is the optional parent widget.
        """

        QMessageBox.information(parent, title, text)

    def line_edit(self, factory, attribute_type, parent):
        """ Create an editor based on a line editor.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor which will be an instance of
            :class:`~PyQt4.QtGui.QLineEdit`.  Any factory properties will be
            applied to the line editor.
        """

        return QLineEdit(parent)

    def list_widget(self, factory, attribute_type, parent):
        """ Create an editor for a list.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor which will be an instance of
            :class:`~PyQt4.QtGui.QListWidget`.
        """

        return QListWidget(parent)

    def menu(self, title, parent):
        """ Create a menu.

        :param title:
            is the menu's title.
        :param parent:
            is the menu's parent.
        :return:
            the menu which will be an instance of :class:`~PyQt4.QtGui.QMenu`.
        """

        return QMenu(title, parent)

    def menu_bar(self):
        """ Create a menu bar.

        :return:
            the menu bar which will be an instance of
            :class:`~PyQt4.QtGui.QMenuBar`.
        """

        return QMenuBar()

    def message_area(self, factory, parent):
        """ Create a message area.

        :param factory:
            is the factory creating the message area.
        :param parent:
            is the optional parent widget.
        :return:
            the message area which will be an instance of
            :class:`~PyQt4.QtGui.QLabel`.
        """

        return QLabel(parent, styleSheet='color: red', wordWrap=True)

    def push_button(self, factory, attribute_type, parent):
        """ Create an editor based on a push button.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor which will be an instance of
            :class:`~PyQt4.QtGui.QPushButton`.  Any factory properties will be
            applied to the button.
        """

        return QPushButton(factory.get_attribute_label(attribute_type), parent)

    def question(self, title, text, parent=None, default_no=True):
        """ Ask the user a question with a yes or no answer.

        :param title:
            is the title, typically used as the title of a dialog.
        :param text:
            is the text of the question.
        :param parent:
            is the optional parent widget.
        :param default_no:
            is ``True`` if the default answer to the question is "no".
        :return:
            ``True`` if the answer was "yes".
        """

        default_btn = QMessageBox.No if default_no else QMessageBox.Yes
        btn = QMessageBox.question(parent, title, text,
                buttons=QMessageBox.Yes|QMessageBox.No,
                defaultButton=default_btn)

        return btn == QMessageBox.Yes

    def radio_buttons(self, factory, attribute_type, parent):
        """ Create an editor that implements a set of radio buttons.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor.  Any factory properties will be applied to each of the
            buttons.
        """

        from .radio_buttons import RadioButtons

        return RadioButtons(factory, attribute_type, parent)

    def remove_button(self, factory, parent):
        """ Create a button to remove an item from a collection.

        :param factory:
            is the factory creating the editor.
        :param parent:
            is the optional parent widget.
        :return:
            the button which will be a :class:`~PyQt4.QtGui.QAbstractButton`.
        """

        return QToolButton(parent, icon=self.remove_icon, toolTip="Remove")

    def spin_box(self, factory, attribute_type, parent):
        """ Create an editor based on a spin box.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor which will be an instance of
            :class:`~PyQt4.QtGui.QSpinBox`.  Any factory properties will be
            applied to the spin box.
        """

        return QSpinBox(parent)

    def splitter(self, factory, items):
        """ Create a splitter view to contain a number of items.

        :param factory:
            is the factory creating the view.
        :param items:
            is a sequence of :class:`~PyQt4.QtGui.QLayout` or
            :class:`~PyQt4.QtGui.QWidget` instances.
        :return:
            the view which will be an instance of
            :class:`~PyQt4.QtGui.QSplitter`.
        """

        splitter = QSplitter()

        for item in items:
            if isinstance(item, QLayout):
                # The item needs to be a widget.
                widget = QWidget()
                widget.setLayout(item)
                item = widget

            splitter.addWidget(item)

        return splitter

    def table_widget(self, factory, attribute_type, parent):
        """ Create an editor for a table.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor which will be an instance of
            :class:`~PyQt4.QtGui.QTableWidget`.
        """

        return QTableWidget(parent)

    def text_edit(self, factory, attribute_type, parent):
        """ Create an editor based on a text editor.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor which will be an instance of
            :class:`~PyQt4.QtGui.QTextEdit`.  Any factory properties will be
            applied to the text editor.
        """

        return QTextEdit(parent)

    def tool_button(self, factory, attribute_type, parent):
        """ Create an editor based on a tool button.

        :param factory:
            is the factory creating the editor.
        :param attribute_type:
            is the attribute type.
        :param parent:
            is the parent widget.
        :return:
            the editor which will be an instance of
            :class:`~PyQt4.QtGui.QToolButton`.  Any factory properties will be
            applied to the button.
        """

        return QToolButton(parent)

    def up_button(self, factory, parent):
        """ Create a button to move an item in a collection up.

        :param factory:
            is the factory creating the editor.
        :param parent:
            is the optional parent widget.
        :return:
            the button which will be a :class:`~PyQt4.QtGui.QAbstractButton`.
        """

        return QToolButton(parent, icon=self.up_icon, toolTip="Move up")

    def v_box(self, factory, items):
        """ Create a view to contain a number of items arranged vertically.

        :param factory:
            is the factory creating the view.
        :param items:
            is a sequence of :class:`~PyQt4.QtGui.QLayout`,
            :class:`~PyQt4.QtGui.QWidget` and :class:`~dip.ui.Stretch`
            instances.
        :return:
            the view which will be an instance of
            :class:`~PyQt4.QtGui.QVBoxLayout`.
        """

        layout = QVBoxLayout()

        self._configure_box_layout(layout, items)

        return layout

    def warning(self, title, text, parent=None, detail=''):
        """ Display a warning message to the user.

        :param title:
            is the title, typically used as the title of a dialog.
        :param text:
            is the text of the warning.
        :param parent:
            is the optional parent widget.
        :param detail:
            is the optional additional detail.
        """

        if detail != '':
            msg_box = QMessageBox(parent, windowTitle=title, text=text,
                    detailedText=detail, icon=QMessageBox.Warning)
            msg_box.exec_()
        else:
            QMessageBox.warning(parent, title, text)

    def wizard(self, factory, pages, parent):
        """ Create a wizard view.

        :param factory:
            is the factory creating the view.
        :param pages:
            is the list of wizard pages.
        :param parent:
            is the optional parent widget.
        :return:
            the view.
        """

        wizard = QWizard(parent)

        for page in pages:
            wizard.addPage(page)

        return wizard

    def wizard_page(self, factory, content, parent):
        """ Create a wizard page view.

        :param factory:
            is the factory creating the view.
        :param content:
            is the content of the view and is either a
            :class:`~PyQt4.QtGui.QLayout` or a :class:`~PyQt4.QtGui.QWidget`.
        :param parent:
            is the optional parent widget.
        :return:
            the view.
        """

        wizard_page = QWizardPage(parent)
        self._add_content(wizard_page, content)

        return wizard_page

    @add_icon.default
    def add_icon(self):
        """ Create the default add icon. """

        return QIcon(os.path.join(os.path.dirname(__file__), 'add.png'))

    @browse_icon.default
    def browse_icon(self):
        """ Create the default browse icon. """

        return QApplication.style().standardIcon(QStyle.SP_DirIcon)

    @down_icon.default
    def down_icon(self):
        """ Create the default down icon. """

        return QApplication.style().standardIcon(QStyle.SP_ArrowDown)

    @remove_icon.default
    def remove_icon(self):
        """ Create the default remove icon. """

        return QIcon(os.path.join(os.path.dirname(__file__), 'remove.png'))

    @up_icon.default
    def up_icon(self):
        """ Create the default up icon. """

        return QApplication.style().standardIcon(QStyle.SP_ArrowUp)

    @staticmethod
    def _configure_box_layout(layout, items):
        """ Configure a box layout. """

        for item in items:
            if isinstance(item, QLayout):
                layout.addLayout(item)
            elif isinstance(item, QWidget):
                layout.addWidget(item)
            else:
                layout.addStretch(item.stretch)

    @staticmethod
    def _add_content(widget, content):
        """ Add content to a widget wrapping it in a layout if necessary. """

        if isinstance(content, QWidget):
            layout = QVBoxLayout()
            layout.addWidget(content)

            content = layout

        widget.setLayout(content)
