# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtCore import Qt
from PyQt4.QtGui import (QButtonGroup, QGroupBox, QHBoxLayout, QRadioButton,
        QVBoxLayout, QWidget)

from dip.model import adapt, Instance
from dip.ui import BaseEditorAdapter, IRadioButtonsEditor


def RadioButtons(factory, attribute_type, parent):
    """ Create an editor that will implement a set of radio buttons.

    :param factory:
        is the factory creating the editor.
    :param attribute_type:
        is the attribute type.
    :param parent:
        is the optional parent widget.
    :return:
        the editor.
    """

    # Create the layout.
    if factory.orientation == Qt.Horizontal:
        layout = QHBoxLayout()
    else:
        layout = QVBoxLayout()

    # Create the base widget.
    if factory.frame == 'none':
        editor = QWidget(parent)
        layout.setContentsMargins(0, 0, 0, 0)
    else:
        editor = QGroupBox(parent,
                title=factory.get_attribute_label(attribute_type))

        if factory.frame == 'flat':
            editor.setFlat(True)

    # Create the button group.  Set its object name for automation.
    button_group = QButtonGroup(editor, objectName=factory.id)

    # Create the buttons and add them to the layout and the button group.
    for id, member in enumerate(attribute_type.members):
        button = QRadioButton(factory.get_natural_name(member))
        button_group.addButton(button, id)
        layout.addWidget(button)

    editor.setLayout(layout)

    return editor


@adapt(QButtonGroup, QWidget, to=IRadioButtonsEditor)
class QButtonGroupQWidgetIRadioButtonsEditorAdapter(BaseEditorAdapter):
    """ An adapter to implement IEditor for a QButtonGroup or a QWidget that
    is the parent of a QButtonGroup.
    """

    # The button group.
    _button_group = Instance(QButtonGroup)

    def configure(self):
        """ Configure the editor. """

        button_group = self._button_group
        buttons = button_group.buttons()
        nr_buttons = len(buttons)
        nr_members = len(self.attribute_type.members)

        if nr_members != nr_buttons:
            raise TypeError(
                    "Enum has {0} members but the QButtonGroup contains {1} "
                    "buttons".format(nr_members, nr_buttons))

        for id, button in enumerate(buttons):
            button_group.setId(button, id)
            button.setEnabled(not self.factory.read_only)

            self.factory.configure_editor(button, self.attribute_type)

    @IRadioButtonsEditor.label_policy.default
    def label_policy(self):
        """ Return the label policy. """

        return 'embedded' if isinstance(self.adaptee, QGroupBox) else 'default'

    @IRadioButtonsEditor.enabled.setter
    def enabled(self, enabled):
        """ The setter for the enabled state. """

        for button in self._button_group.buttons():
            button.setEnabled(enabled)

    @IRadioButtonsEditor.value.getter
    def value(self):
        """ The getter for the editor's value. """

        return self.attribute_type.members[self._button_group.checkedId()]

    @value.setter
    def value(self, value):
        """ The setter for the editor's value. """

        id = self.attribute_type.members.index(value)
        self._button_group.button(id).setChecked(True)

    @BaseEditorAdapter.change_signal.getter
    def change_signal(self):
        """ Get the editor's change signal. """

        return self._button_group.buttonClicked

    @_button_group.default
    def _button_group(self):
        """ Return the button group. """

        if isinstance(self.adaptee, QButtonGroup):
            return self.adaptee

        button_group = self.adaptee.findChild(QButtonGroup)
        if button_group is None:
            raise TypeError("{0} implementing a RadioButtons does not have a "
                    "QButtonGroup child".format(type(self.adaptee).__name__))

        return button_group
