# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtGui import QWidget

from dip.model import adapt, Adapter, MappingProxy, Model, Subclass

from .i_view import IView
from .qobject_factory import QObjectFactory


class ViewFactory(QObjectFactory):
    """ The ViewFactory class is the base class for all factories that
    create views, sub-views and editors.
    """

    # The specific interface that the view implements.
    view_interface = Subclass(IView, default=IView)

    def __call__(self, model, toolkit, controller=None, parent=None):
        """ Create a view for a model.  This will always be a
        :class:`~PyQt4.QtGui.QWidget`.

        :param model:
            is the model which is either a :class:`~dip.model.Model` instance
            or a Python mapping object.
        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation used to create the
            view.
        :param controller:
            is the optional controller.  If this is ``None`` then
            :meth:`~dip.ui.ViewFactory.create_controller` will be called to
            create one.
        :param parent:
            is the optional parent widget.  It is typically only used for
            dialog based views.
        :return:
            the :class:`~PyQt4.QtGui.QWidget` that is the root of the view's
            implementation.
        """

        # Make sure we have a controller.
        if controller is None:
            controller = self.create_controller()

        # Make sure we have a Model.
        if not isinstance(model, Model):
            model = MappingProxy(model)

        controller.model = model

        # Create the view itself.
        view = self.create_view(controller, toolkit, parent)
        self.finalize(view)

        controller.view = view

        # Update any message areas according to the current state of the model.
        controller.validate()

        # Make sure we return a QWidget.
        if not isinstance(view, QWidget):
            widget = QWidget(parent)
            widget.setLayout(view)
            view = widget

            # Make sure the widget implements IView.
            iview = IView(view)
            iview.controller = controller
            iview.factory = self
            iview.toolkit = toolkit

        return view

    def create_view(self, controller, toolkit, parent):
        """ Create the view.

        This must be reimplemented by a sub-class.

        :param controller:
            is the controller.
        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation used to create the
            view.
        :param parent:
            is the optional parent widget.
        :return:
            the :class:`~PyQt4.QtGui.QLayout` or :class:`~PyQt4.QtGui.QWidget`
            that is the root of the view's implementation.
        """

        raise NotImplementedError

    def create_controller(self):
        """ Create an implementation of :class:`~dip.ui.IController`.

        :return:
            the controller.
        """

        from .controller import Controller

        return Controller()

    def bind(self, view, controller, toolkit):
        """ Bind a view created by this factory.

        :param view:
            is the view.
        :param controller:
            is the controller.
        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation used to create the
            view.
        """

        iview = self.view_interface(view)
        iview.controller = controller
        iview.factory = self
        iview.toolkit = toolkit

        iview.configure()


@adapt(QWidget, to=IView)
class QWidgetIViewAdapter(Adapter):
    """ An adapter to implement IView for the QWidget used to wrap a QLayout.
    """

    def configure(self):
        """ Configure the view.  This is called when everything else has been
        set up.
        """

        # Apply the factory properties to the wrapped layout.
        self.factory.configure_properties(self.adaptee.layout())
