# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from dip.model import isadapted

from .i_composite_view import ICompositeView
from .i_editor import IEditor
from .controller import Controller


class WizardController(Controller):
    """ The WizardController class is a :term:`controller` for handling
    wizards.
    """

    def validate_view(self):
        """ Validate the data in the view.  This implementation considers the
        view to be valid if all its editors have valid values.

        :return:
            a string which will be empty if the view is valid, otherwise it
            will explain why the view is invalid.
        """

        for page_nr, page_id in enumerate(self.view.pageIds()):
            invalid_reason = self.validate_page(page_nr)

            # Allow for the initial validation before the current identifier is
            # set.
            current_id = self.view.currentId()
            if current_id < 0:
                current_id = 0

            if page_id == current_id:
                page = self.view.page(page_id)
                page.dip_is_complete = (invalid_reason == '')
                page.completeChanged.emit()

            if invalid_reason != '':
                break
        else:
            invalid_reason = ''

        return invalid_reason

    def validate_page(self, page_nr):
        """ Validate the data in a page.  This implementation considers the
        page to be valid if all its editors have valid values.

        :param page_nr:
            is the number of the page.  This is the index of the page in the
            wizard and is not the same as the identifier allocated by Qt.
        :return:
            a string which will be empty if the page is valid, otherwise it
            will explain why the page is invalid.
        """

        page_id = self.view.pageIds()[page_nr]
        page = self.view.page(page_id)

        for subview in ICompositeView(page).subviews():
            if isadapted(subview, IEditor):
                ieditor = IEditor(subview)

                # Ignore disabled editors as they may legitimately contain
                # invalid values.
                if ieditor.enabled:
                    invalid_reason = ieditor.invalid_reason
                    if invalid_reason != '':
                        invalid_reason = "{0}: {1}.".format(ieditor.label,
                                invalid_reason)
                        break
        else:
            invalid_reason = ''

        return invalid_reason
