# Copyright (c) 2010 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from dip.model import implements, Model, observe
from dip.shell import IShellObject, IToolFactory

from .i_py_module import IPyModule
from .py_module import PyModule


@implements(IToolFactory)
class PyEditorFactory(Model):
    """ The PyEditorFactory class is the factory for a :term:`tool` that
    allows a Python module to be edited.
    """

    # The name of the tool.
    name = "Python Editor"

    def __call__(self, obj, toolkit):
        """ Create an instance of the tool.

        :param obj:
            is the obj on which the tool is to operate on.
        :param toolkit:
            is the :class:`~dip.ui.IToolkit` implementation that can be used to
            create widgets.
        :return:
            the instance of the tool.  This must be a
            :class:`~PyQt4.QtGui.QWidget` instance and, therefore, can be
            adapted to the :class:`~dip.tools.ITool` interface.
        """

        from .py_editor import PyEditor

        py_editor = PyEditor()

        if isinstance(obj, PyModule):
            obj.editor = py_editor
        else:
            # The object doesn't use the editor to keep the text so we have to
            # do it the more obvious (but slower) way.
            py_editor.setText(obj.text)

            observe('text', obj, lambda c: py_editor.setText(c.new))
            py_editor.textChanged.connect(
                    lambda: setattr(obj, 'text', py_editor.text()))

        # Connect up the editor and the object's dirty flag.
        so = IShellObject(obj)

        observe('dirty', so, lambda c: py_editor.setModified(c.new))
        py_editor.modificationChanged.connect(
                lambda modification: setattr(so, 'dirty', modification))

        return py_editor

    def handles(self, obj):
        """ Check if the tool can handle an object.

        :param obj:
            is the object.
        :return:
            ``True`` if the tool can handle the object.
        """

        return isinstance(obj, IPyModule)
