import glob
import os
import sys

from PyQt4.QtGui import QApplication, QDialog, QWizard

from dip.model import Instance, List, Model, Str
from dip.ui import (CollectionButtons, DirectoryValidator, FileSelector, Form,
        IToolkit, ListWidget, MessageArea, Stretch, VBox, Wizard, WizardPage)


class LineCounter(Model):
    """ The LineCounter class is a combined model and view for counting the
    total number of lines in a set of files.
    """

    # The name of the root directory.
    root_directory = Str(required='yes', validator=DirectoryValidator)

    # The list of glob-like filters to apply to names of files in the directory
    # hierachy.  If the list is empty then all files are processed.
    filters = List(Str(required='stripped'))

    # The toolkit to use.
    toolkit = Instance(IToolkit)

    # The wizard used to get the input from the user.  Note that this is an
    # ordinary class attribute.
    wizard = Wizard(
            WizardPage(
                VBox (
                    Form(FileSelector('root_directory', mode='directory')),
                    Stretch(),
                    MessageArea()
                ),
                title="Directory name",
                subTitle="Enter the name of the directory containing the "
                        "files whose lines are to be counted."
            ),
            WizardPage(
                VBox(
                    CollectionButtons('filters'),
                    ListWidget('filters'),
                    Stretch(),
                    MessageArea()
                ),
                title="File filters",
                subTitle="Enter the glob-like filters to apply to the file "
                        "names."
            ),
            options=QWizard.HaveFinishButtonOnEarlyPages,
            windowTitle="Line counter"
        )

    def populate(self):
        """ Populate the model with input from the user.

        :return:
            ``True`` if the user didn't cancel.
        """

        ui = self.wizard(self, self.toolkit)

        return ui.exec_() == QDialog.Accepted

    def perform(self):
        """ Count the number of lines and display the total. """

        filepaths = []

        for dirpath, _, filenames in os.walk(self.root_directory):
            if len(self.filters) == 0:
                # There are no filters so look at each file.
                for filename in filenames:
                    filepaths.append(os.path.join(dirpath, filename))
            else:
                # Apply the filters.
                for filter in self.filters:
                    filepaths.extend(glob.glob(os.path.join(dirpath, filter)))

        # Count the files in each file.
        line_count = 0

        for filepath in filepaths:
            try:
                with open(filepath) as fh:
                    line_count += len(fh.readlines())
            except UnicodeDecodeError:
                # Assume it is a binary file and ignore it.
                pass
            except EnvironmentError as err:
                self.toolkit.warning("Line counter",
                        "There was an error reading the file <tt>{0}</tt>."
                                .format(filepath),
                        detail=str(err))

        # Tell the user.
        self.toolkit.information("Line counter",
                "There were {0} lines in {1} files.".format(line_count,
                        len(filepaths)))

    @toolkit.default
    def toolkit(self):
        """ Create the default toolkit. """

        from dip.ui.toolkits.qt import QtToolkit

        return QtToolkit()


# Every PyQt GUI application needs a QApplication.
app = QApplication(sys.argv)

# Create the model/view.
line_counter = LineCounter()

# Populate the model with input from the user.
if line_counter.populate():
    # Perform the task, i.e. count the lines.
    line_counter.perform()
