# Copyright (c) 2011 Riverbank Computing Limited.


try:
    import unittest2 as unittest
except ImportError:
    import unittest

from PyQt4.QtCore import QBuffer, QByteArray, QPoint

from dip.io import ICodecsFactory, IStreamingStorage, IStreamingStorageFactory
from dip.io.codecs import XmlDecoder, XmlEncoder
from dip.io.storage import QIODeviceStorage
from dip.model import (Any, Bool, Dict, Enum, Float, implements, Instance,
        Int, List, Model, Set, Str)


@implements(ICodecsFactory)
class TestXmlCodecsFactory(Model):
    """ The test XML codecs factory. """

    id = 'test.codecs'

    def decodes(self, obj):
        return isinstance(obj, Model)

    def decoder(self):
        return XmlDecoder(factory=self)

    def encodes(self, obj):
        return isinstance(obj, Model)

    def encoder(self):
        return XmlEncoder(factory=self)


@implements(IStreamingStorage)
class MemoryStorage(QIODeviceStorage):
    """ The test memory storage. """

    def qiodevice(self, location):
        return QBuffer(memory_storage)

    def explicit_location(self, location=''): pass
    def implicit_location(self, obj): pass


@implements(IStreamingStorageFactory)
class MemoryStorageFactory(Model):
    """ The test memory storage factory. """

    def __call__(self):
        # Make sure the storage is initialised.
        memory_storage.clear()

        return MemoryStorage(factory=self,
                codecs_factory=TestXmlCodecsFactory())

    def valid_location(self, location): pass


class SubModel(Model):
    """ A test model used as an attribute of a main model. """

    value = Int(10)


# The actual storage.
memory_storage = QByteArray()

# The storage factory.
storage_factory = MemoryStorageFactory()


class TestModel(unittest.TestCase):
    """ This tests the XML decoder and encoder. """

    def test_empty_model(self):
        """ Test an empty model. """

        storage = storage_factory()

        class Klass(Model):
            pass

        k_write = Klass()

        storage.write(k_write, location=None)
        k_read = storage.read(Klass(), location=None)

    def test_Any(self):
        """ Test an Any attribute. """

        class Klass(Model):
            attr = Any(5)

        self._test_attr(Klass)

    def test_Any_pickle(self):
        """ Test an Any attribute that will use pickle. """

        class Klass(Model):
            attr = Any(QPoint(10, 20))

        self._test_attr(Klass)

    def test_Bool(self):
        """ Test a Bool attribute. """

        class Klass(Model):
            attr = Bool(True)

        self._test_attr(Klass)

    def test_Dict(self):
        """ Test a Dict attribute. """

        class Klass(Model):
            attr = Dict(Int(), Str())

        storage = storage_factory()

        m_write = Klass(attr={1: 'one', 2: 'two'})

        storage.write(m_write, location=None)
        m_read = storage.read(Klass(), location=None)

        self.assertDictEqual(m_read.attr, m_write.attr)

    def test_Enum(self):
        """ Test an Enum attribute. """

        class Klass(Model):
            attr = Enum('one', 'two', 'three')

        self._test_attr(Klass)

    def test_Float(self):
        """ Test an Int attribute. """

        class Klass(Model):
            attr = Float(10.33)

        self._test_attr(Klass)

    def test_Instance(self):
        """ Test an Instance attribute. """

        class Klass(Model):
            attr = Instance(SubModel)

        storage = storage_factory()

        m_write = Klass(attr=SubModel())

        storage.write(m_write, location=None)
        m_read = storage.read(Klass(), location=None)

        self.assertEqual(m_read.attr.value, m_write.attr.value)

    def test_Int(self):
        """ Test an Int attribute. """

        class Klass(Model):
            attr = Int(10)

        self._test_attr(Klass)

    def test_List(self):
        """ Test a List attribute. """

        class Klass(Model):
            attr = List(Int())

        storage = storage_factory()

        m_write = Klass(attr=[1, 2, 3, 5])

        storage.write(m_write, location=None)
        m_read = storage.read(Klass(), location=None)

        self.assertListEqual(m_read.attr, m_write.attr)

    def test_Set(self):
        """ Test a Set attribute. """

        class Klass(Model):
            attr = Set(Int())

        storage = storage_factory()

        m_write = Klass(attr=set((1, 2, 3, 5)))

        storage.write(m_write, location=None)
        m_read = storage.read(Klass(), location=None)

        self.assertSetEqual(m_read.attr, m_write.attr)

    def test_Str(self):
        """ Test a Str attribute. """

        class Klass(Model):
            # Note the use of characters that need to be escaped.
            attr = Str('<>@"')

        self._test_attr(Klass)

    def _test_attr(self, factory):
        """ Check that the (non-collection) attr attribute of a model can be
        encoded and decoded.
        """

        storage = storage_factory()

        m_write = factory()

        storage.write(m_write, location=None)
        m_read = storage.read(factory(), location=None)

        self.assertEqual(m_read.attr, m_write.attr)


if __name__ == '__main__':
    unittest.main()
