# Copyright (c) 2011 Riverbank Computing Limited.


if __name__ == '__main__':
    import sys
    from PyQt4.QtGui import QApplication

    app = QApplication(sys.argv)


try:
    import unittest2 as unittest
except ImportError:
    import unittest

from PyQt4.QtGui import QWidget
from PyQt4.QtTest import QTest

from dip.automate import Robot
from dip.model import implements, Instance, Model, Str
from dip.shell import IShellAction, IShellManager
from dip.shell.shells.qmainwindow import QMainWindowShell


# The mock shell manager.
@implements(IShellManager)
class MockShellManager(Model):

    # The id of the action that was triggered.
    mock_action_triggered = Str()

    # The mock tool.
    mock_tool = Instance(QWidget)

    def close(self, tool):
        self.mock_action_triggered = 'dip.shell.actions.close'

        return False

    def new(self):
        self.mock_action_triggered = 'dip.shell.actions.new'

        return None

    def open(self, location=''):
        self.mock_action_triggered = 'dip.shell.actions.open'

        return None

    def quit(self):
        self.mock_action_triggered = 'dip.shell.actions.quit'

        return False

    def save(self, tool):
        self.mock_action_triggered = 'dip.shell.actions.save'

    def save_as(self, tool):
        self.mock_action_triggered = 'dip.shell.actions.save_as'

    def mock_reset(self):
        """ Reset the manager. """

        self.mock_action_triggered = ''
        self.active_tool = None

        # Enabled the state-dependent actions.
        action_ids = ('dip.shell.actions.close', 'dip.shell.actions.new',
                'dip.shell.actions.open', 'dip.shell.actions.save',
                'dip.shell.actions.save_as')

        for action in self.shell.actions:
            if isinstance(action, IShellAction) and action.id in action_ids:
                action.visible = action.enabled = True

    @mock_tool.default
    def mock_tool(self):
        return QWidget()

mock_shell_manager = MockShellManager()


# The test shell.
shell = None


class TestShell(unittest.TestCase):
    """ This tests a shell. """

    def setUp(self):
        """ Initialise the test shell, creating it if necessary. """

        # Create it if this is the first call.
        global shell

        if shell is None:
            shell = QMainWindowShell(id='Shell',
                    shell_manager=mock_shell_manager)

        mock_shell_manager.mock_reset()

        shell.widget.show()
        QTest.qWaitForWindowShown(shell.widget)
        shell.widget.activateWindow()

    def tearDown(self):
        """ Tidy up the test widget. """

        shell.widget.hide()

    def test_close(self):
        """ Test the close shell action with no tool. """

        Robot.simulate('Shell', 'trigger', 'dip.shell.actions.close')
        self.assertEqual(mock_shell_manager.mock_action_triggered, '')

    def test_close_tool(self):
        """ Test the close shell action with a tool. """

        mock_shell_manager.active_tool = mock_shell_manager.mock_tool

        Robot.simulate('Shell', 'trigger', 'dip.shell.actions.close')
        self.assertEqual(mock_shell_manager.mock_action_triggered,
                'dip.shell.actions.close')

    def test_new(self):
        """ Test the new shell action. """

        Robot.simulate('Shell', 'trigger', 'dip.shell.actions.new')
        self.assertEqual(mock_shell_manager.mock_action_triggered,
                'dip.shell.actions.new')

    def test_open(self):
        """ Test the open shell action. """

        Robot.simulate('Shell', 'trigger', 'dip.shell.actions.open')
        self.assertEqual(mock_shell_manager.mock_action_triggered,
                'dip.shell.actions.open')

    def test_quit(self):
        """ Test the quit shell action. """

        Robot.simulate('Shell', 'trigger', 'dip.shell.actions.quit')
        self.assertEqual(mock_shell_manager.mock_action_triggered,
                'dip.shell.actions.quit')

    def test_save(self):
        """ Test the save shell action with no tool. """

        Robot.simulate('Shell', 'trigger', 'dip.shell.actions.save')
        self.assertEqual(mock_shell_manager.mock_action_triggered, '')

    def test_save_tool(self):
        """ Test the save shell action with a tool. """

        mock_shell_manager.active_tool = mock_shell_manager.mock_tool

        Robot.simulate('Shell', 'trigger', 'dip.shell.actions.save')
        self.assertEqual(mock_shell_manager.mock_action_triggered,
                'dip.shell.actions.save')

    def test_save_as(self):
        """ Test the save as shell action with no tool. """

        Robot.simulate('Shell', 'trigger', 'dip.shell.actions.save_as')
        self.assertEqual(mock_shell_manager.mock_action_triggered, '')

    def test_save_as_tool(self):
        """ Test the save as shell action with a tool. """

        mock_shell_manager.active_tool = mock_shell_manager.mock_tool

        Robot.simulate('Shell', 'trigger', 'dip.shell.actions.save_as')
        self.assertEqual(mock_shell_manager.mock_action_triggered,
                'dip.shell.actions.save_as')


if __name__ == '__main__':
    unittest.main()
