# Copyright (c) 2011 Riverbank Computing Limited.


if __name__ == '__main__':
    import sys
    from PyQt4.QtGui import QApplication

    app = QApplication(sys.argv)


try:
    import unittest2 as unittest
except ImportError:
    import unittest

import os

from dip.automate import Robot
from dip.model import Bool, Enum, Float, Int, Model, Str, Trigger
from dip.ui import (CheckBox, ComboBox, FileSelector, FloatSpinBox, LineEdit,
        PushButton, RadioButtons, SpinBox, TextEdit, ToolButton)
from dip.ui.toolkits.qt import QtToolkit


class TestEditors(unittest.TestCase):
    """ This tests editors based on a model. """

    def test_CheckBox(self):
        """ Test a CheckBox editor. """

        class TestModel(Model):
            attr = Bool(False)

        model = TestModel()
        view = CheckBox('attr', id='Editors.CheckBox')
        ui = view(model, QtToolkit())
        ui.show()

        Robot.simulate('Editors.CheckBox', 'set', True)
        self.assertTrue(model.attr)

    def test_ComboBox(self):
        """ Test a ComboBox editor. """

        class TestModel(Model):
            attr = Enum('yes', 'no')

        model = TestModel()
        view = ComboBox('attr', id='Editors.ComboBox')
        ui = view(model, QtToolkit())
        ui.show()

        Robot.simulate('Editors.ComboBox', 'set', 'no')
        self.assertEqual(model.attr, 'no')

    def test_DirectorySelector(self):
        """ Test a FileSelector editor selecting a directory. """

        class TestModel(Model):
            attr = Str("/Replace/this")

        model = TestModel()
        view = FileSelector('attr', mode='directory',
                id='Editors.DirectorySelector')
        ui = view(model, QtToolkit())
        ui.show()

        # Get a known valid directory name.
        valid = os.path.dirname(__file__)

        Robot.simulate('Editors.DirectorySelector', 'set', valid)
        self.assertEqual(model.attr, valid)

    def test_FileSelector(self):
        """ Test a FileSelector editor. """

        class TestModel(Model):
            attr = Str("/Replace/this")

        model = TestModel()
        view = FileSelector('attr', id='Editors.FileSelector')
        ui = view(model, QtToolkit())
        ui.show()

        # Get a known valid file name.
        valid = __file__

        Robot.simulate('Editors.FileSelector', 'set', valid)
        self.assertEqual(model.attr, valid)

    def test_FloatSpinBox(self):
        """ Test a FloatSpinBox editor. """

        class TestModel(Model):
            attr = Float(10.0)

        model = TestModel()
        view = FloatSpinBox('attr', id='Editors.FloatSpinBox')
        ui = view(model, QtToolkit())
        ui.show()

        Robot.simulate('Editors.FloatSpinBox', 'set', 20.0)
        self.assertEqual(model.attr, 20.0)

    def test_LineEdit(self):
        """ Test a LineEdit editor. """

        class TestModel(Model):
            attr = Str("Replace this...")

        model = TestModel()
        view = LineEdit('attr', id='Editors.LineEdit')
        ui = view(model, QtToolkit())
        ui.show()

        Robot.simulate('Editors.LineEdit', 'set', "...with this")
        self.assertEqual(model.attr, "...with this")

    def test_PushButton(self):
        """ Test a PushButton editor. """

        class TestModel(Model):
            attr = Trigger()

            @attr.observer
            def attr(self, change):
                self.observed = change

        model = TestModel()
        view = PushButton('attr', id='Editors.PushButton')
        ui = view(model, QtToolkit())
        ui.show()

        Robot.simulate('Editors.PushButton', 'trigger')
        self.assertTrue(model.observed)

    def test_RadioButtons(self):
        """ Test a RadioButtons editor. """

        class TestModel(Model):
            attr = Enum('yes', 'no')

        model = TestModel()
        view = RadioButtons('attr', id='Editors.RadioButtons')
        ui = view(model, QtToolkit())
        ui.show()

        Robot.simulate('Editors.RadioButtons', 'set', 'no')
        self.assertEqual(model.attr, 'no')

    def test_SpinBox(self):
        """ Test a SpinBox editor. """

        class TestModel(Model):
            attr = Int(10)

        model = TestModel()
        view = SpinBox('attr', id='Editors.SpinBox')
        ui = view(model, QtToolkit())
        ui.show()

        Robot.simulate('Editors.SpinBox', 'set', 20)
        self.assertEqual(model.attr, 20)

    def test_TextEdit(self):
        """ Test a TextEdit editor. """

        class TestModel(Model):
            attr = Str("Replace this...")

        model = TestModel()
        view = TextEdit('attr', id='Editors.TextEdit')
        ui = view(model, QtToolkit())
        ui.show()

        Robot.simulate('Editors.TextEdit', 'set', "...with this")
        self.assertEqual(model.attr, "...with this")

    def test_ToolButton(self):
        """ Test a ToolButton editor. """

        class TestModel(Model):
            attr = Trigger()

            @attr.observer
            def attr(self, change):
                self.observed = change

        model = TestModel()
        view = ToolButton('attr', id='Editors.ToolButton')
        ui = view(model, QtToolkit())
        ui.show()

        Robot.simulate('Editors.ToolButton', 'trigger')
        self.assertTrue(model.observed)


if __name__ == '__main__':
    unittest.main()
