# Copyright (c) 2011 Riverbank Computing Limited.


if __name__ == '__main__':
    import sys
    from PyQt4.QtGui import QApplication

    app = QApplication(sys.argv)


try:
    import unittest2 as unittest
except ImportError:
    import unittest

from PyQt4.QtTest import QTest

from dip.automate import AutomationError, Robot
from dip.model import List, Model, Str
from dip.ui import CollectionButtons, TableWidget, VBox
from dip.ui.toolkits.qt import QtToolkit


# The test model.
class TestRow(Model):
    col0 = Str()
    col1 = Str()

class TestModel(Model):
    attr = List(TestRow)

model = None


# The test GUI.
view = VBox(CollectionButtons('attr'),
        TableWidget('attr', columns=['col0', 'col1']), id='TableEditor')

ui = None


class TestTableWidgetEditor(unittest.TestCase):
    """ This tests the table widget editor. """

    def setUp(self):
        """ Initialise the table widget, creating it if necessary. """

        # Create it if this is the first call.
        global model, ui

        if model is None:
            model = TestModel()

        if ui is None:
            ui = view(model, QtToolkit())

        model.attr = []

        ui.show()
        QTest.qWaitForWindowShown(ui)
        ui.activateWindow()

    def tearDown(self):
        """ Tidy up the table widget. """

        ui.hide()

    def test_set(self):
        """ Test set of a single cell. """

        model.attr = [TestRow(col0="Old")]
        Robot.simulate('TableEditor:attr', 'set', "New", 0, 0)
        self.assertEqual(model.attr[0].col0, "New")

    def test_set_row(self):
        """ Test set of a whole row. """

        model.attr = [TestRow(col0="Old 0", col1="Old 1")]
        Robot.simulate('TableEditor:attr', 'set', ["New 0", "New 1"], 0)
        self.assertEqual(model.attr[0].col0, "New 0")
        self.assertEqual(model.attr[0].col1, "New 1")

    def test_set_invalid_row(self):
        """ Test set with an invalid row. """

        with self.assertRaises(AutomationError):
            Robot.simulate('TableEditor:attr', 'set', "Invalid", -1, 0)

    def test_set_invalid_column(self):
        """ Test set with an invalid column. """

        with self.assertRaises(AutomationError):
            Robot.simulate('TableEditor:attr', 'set', "Invalid", 0, -1)

    def test_add(self):
        """ Test adding a single cell using a tool button. """

        Robot.simulate('TableEditor:attr.add', 'trigger')
        Robot.simulate('TableEditor:attr', 'set', "New")
        self.assertEqual(model.attr[0].col0, "New")

    def test_add_row(self):
        """ Test adding a whole row using a tool button. """

        Robot.simulate('TableEditor:attr.add', 'trigger')
        Robot.simulate('TableEditor:attr', 'set', ["New 0", "New 1"])
        self.assertEqual(model.attr[0].col0, "New 0")
        self.assertEqual(model.attr[0].col1, "New 1")

    def test_remove(self):
        """ Test removing using a tool button. """

        model.attr = [TestRow(col0="First"), TestRow(col0="Second")]
        Robot.simulate('TableEditor:attr', 'select', 0, 0)
        Robot.simulate('TableEditor:attr.remove', 'trigger')
        self.assertEqual(model.attr[0].col0, "Second")

    def test_up(self):
        """ Test moving up. """

        model.attr = [TestRow(col0="First"), TestRow(col0="Second"),
                TestRow(col0="Third")]
        Robot.simulate('TableEditor:attr', 'select', 1, 0)
        Robot.simulate('TableEditor:attr.up', 'trigger')
        now = [model.attr[r].col0 for r in range(len(model.attr))]
        self.assertEqual(now, ["Second", "First", "Third"])

    def test_down(self):
        """ Test moving down. """

        model.attr = [TestRow(col0="First"), TestRow(col0="Second"),
                TestRow(col0="Third")]
        Robot.simulate('TableEditor:attr', 'select', 1, 0)
        Robot.simulate('TableEditor:attr.down', 'trigger')
        now = [model.attr[r].col0 for r in range(len(model.attr))]
        self.assertEqual(now, ["First", "Third", "Second"])


if __name__ == '__main__':
    unittest.main()
